package com.gau.utils.net;

import java.util.Map;

import android.content.Context;

import com.gau.utils.net.request.THttpRequest;
import com.gau.utils.net.util.HeartSetting;

/**
 * @author liguoliang
 *
 */
public class HttpAdapter {

	private HttpConnectScheduler mConnectScheduler = null;

	public HttpAdapter(Context context) {
		mConnectScheduler = new HttpConnectScheduler(context);
	}

	/**
	 * 添加网络请求 默认为非长连接,异步联网
	 * 
	 * @param request
	 */
	public void addTask(THttpRequest request) {
		if (request == null) {
			return;
		}
		if (mConnectScheduler != null) {
			mConnectScheduler.addRequest(request);
		}
	}

	/**
	 * 取消网络请求
	 * 
	 * @param request
	 */
	public void cancelTask(THttpRequest request) {
		if (mConnectScheduler != null) {
			mConnectScheduler.cancelRequest(request);
		}
	}

	/**
	 * 取消网络请求
	 * 
	 * @param request
	 */
	public void cancelTask(String url) {
		if (mConnectScheduler != null) {
			mConnectScheduler.cancelRequest(url);
		}
	}
	
	public void cleanup() {
		if (mConnectScheduler != null) {
			mConnectScheduler.cleanup();
		}
	}

	/**
	 * 设置最大网络并发线程数
	 * 
	 * @author huyong
	 * @param maxConnectThreadNum
	 */
	public void setMaxConnectThreadNum(int maxConnectThreadNum) {
		if (mConnectScheduler != null) {
			mConnectScheduler.setMaxConnectThreadNum(maxConnectThreadNum);
		}
	}

	/**
	 * 获取当前最大网络并发线程数
	 * 
	 * @author huyong
	 * @return
	 */
	public int getMaxConnectThreadNum() {
		int threadNum = 1;
		if (mConnectScheduler != null) {
			threadNum = mConnectScheduler.getMaxConnectThreadNum();
		}
		return threadNum;
	}
	
	/**
	 * 添加心跳包地址映射表,如果已经存在,则替换旧值
	 * @param host
	 * @param heartUrl
	 * @return 被替换的心跳包地址值
	 */
	public String putCommonHeartUrl(String host, String heartUrl) {
		if (mConnectScheduler != null) {
			return mConnectScheduler.putCommonHeartUrl(host, heartUrl);
		}
		return null;
	}
	
	/**
	 * 移除心跳包地址键值对
	 * @param host
	 * @return 被删除的心跳包地址值
	 */
	public String removeHeartUrl(String host) {
		if (mConnectScheduler != null) {
			return mConnectScheduler.removeHeartUrl(host);
		}
		return null;
	}
	
	public String getHeartUrl(String host) {
		if (mConnectScheduler != null) {
			return mConnectScheduler.getHeartUrl(host);
		}
		return null;
	}
	
	/**
	 * 获取全部心跳包地址映射表
	 * @return
	 */
	public Map<String, String> getAllCommonHeartUrl() {
		if (mConnectScheduler != null) {
			return mConnectScheduler.getAllCommonHeartUrl();
		}
		return null;
	}
	
	/**
	 * 设置通用的心跳包时间间隔,仅在长连接的时候有效
	 * @param heartTime
	 */
	public void setCommonHeartTime(long heartTime) {
		if (mConnectScheduler != null) {
			mConnectScheduler.setCommonHeartTime(heartTime);
		}
	}
	
	/**
	 * 获取通用的心跳包时间间隔
	 * @return
	 */
	public long getCommonHeartTime() {
		if (mConnectScheduler != null) {
			return mConnectScheduler.getCommonHeartTime();
		}
		return HeartSetting.HEART_TIME_NO_SETTING;
	}
}
