package com.gau.utils.net;

import org.apache.http.HttpResponse;

import com.gau.utils.net.request.THttpRequest;
import com.gau.utils.net.response.IResponse;

/**
 * 连接的观察者
 * 
 * @author guoshengliang
 * @version 1.0
 */
public interface IConnectListener {

	/**
	 * 当网络请求结束时激活
	 * 
	 * @param request
	 *            结束的请求对象
	 * @param response
	 *            反馈的结果句柄
	 */
	public abstract void onFinish(THttpRequest request, IResponse response);

	/**
	 * 当网络请求发生异常时激活
	 * 
	 * @param request
	 *            發生異常的請求對象
	 *            
	 * @param response
	 *            网络回应对象
	 *            
	 * @param reason
	 *            異常的原因Code
	 */
	public abstract void onException(THttpRequest request, HttpResponse response, int reason);
	
	/**
	 * 当网络请求发生异常时激活
	 * 
	 * @param request
	 *            發生異常的請求對象
	 * @param reason
	 *            異常的原因Code
	 */
	public abstract void onException(THttpRequest request, int reason);

	/**
	 * 当网络请求启动时激活
	 * 
	 * @param request
	 *            啟動的網絡請求對象
	 */
	public abstract void onStart(final THttpRequest request);
}
