package com.gau.utils.net;

import java.util.concurrent.TimeUnit;

import com.jiubang.core.thread.AbstractThreadExecutor;
import com.jiubang.core.thread.ThreadPoolManager;

/**
 * 
 * @author yangguanxiang
 *
 */
public class NetThreadExecutorProxy {

	private static NetThreadExecutor sExecutor;
	private final static String POOL_NAME = "net_thread_pool";
	private final static int DEFAULT_CORE_POOL_SIZE = 2;
	private final static int DEFAULT_MAX_POOL_SIZE = 4;
	private final static int KEEP_ALIVE_TIME = 60;

	static {
		sExecutor = new NetThreadExecutor();
	}

	/**
	 * 执行一个异步任务
	 * @param task 需要执行的任务
	 */
	public static void execute(Runnable task) {
		sExecutor.execute(task);
	}

	/**
	 * 执行一个异步任务
	 * @param task 需要执行的任务
	 * @param threadName 线程名称
	 */
	public static void execute(Runnable task, String threadName) {
		sExecutor.execute(task, threadName);
	}

	/**
	 * 执行一个异步任务
	 * @param task 需要执行的任务
	 * @param priority 线程优先级，该值来自于Thread，不是来自于Process；
	 * 如需要设置OS层级的优先级，可以在task.run方法开头调用如Process.setThreadPriority(Process.THREAD_PRIORITY_BACKGROUND);
	 * Android建议使用OS层级设置优先级，效果更显著
	 */
	public static void execute(Runnable task, int priority) {
		sExecutor.execute(task, priority);
	}

	/**
	 * 执行一个异步任务
	 * @param task 需要执行的任务
	 * @param threadName 线程名称
	 * @param priority 线程优先级，该值来自于Thread，不是来自于Process；
	 * 如需要设置OS层级的优先级，可以在task.run方法开头调用如Process.setThreadPriority(Process.THREAD_PRIORITY_BACKGROUND);
	 * Android建议使用OS层级设置优先级，效果更显著
	 */
	public static void execute(Runnable task, String threadName, int priority) {
		sExecutor.execute(task, threadName, priority);
	}

	/**
	 * 取消指定的任务
	 * @param task
	 */
	public static void cancel(final Runnable task) {
		sExecutor.cancel(task);
	}

	/**
	 * 销毁对象
	 */
	public static void destroy() {
		sExecutor.destroy();
	}

	/**
	 * 
	 * @author yangguanxiang
	 *
	 */
	private static class NetThreadExecutor extends AbstractThreadExecutor {

		private NetThreadExecutor() {

		}

		@Override
		protected ThreadPoolManager initThreadPoolManager() {
			ThreadPoolManager manager = ThreadPoolManager.buildInstance(POOL_NAME,
					DEFAULT_CORE_POOL_SIZE, DEFAULT_MAX_POOL_SIZE, KEEP_ALIVE_TIME,
					TimeUnit.SECONDS, false, getTaskExecuteListener());
			manager.allowCoreThreadTimeOut(true);
			return manager;
		}
	}
}
