/**
 * 
 */
package com.gau.utils.net.connector;

import android.content.Context;

import com.gau.utils.net.IConnectListener;
import com.gau.utils.net.request.THttpRequest;

/**
 * @author liguoliang
 * 
 */
public abstract class AbstractConnector implements IConnector {

	/**
	 * 链接器当前链接的request
	 */
	protected THttpRequest mRequest;

	/**
	 * 连接情况的观察者
	 */
	protected IConnectListener mConnectObserver;

	/**
	 * 上下文，通过上下文检测网络相关信息
	 */
	protected Context mContext;
	
	/**
	 * 是否空闲
	 */
	protected boolean mIsIdle = true;

	/**
	 * 构造函数
	 * 
	 * @param httpRequest
	 *            網絡請求數據對象
	 * @param context
	 *            上下文對象
	 * @throws IllegalArgumentException
	 *             如果httpRequest或context為null拋出
	 */
	public AbstractConnector(THttpRequest httpRequest, Context context)
			throws IllegalArgumentException {
		if (httpRequest == null || context == null) {
			throw new IllegalArgumentException("httpRequest==null");
		}
		mRequest = httpRequest;
		this.mContext = context;
		mConnectObserver = httpRequest.getReceiver();
	}

	/**
	 * 构造函数,可以自定义一个外部链接观察者
	 * 
	 * @param httpRequest
	 *            網絡請求數據對象
	 * @param connectReceiver
	 *            链接观察者
	 * @param context
	 *            上下文對象
	 * @throws IllegalArgumentException
	 *             如果httpRequest或context為null拋出
	 */
	public AbstractConnector(THttpRequest httpRequest,
			IConnectListener connectReceiver, Context context)
			throws IllegalArgumentException {
		this(httpRequest, context);
		mConnectObserver = connectReceiver;
	}
	
	public THttpRequest getRequset() {
		return mRequest;
	}
	
	/**
	 * 是否空闲
	 * @return
	 */
	public boolean isIdle() {
		return mIsIdle;
	}
}
