package com.gau.utils.net.connector;

import java.io.IOException;
import java.net.SocketTimeoutException;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.apache.http.Header;
import org.apache.http.HttpHost;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.conn.ConnectTimeoutException;
import org.apache.http.conn.params.ConnRoutePNames;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.params.CoreConnectionPNames;
import org.apache.http.protocol.HTTP;

import android.content.Context;

import com.gau.utils.net.IConnectListener;
import com.gau.utils.net.IHttpException;
import com.gau.utils.net.INetRecord;
import com.gau.utils.net.NetException;
import com.gau.utils.net.NetThreadExecutorProxy;
import com.gau.utils.net.asrFiltier.IAsrFilter;
import com.gau.utils.net.request.THttpRequest;
import com.gau.utils.net.response.IResponse;
import com.gau.utils.net.util.NetLog;
import com.gau.utils.net.util.NetUtil;

/**
 * http链接器，通過加載THttpRequest，然后返回IResponse来实现http网络请求交互
 * 
 * @author guoshengliang
 * @version 1.0
 */
public class HttpConnector extends AbstractConnector implements Runnable {

	/**
	 * 异步链接的线程对象
	 */
	private Thread mConnectThread;

	/**
	 * 切换url地址次数
	 */
	private int mSwitchUriTimes = 0;

	/**
	 * 构造函数
	 * 
	 * @param httpRequest
	 *            網絡請求數據對象
	 * @param context
	 *            上下文對象
	 * @throws IllegalArgumentException
	 *             如果httpRequest或context為null拋出
	 */
	public HttpConnector(THttpRequest httpRequest, Context context)
			throws IllegalArgumentException {
		super(httpRequest, context);
	}

	/**
	 * 构造函数,可以自定义一个外部链接观察者
	 * 
	 * @param httpRequest
	 *            網絡請求數據對象
	 * @param connectReceiver
	 *            链接观察者
	 * @param context
	 *            上下文對象
	 * @throws IllegalArgumentException
	 *             如果httpRequest或context為null拋出
	 */
	public HttpConnector(THttpRequest httpRequest,
			IConnectListener connectReceiver, Context context)
			throws IllegalArgumentException {
		super(httpRequest, connectReceiver, context);
	}

	/**
	 * 异步启动网络链接
	 */
	@Override
	public void connectAsynchronous() {
//		if (mConnectThread == null) {
//			NetLog.info("testBattery, Begin HttpConnector connectAsynchronous",
//					null);
//			mConnectThread = new Thread(this);
//			mConnectThread.setPriority(mRequest.getRequestPriority());
//			mConnectThread.start();
//			NetLog.info("testBattery, End HttpConnector connectAsynchronous",
//					null);
//		}
		// TODO 考虑是否要做错误处理或返回码
		
		if (mConnectThread == null) {
			mConnectThread = new Thread(this);
			NetThreadExecutorProxy.execute(mConnectThread, mRequest.getRequestPriority());
		}
	}

	/**
	 * 关闭当前连接
	 * 
	 * @author huyong
	 */
	@Override
	public void cancelCurrentConnect() {
		// mConnectThread.stop();
	}

	@Override
	public void connect() {
		NetLog.info("testBattery, Begin HttpConnector run", null);
		try {
			// 初始连接的是首选地址
			mRequest.setCurrentUrl(mRequest.getUrl());
			mConnectObserver.onStart(mRequest);
			IResponse response = connectSynchronous(mRequest);
			mConnectObserver.onFinish(mRequest, response);
		} catch (ClientProtocolException e) {
			NetLog.erro("ClientProtocolException", e);
			e.printStackTrace();
			mConnectObserver.onException(mRequest, IHttpException.ClientProtocolException);
		} catch (NetException e) {
			// 服务器异常
			NetLog.erro("IOException", e);
			e.printStackTrace();
			try {
				mConnectObserver.onException(mRequest, e.mResponse, e.mErrorCode);
			} catch (AbstractMethodError error) {
				// 若版本过早的应用没有上述方法，则使用旧的方法
				mConnectObserver.onException(mRequest, e.mErrorCode);
			}
		} catch (IOException e) {
			// 网络异常
			NetLog.erro("IOException", e);
			e.printStackTrace();
			mConnectObserver.onException(mRequest, IHttpException.IOException);
		} catch (IllegalAccessException e) {
			// 获取不到有效接入点
			NetLog.erro("can't find netWork", e);
			e.printStackTrace();
			mConnectObserver.onException(mRequest, IHttpException.IllegalAccessException);
		} catch (OutOfMemoryError e) {
			e.printStackTrace();
			NetLog.erro("unkown exception ", e);
			System.gc();
			mConnectObserver.onException(mRequest, IHttpException.OOMException);
		} catch (Throwable e) {
			// 特殊异常保护
			e.printStackTrace();
			NetLog.erro("unkown exception ", e);
			mConnectObserver.onException(mRequest, IHttpException.ThrowableException);
		}
		NetLog.info("testBattery, Begin HttpConnector run", null);
	}

	@Override
	public void closeConnect() {

	}

	@Override
	public void run() {
		connect();
	}

	/**
	 * 配置http头信息
	 * 
	 * @param httpRequest
	 *            待配置的請求對象
	 */
	private void configHttpHeader(HttpRequestBase httpRequest) {
		List<Header> list = mRequest.getHeader();
		if (list != null) {
			int listSize = list.size();
			for (int i = 0; i < listSize; i++) {
				httpRequest.addHeader(list.get(i));
			}
		}
	}

	/**
	 * 配置链接对象信息
	 * 
	 * @param httpClient
	 *            待配置的连接器
	 * @throws IllegalAccessException
	 *             如果找不到有效APN抛出
	 */
	private void configClient(HttpClient httpClient)
			throws IllegalAccessException {
		int connectType = NetUtil.getNetWorkType(mContext);
		// 代理的设置
		if (NetUtil.NETWORKTYPE_CMWAP == connectType) {
			HttpHost proxy = NetUtil.getProxy(mContext);
			httpClient.getParams().setParameter(ConnRoutePNames.DEFAULT_PROXY,
					proxy);
		} else if (connectType == NetUtil.NETWORKTYPE_NULL) {
			throw new IllegalAccessException();
		}

		httpClient.getParams().setParameter(CoreConnectionPNames.SO_TIMEOUT,
				mRequest.getSocketTimeoutValue()); // 从socket读数据时发生阻塞的超时时间
		httpClient.getParams().setParameter(
				CoreConnectionPNames.CONNECTION_TIMEOUT,
				mRequest.getTimeoutValue()); // 连接的超时时间
	}

	/**
	 * 关闭当前连接器，释放资源
	 * 
	 * @param client
	 */
	private void shutDown(DefaultHttpClient client) {
		if (client != null) {
			NetLog.debug("shutDown client ", null);
			client.getConnectionManager().shutdown();
		}
	}

	/**
	 * 根据当前联网失败的次数选择网络地址
	 * 
	 * @param request
	 * @return
	 */
	private URI selectURI(THttpRequest request) {
		if (request == null) {
			return null;
		}
		if (request.getAllUrl() != null
				&& mSwitchUriTimes < request.getAllUrl().size()) {
			return request.getAllUrl().get(mSwitchUriTimes);
		}
		return null;
	}

	/**
	 * 同步进行网络链接，该操作會阻塞线程 根据当前地址列表连接网络,如果联网失败,会根据THttpRequest的getRetryTime次数来进行重连
	 * 如果同一个达到了联网重连次数仍无法连接,会切换地址继续重连,直到所有地址无效为止
	 * 
	 * @param request
	 *            请求对象
	 * @return 請求反饋的Response句柄
	 * @throws ClientProtocolException
	 * @throws IOException
	 * @throws IllegalAccessException
	 */
	private IResponse connectSynchronous(THttpRequest request)
			throws ClientProtocolException, IOException,
			IllegalAccessException, Exception {
		NetLog.info("StartConnect url= " + request.getUrl(), null);
		NetLog.info(
				"testBattery, Begin HttpConnector connectSynchronous url = "
						+ request.getUrl(), null);
		DefaultHttpClient client = new DefaultHttpClient();

		long nowTime = System.currentTimeMillis();

		// 联网信息记录
		IResponse reponseData = null;
		INetRecord record = request.getNetRecord();
		try {
			URI uri = selectURI(request);
			if (uri == null) {
				// 不用在此处记录,后面catch会记录
				// if(record != null) {
				//
				// }
				throw new NetException(IHttpException.URL_ILLEGAL);
			}
			request.setCurrentUrl(uri);
			HttpHost httphost = new HttpHost(uri.getHost(), uri.getPort());
			HttpResponse response = null;
			configClient(client);
			int protocol = request.getProtocol();
			if (protocol == THttpRequest.PROTOCOL_GET
					|| (protocol == THttpRequest.PROTOCOL_NOT_DEFINE && request.getPostData() == null)) {
				// use get
				HttpGet get = new HttpGet(uri);
				configHttpHeader(get);
				if (record != null) {
					// 连接开始回调
					record.onStartConnect(request, null, null);
				}
				response = client.execute(httphost, get);
				if (record != null) {
					// 连接成功回调
					record.onConnectSuccess(request, null, null);
				}
			} else {
				// use post method
				HttpPost httppost = new HttpPost(uri);
				/*
				 * httppost.addHeader("Content-Type",
				 * "application/x-www-form-urlencoded");
				 */
				configHttpHeader(httppost);
				if (request.getPostData() != null) {
					ByteArrayEntity reqEntity = new ByteArrayEntity(request.getPostData());
					reqEntity.setChunked(false);
					httppost.setEntity(reqEntity);
				} else {
					HashMap<String, String> paramMap = request.getParamMap();
					if (paramMap != null && !paramMap.isEmpty()) {
						ArrayList<NameValuePair> nameValuePairs = new ArrayList<NameValuePair>();
						for (String key : paramMap.keySet()) {
							String value = paramMap.get(key);
							BasicNameValuePair pair = new BasicNameValuePair(key, value);
							nameValuePairs.add(pair);
						}
						UrlEncodedFormEntity formEntity = new UrlEncodedFormEntity(nameValuePairs,
								HTTP.UTF_8);
						formEntity.setChunked(false);
						httppost.setEntity(formEntity);
					}
				}
				if (record != null) {
					// 连接开始回调
					record.onStartConnect(request, null, null);
				}
				response = client.execute(httphost, httppost);
				if (record != null) {
					// 连接成功回调
					record.onConnectSuccess(request, null, null);
				}
			}

			int responseCode = response.getStatusLine().getStatusCode();

			// printlog
			long connectOverTime = System.currentTimeMillis();
			long connectTime = connectOverTime - nowTime;
			NetLog.info("connectTime= " + connectTime, null);
			NetLog.debug("responseCode= " + responseCode, null);
			if (responseCode == HttpStatus.SC_OK) {
				IAsrFilter filter = request.getAsrFilter();
				if (filter != null && filter.isAsrResponse(response)) {
					NetLog.debug("find AsrResponse", null);
					// 使用相同地址重连
					int retryTime = request.getCurRetryTime();
					if (retryTime > 0) {
						request.setCurRetryTime(retryTime - 1);
						reponseData = connectSynchronous(request);
					} else {
						// 联网失败，抛出异常
						// DOTO 抛个特殊的异常
						// throw new IOException();
						++mSwitchUriTimes;
						if (mSwitchUriTimes < request.getAllUrl().size()) {
							// 切换网络地址重连
							reponseData = connectSynchronous(request);
						} else {
							throw new NetException(
									IHttpException.FilterException);
						}
					}
				} else {
					reponseData = request.getOperator().operateHttpResponse(
							request, response);

					if (record != null) {
						// 数据传输完成回调
						record.onTransFinish(request, null, null);
					}

					// printlog detector
					long dataTrafficTime = System.currentTimeMillis()
							- connectOverTime;
					NetLog.info("DataTrafficTime= " + dataTrafficTime, null);
					return reponseData;
				}

				// } else if (responseCode == HttpStatus.SC_MOVED_PERMANENTLY
				// || responseCode == HttpStatus.SC_MOVED_TEMPORARILY) {
				// 重定向
				// String location = response.getFirstHeader("location")
				// .getValue();
				// if (location != null) {
				// THttpRequest newRequest = request.clone();
				// try {
				// newRequest.setUrl(location);
				// } catch (URISyntaxException e) {
				// e.printStackTrace();
				// }
				// return connectSynchronous(newRequest);
				// }
				// } else if (responseCode ==
				// HttpStatus.SC_INTERNAL_SERVER_ERROR){
				// int retryTime = request.getCurRetryTime();
				// if (retryTime > 0) {
				// request.setCurRetryTime(retryTime - 1);
				// connectSynchronous(request);
				// } else {
				// // 抛个特殊的异常
				// // throw new ServerException();
				// throw new NetException(responseCode);
				// }
			} else {
				// 未知原因网络连接异常，则重新连接。
				// 如果Http返回码为503，不要进行重试。Service Unavailable 服务器由于维护或者负载过重未能应答
				if (responseCode == HttpStatus.SC_SERVICE_UNAVAILABLE) {
					//503，把重试次数设置为0
					request.setCurRetryTime(0);
				}
				// 使用相同地址重连
				int retryTime = request.getCurRetryTime();
				if (retryTime > 0) {
					request.setCurRetryTime(retryTime - 1);
					reponseData = connectSynchronous(request);
				} else {
					// 抛个特殊的异常
					// throw new IOException();
					++mSwitchUriTimes;
					if (mSwitchUriTimes < request.getAllUrl().size()) {
						// 切换网络地址重连
						reponseData = connectSynchronous(request);
					} else {
						throw new NetException(response, responseCode);
					}
				}
			}
		} catch (Exception e) {
			// e.printStackTrace();
			// 统一在此处反馈异常码
			if (record != null) {
				record.onException(e, null, null);
			}

			// 使用相同地址重连
			int retryTime = request.getCurRetryTime();
			if (retryTime > 0) {
				request.setCurRetryTime(retryTime - 1);
				reponseData = connectSynchronous(request);
			} else {
				++mSwitchUriTimes;
				if (mSwitchUriTimes < request.getAllUrl().size()) {
					// 切换网络地址重连
					reponseData = connectSynchronous(request);
				} else {
					if (e instanceof SocketTimeoutException) {
						throw new NetException(
								IHttpException.SocketTimeoutException);
					} else if (e instanceof ConnectTimeoutException) {
						throw new NetException(
								IHttpException.ConnectTimeoutException);
					} else {
						throw e;
					}
				}

				// throw new IOException();
			}
		} finally {
			shutDown(client);
			request.setCurrentUrl(null);
		}
		NetLog.info("testBattery, end HttpConnector connectSynchronous url = "
				+ request.getUrl(), null);
		return reponseData;
	}
}
