/**
 * 
 */
package com.gau.utils.net.util;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 心跳包参数设置
 * @author liguoliang
 *
 */
public class HeartSetting {
	/**
	 * 通用心跳包地址列表
	 */
	private Map<String, String> mHeartMap = new ConcurrentHashMap<String, String>();
	
	/**
	 * 通用的心跳包时间间隔设置
	 */
	private long mCommonHeartTime = HEART_TIME_NO_SETTING;
	
	/**
	 * 心跳包时间未设置
	 */
	public final static long HEART_TIME_NO_SETTING = -1;
	
	/**
	 * 默认发送心跳包的时间间隔：10s 注意心跳包时间间隔不能超过服务器的超时时间
	 */
	public static final long DEFAULT_HEART_TIME_INTERVAL = 10 * 1000;
	
	/**
	 * 添加心跳包地址映射表,如果已经存在,则替换旧值
	 * @param host
	 * @param heartUrl
	 * @return 被替换的心跳包地址值
	 */
	public String putHeartUrl(String host, String heartUrl) {
		return mHeartMap.put(host, heartUrl);
	}

	/**
	 * 移除心跳包地址键值对
	 * @param host
	 * @return 被删除的心跳包地址值
	 */
	public String removeHeartUrl(String host) {
		return mHeartMap.remove(host);
	}
	
	public String getHeartUrl(String host) {
		return mHeartMap.get(host);
	}
	
	public Map<String, String> getAllHeartUrl() {
		return mHeartMap;
	}
	
	public void setCommonHeartTime(long heartTime) {
		mCommonHeartTime = heartTime;
	}
	
	public long getCommonHeartTime() {
		return mCommonHeartTime;
	}
}
