package com.gau.utils.net.util;

import org.apache.http.HttpHost;
import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Proxy;

/**
 * 网络工具类
 * 
 * @author guoshengliang 
 * @ ver 1.0
 */
public class NetUtil {

	public static final int NETWORKTYPE_WIFI = 1;

	public static final int NETWORKTYPE_CMWAP = 2;

	public static final int NETWORKTYPE_CMNET = 3;

	public static final int NETWORKTYPE_NULL = -1;

	public static boolean isPerferNetWorkCanUse(Context context) {
		ConnectivityManager connectivity = ( ConnectivityManager ) context
				.getSystemService(Context.CONNECTIVITY_SERVICE);
		return isPerferNetWorkCanUse(connectivity);
	}

	private static boolean isPerferNetWorkCanUse(
			ConnectivityManager connectivity) {
		NetworkInfo info = connectivity.getActiveNetworkInfo();
		if (info == null
				|| (info.getState() != NetworkInfo.State.CONNECTING && info
						.getState() != NetworkInfo.State.CONNECTED)) {
			return false;
		}
		return true;
	}

	public static boolean isDefineProxy(Context context) {
		boolean isDefineProxy = false;
		ConnectivityManager connectivity = ( ConnectivityManager ) context
				.getSystemService(Context.CONNECTIVITY_SERVICE);
		boolean isCanUse = isPerferNetWorkCanUse(connectivity);
		if (isCanUse) {
			if (Proxy.getDefaultHost() != null
					|| Proxy.getHost(context) != null) {
				isDefineProxy = true;
			}
		}
		return isDefineProxy;
	}

	public static boolean isCanUse(NetworkInfo network) {
		if (network == null
				|| (network.getState() != NetworkInfo.State.CONNECTING && network
						.getState() != NetworkInfo.State.CONNECTED)) {
			return false;
		}
		return true;
	}

	public static HttpHost getProxy(Context context) {
		String proxy = Proxy.getHost(context);
		if (proxy == null) {
			proxy = Proxy.getDefaultHost();
		}
		int port = Proxy.getPort(context);
		HttpHost host = new HttpHost(proxy, port);
		return host;
	}

	public static int getNetWorkType(Context context) {
		int netWorkType = NETWORKTYPE_NULL;
		ConnectivityManager connectivity = ( ConnectivityManager ) context
				.getSystemService(Context.CONNECTIVITY_SERVICE);
		NetworkInfo info = connectivity.getActiveNetworkInfo();
		if (info == null
				|| (info.getState() != NetworkInfo.State.CONNECTING && info
						.getState() != NetworkInfo.State.CONNECTED)) {
			// 不能获取到有效网络
			netWorkType = NETWORKTYPE_NULL;
		} else {
			if (info.getType() == ConnectivityManager.TYPE_WIFI) {
				netWorkType = NETWORKTYPE_WIFI;
			} else if (info.getType() == ConnectivityManager.TYPE_MOBILE) {
				if (Proxy.getDefaultHost() != null
						|| Proxy.getHost(context) != null) {
					netWorkType = NETWORKTYPE_CMWAP;
				} else {
					netWorkType = NETWORKTYPE_CMNET;
				}
			}
		}
		return netWorkType;
	}

}
