package com.go.launcher.taskmanager;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import com.go.launcher.util.ProcessHelperUtil;

import android.annotation.SuppressLint;
import android.app.ActivityManager;
import android.app.ActivityManager.MemoryInfo;
import android.app.ActivityManager.RunningAppProcessInfo;
import android.content.Context;
import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.os.Build;
import android.os.SystemClock;
import android.util.Log;

public abstract class AbstractTaskManager implements ITaskManager {
	protected static class _PROCESSINFORINTERNAL {
		public int _pid;
		public int _ppid;
		public String _name;
		public long _ramused;
	}

	protected static class _APPINFORINTERNAL {
		public String _name;
		public String _procname;
		public Drawable _icon;
		public int _pid;
		public int _memory;
	}

	private boolean _init = false;
	private ActivityManager _am;
	private MemoryInfo _memInfo = null;
	private volatile long _totalMemroy = 0;
	protected HashMap<String, Intent> _appInforMap = null;
	protected LockFreeList<_APPINFORINTERNAL> _appList = null;
	protected ArrayList<_APPINFOR> _appInforList = new ArrayList<_APPINFOR>();
	private String _appName = "com.jiubang.ggheart";

	public AbstractTaskManager(Context context){
		init(context);
	}
	private void init(Context _ctx) {
		if (_init) {
			return;
		}

		_appList = new LockFreeList<_APPINFORINTERNAL>();
		_am = (ActivityManager) _ctx.getSystemService(Context.ACTIVITY_SERVICE);
		_memInfo = new MemoryInfo();
		_appInforMap = new HashMap<String, Intent>();
		_appName = _ctx.getPackageName();

		getTotalMemory();
		_init = true;
	}

	/**
	 * 获取当前运行的程序列表
	 * @return 程序列表
	 */
	public synchronized ArrayList<_APPINFOR> retriveAppList(Context context) {
		loadAppTable();
		updateRunningAppList(context);

		_appInforList.clear();
		for (_APPINFORINTERNAL it : _appList) {
			_APPINFOR _app = new _APPINFOR();
			if (_appInforMap.containsKey(it._procname)) {
				_app._infor = _appInforMap.get(it._procname);
			}
			_app._pid = it._pid;
			_appInforList.add(_app);
		}
		return _appInforList;
	}

	/**
	 * 终止程序
	 * @param pid 程序的pid
	 */
	public void terminateAppByPid(int pid) {
		if (_init) {
			List<RunningAppProcessInfo> listApps = _am.getRunningAppProcesses();
			for (RunningAppProcessInfo ti : listApps) {
				if (ignoreTask(ti.processName)) {
					continue;
				}

				if (ti.pid == pid) {
					String[] pkList = ti.pkgList;
					this._am.restartPackage(pkList[0]);
				}
			}
		}
	}

	/**
	 * 终止所有程序
	 */
	public void terminateAll() {
		if (_init) {
			List<RunningAppProcessInfo> listApps = this._am.getRunningAppProcesses();
			for (RunningAppProcessInfo ti : listApps) {
				if (ignoreTask(ti.processName)) {
					continue;
				}

				String[] pkList = ti.pkgList;
				this._am.restartPackage(pkList[0]);
			}
		}
	}

	/**
	 * 获取手机总内存
	 * @return 手机总内存(KB)
	 */
	public long retriveTotalMemory() {
		int retry = 0;
		int maxRetry = 4;
		while (_totalMemroy == 0 && retry < maxRetry) {
			if (mTotalMemThread == null) {
				doGetTotalMemory();
			}
			SystemClock.sleep(500);
			retry++;
		}
		return this._totalMemroy;
	}

	/**
	 * 获取手机当前可用内存
	 * @return 可用内存(KB)
	 */
	public long retriveAvailableMemory() {
		_am.getMemoryInfo(this._memInfo);
		return this._memInfo.availMem >> 10; // div 1024
	}

	private Thread mTotalMemThread;
	
	@SuppressLint("NewApi")
	private void getTotalMemory() {
		if (Build.VERSION.SDK_INT >= 16) {
			try {
				_am.getMemoryInfo(this._memInfo);
				_totalMemroy = this._memInfo.totalMem >> 10;
			} catch (Throwable t) {

			}
		}
		if (_totalMemroy == 0) {
			mTotalMemThread = new Thread("getmeminfo") {
				@Override
				public void run() {
					doGetTotalMemory();
					mTotalMemThread = null;
				}
			};
			mTotalMemThread.start();
		}
	}
	
	private synchronized void doGetTotalMemory() {
		FileReader fileReader = null;
		BufferedReader bReader = null;
		try {
			String cat = null;
			fileReader = new FileReader("/proc/meminfo");
			bReader = new BufferedReader(fileReader, 4096);
			cat = bReader.readLine(); // 读取第一行“MemTotal: 415268 kB”
			if (cat != null) {
				String[] array = cat.split("\\s+");
				if (array != null && array.length >= 1) {
					// 数组第2个为内存大小
					_totalMemroy = Long.parseLong(array[1]);
				}
			}
		} catch (NumberFormatException e) {
			Log.e("taskManager", "getTotalMemory error");
		} catch (FileNotFoundException e1) {
			Log.e("taskManager", "getTotalMemory error");
		} catch (IOException e) {
			Log.e("taskManager", "getTotalMemory error");
		} finally {
			if (bReader != null) {
				try {
					bReader.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

    private void updateRunningAppList(Context context) {
        _appList.clear();
        try {

            if (Build.VERSION.SDK_INT >= 21) {
                //获取5.0以及以上的正在运行应用
                List<ProcessHelperBean> list = ProcessHelperUtil
                        .getProcBeans(context);
                HashMap<String, Integer> procNameMap = new HashMap<String, Integer>();

                for (int i = 0; i < list.size(); i++) {
                    ProcessHelperBean processHelperBean = list.get(i);
                    if (processHelperBean == null
                            || processHelperBean.getProcessName() == null) {
                        continue;
                    }

                    if (ignoreTask(processHelperBean.getProcessName())) {
                        continue;
                    }

                    String[] pkList = processHelperBean.getPkgLists();
                    if (null == pkList
                            || !processHelperBean.getProcessName().equals(
                                    pkList[0])
                            || !this._appInforMap.containsKey(processHelperBean
                                    .getProcessName())
                            || procNameMap.containsKey(processHelperBean
                                    .getProcessName())) {
                        continue;
                    }

                    if (ProcessHelperUtil.isAppStop(context, pkList[0])) {
                        continue;
                    }

                    _APPINFORINTERNAL _appInternal = new _APPINFORINTERNAL();
                    _appInternal._pid = processHelperBean.getPid();
                    _appInternal._procname = processHelperBean.getProcessName();
                    _appList.add(_appInternal);

                    procNameMap.put(_appInternal._procname, _appInternal._pid);

                }

            } else {
                //5.0以下获取正在运行应用
                List<RunningAppProcessInfo> listApps = _am
                        .getRunningAppProcesses();
                for (RunningAppProcessInfo ti : listApps) {
                    if (ti == null || ti.processName == null) {
                        continue;
                    }

                    if (ignoreTask(ti.processName)) {
                        continue;
                    }

                    String[] pkList = ti.pkgList;
                    if (!ti.processName.equals(pkList[0])
                            || !this._appInforMap.containsKey(ti.processName)) {
                        continue;
                    }

                    _APPINFORINTERNAL _appInternal = new _APPINFORINTERNAL();
                    _appInternal._pid = ti.pid;
                    _appInternal._procname = ti.processName;
                    _appList.add(_appInternal);
                }
            }

        } catch (OutOfMemoryError e) {
            System.gc();
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }

    }

	/**
	 * 获得手机中已经安装的程序信息
	 */
	protected abstract void loadAppTable();

	/**
	 * 清理任务管理器，释放资源 
	 */
	@Override
	public synchronized void clearTaskManager() {
		_appInforMap.clear();
		_appInforList.clear();
		System.gc();
	}

	private boolean ignoreTask(String processName) {
		return processName.startsWith("android")
				|| processName.startsWith("com.android.inputmethod")
				|| processName.equalsIgnoreCase("system")
				|| processName.equalsIgnoreCase("com.android.alarmclock")
				|| processName.equalsIgnoreCase("com.android.phone")
				|| processName.equalsIgnoreCase("com.google.process.gapps")
				|| processName.equalsIgnoreCase(_appName)
				|| processName.equalsIgnoreCase("com.htc.android.mail")
				|| processName.equalsIgnoreCase("com.motorola.android.vvm")
				|| processName.equalsIgnoreCase("android.process.acore");
	}
}
