package com.go.util.lib;

import android.graphics.Paint;
import android.os.Build;
import android.view.View;
import android.view.ViewGroup;

public class ViewCompatProxy {
	public static boolean IS_HONEYCOMB = Build.VERSION.SDK_INT >= 11;
	public static boolean IS_HONEYCOMB_MR1 = Build.VERSION.SDK_INT >= 12;
	public static boolean IS_ICS = Build.VERSION.SDK_INT >= 14;
	
	public static final int LAYER_TYPE_SOFTWARE = View.LAYER_TYPE_SOFTWARE;
	public static final int LAYER_TYPE_HARDWARE = View.LAYER_TYPE_HARDWARE;
	public static final int LAYER_TYPE_NONE = View.LAYER_TYPE_NONE;

	public static int getLayerTypeSoftware() {
		return IS_HONEYCOMB ? LAYER_TYPE_SOFTWARE : 1;
	}
	
	public static int getLayerTypeHardware() {
		return IS_HONEYCOMB ? LAYER_TYPE_HARDWARE : 1;
	}
	
	
	public static int getLayerTypeNone() {
		return IS_HONEYCOMB ? LAYER_TYPE_NONE : 1;
	}
	
	
	public static boolean isHardwareAccelerated(View view) {
		return IS_HONEYCOMB && view.isHardwareAccelerated();
	}
	
	public static void setLayerType(View view, int layerType, Paint paint) {
		if (IS_HONEYCOMB) {
			view.setLayerType(layerType, paint);
		}
	}
	
	public static void buildLayer(View view) {
		if (IS_HONEYCOMB_MR1) {
			view.buildLayer(); // API 12+
		}
	}
	
	public static void buildChildrenLayer(ViewGroup viewGroup) {
		if (IS_HONEYCOMB_MR1) {
			int childCount = viewGroup.getChildCount();
			for (int i = 0; i < childCount; i++) {
				viewGroup.getChildAt(i).buildLayer(); // API 12+
			}
		}
	}
	
	public static void setSystemUiVisibility(View view, int visibility) {
		if (IS_HONEYCOMB) {
			view.setSystemUiVisibility(visibility);
		}
	}
}
