package com.jb.ga0.commerce.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import android.annotation.TargetApi;
import android.app.Activity;
import android.app.ActivityManager;
import android.app.ActivityManager.RunningAppProcessInfo;
import android.app.ActivityManager.RunningServiceInfo;
import android.app.ActivityManager.RunningTaskInfo;
import android.content.ActivityNotFoundException;
import android.content.ComponentName;
import android.content.Context;
import android.content.ContextWrapper;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.content.pm.PermissionInfo;
import android.content.pm.ProviderInfo;
import android.content.pm.ResolveInfo;
import android.content.pm.ServiceInfo;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.os.Process;
import android.text.TextUtils;

/**
 * 
 * <br>类描述:应用程序工具类
 * <br>功能详细描述:
 * app common utils
 */
public class AppUtils {
	private static final String TAG = "matt";
	/**用于启动程序的信息界面*/
	public static final String ACTION_SETTINGS = "android.settings.APPLICATION_DETAILS_SETTINGS";
	
	/**
	 * 判断应用是否存在(安装)
	 * @param context
	 * @param packageName 应用包名
	 * @return
	 */
    public static boolean isAppExist(final Context context, final String packageName) {
        if (context == null || TextUtils.isEmpty(packageName)) {
            return false;
        }
        boolean result = false;
        try {
            context.getPackageManager().getPackageInfo(packageName, PackageManager.GET_SHARED_LIBRARY_FILES);
            result = true;
        } catch (NameNotFoundException e) {
            result = false;
        } catch (Exception e) {
            result = false;
        }
        return result;
    }
    
    /**
     * 获取应用版本号
     * @param context
     * @return
     */
    public static int getAppVersionCode(final Context context) {
    	return getAppVersionCode(context, context.getPackageName());
    }
    
    /**
     * 获取应用版本号
     * @param context
     * @param packageName 应用包名
     * @return
     */
    public static int getAppVersionCode(final Context context, final String packageName) {
        int versionCode = -1;
        if (context == null || packageName == null) {
            return versionCode;
        }
        try {
            PackageInfo packageInfo = context.getPackageManager().getPackageInfo(packageName, PackageManager.GET_SHARED_LIBRARY_FILES);
            versionCode = packageInfo.versionCode;
        } catch (NameNotFoundException e) {
        	LogUtils.e(TAG, "Error :" + packageName +  " is not exist.");
        } catch (Exception e) {
            
        }
        return versionCode;
    }
    
    /**
	 * 获取应用版本名称
	 * @param context
	 * @return
	 */
	public static String getAppVersionName(Context context) {
		if (null == context) {
			return "";
		}
		return getAppVersionName(context, context.getPackageName());   
	}  
    
    /**
     * 获取应用版本名称
     * @param context
     * @param packageName 应用包名
     * @return
     */
    public static String getAppVersionName(final Context context, final String packageName) {
        String versionName = "";
        if (context == null || packageName == null) {
            return versionName;
        }
        try {
            PackageInfo packageInfo = context.getPackageManager().getPackageInfo(packageName, PackageManager.GET_SHARED_LIBRARY_FILES);
            versionName = packageInfo.versionName;
        } catch (NameNotFoundException e) {
        	LogUtils.e(TAG, "Error :" + packageName +  " is not exist.");
        } catch (Exception e) {
            
        }
        return versionName;
    }
    
    /**
     * 获取程序名称
     * @param context
     * @param packageName 应用包名
     * @return
     */
    public static String getAppLabel(final Context context, final String packageName) {
        if (packageName == null) {
            return null;
        }
        String label = null;
        try {
            PackageManager pkManager = context.getPackageManager();
            ApplicationInfo info = pkManager.getApplicationInfo(packageName, 0);
            if (info != null) {
                label = pkManager.getApplicationLabel(info).toString();
            }
        } catch (Exception e) {
           //e.printStackTrace();
        }
        return label;
    }
    
    /**
     * 根据Intent获取包名
     * 
     * @param intent
     * @return
     */
    public static String getPackageName(Intent intent) {
        if (intent != null) {
            final ComponentName cn = intent.getComponent();
            if (cn != null) {
                return cn.getPackageName();
            }
        }
        return null;
    }
    
   /**
    * 获取手机所有应用程序列表(包括系统程序)
    * @param context
    * @return
    */
    public static List<ResolveInfo> getLauncherApps(Context context) {
        List<ResolveInfo> infos = null;
        PackageManager packageMgr = context.getPackageManager();
        Intent intent = new Intent(Intent.ACTION_MAIN);
        intent.addCategory(Intent.CATEGORY_LAUNCHER);
        infos = packageMgr.queryIntentActivities(intent, 0);
        return infos;
    }
    
    /**
     * 获取用户安装的所有可启动的应用列表
     */
    public static List<PackageInfo> getLauncherableApps(Context context) {
        PackageManager pm = context.getPackageManager();
        final String ourPackageName =  context.getPackageName();
        List<PackageInfo> packages = pm.getInstalledPackages(0);
        List<PackageInfo> apps = new ArrayList<PackageInfo>();
		for (PackageInfo info : packages) {
			Intent intent = pm.getLaunchIntentForPackage(info.applicationInfo.packageName);
			if (intent != null && !ourPackageName.equals(info.packageName)) {
				apps.add(info);
			}
		}
        return apps;
    }
    
   /**
    * 根据进程名称获取对应pid
    * @param context
    * @param processName 进程名称
    * @return
    */
    public static int getPidByProcessName(Context context, String processName) {
        int pid = 0;
        ActivityManager am = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
        List<ActivityManager.RunningAppProcessInfo> appProcessList = am.getRunningAppProcesses();
        for (ActivityManager.RunningAppProcessInfo runningAppProcessInfo : appProcessList) {
            if (runningAppProcessInfo.processName.equals(processName)) {
                pid = runningAppProcessInfo.pid;
                break;
            }
        }
        return pid;
    }

    /**
     * 杀死当前进程
     */
    public static void killProcess() {
        killProcess(Process.myPid());
    }
    
    /**
     * 杀死进程
     */
    public static void killProcess(int pid) {
        new Exception().printStackTrace();
        Process.killProcess(pid);
    }
    
    /**
     *  获取本应用的顶部Activity类名
     *  
     * @param context
     * @return
     */
    public static String getSelfTopActivityName(Context context) {
      try {
          ActivityManager activityManager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
          List<RunningTaskInfo> tasksInfo = activityManager.getRunningTasks(1);
          if (tasksInfo.size() > 0) {
              // Activity位于堆栈的顶层,如果Activity的类为空则判断的是当前应用是否在前台
              return tasksInfo.get(0).topActivity.getClassName();
          }
      } catch (Exception e) {
      }
      return null;
  }
    
    /**
     * 获取默认运行桌面包名(注:存在多个桌面时且未指定默认桌面时,该方法返回Null,使用时需处理这个情况)
     * @param context
     * @return
     */
    public static String getDefaultLauncher(Context context) {
        final Intent intent = new Intent(Intent.ACTION_MAIN);
        intent.addCategory(Intent.CATEGORY_HOME);
        final ResolveInfo res = context.getPackageManager().resolveActivity(intent, PackageManager.MATCH_DEFAULT_ONLY);
        String defaultLauncherPkg = null;
        if (res.activityInfo == null) {
            // should not happen. A home is always installed, isn't it?
            defaultLauncherPkg = null;
        } else if (res.activityInfo.packageName.equals("android")) {
            // 有多个桌面程序存在，且未指定默认项时；     
            defaultLauncherPkg = null;
        } else {
            defaultLauncherPkg = res.activityInfo.packageName;
        }
        return defaultLauncherPkg;
	}
    
    /**
	 * 采用了新的办法获取APK图标,之前的失败是因为android中存在的一个BUG,通过appInfo.publicSourceDir = apkPath;来修正这个问题.
	 * 详情参见:http://code.google.com/p/android/issues/detail?id=9151
	 * @param context
	 * @param apkPath apk文件路径
	 * @return
	 */
	public static Drawable getApkIcon(Context context, String apkPath) {
		PackageManager pm = context.getPackageManager();
		PackageInfo info = pm.getPackageArchiveInfo(apkPath, PackageManager.GET_ACTIVITIES);
		if (info != null) {
			ApplicationInfo appInfo = info.applicationInfo;
			appInfo.sourceDir = apkPath;
			appInfo.publicSourceDir = apkPath;
			try {
				return appInfo.loadIcon(pm);
			} catch (OutOfMemoryError e) {
				//e.printStackTrace();
			}
		}
		return null;
	}
	
	/**
     * Android 安装应用
     * 
     * @param context Application Context
     * @param filePath APK文件路径
     */
	public static void installApk(Context context, File file) {
        if (file.exists()) {
            Intent i = new Intent(Intent.ACTION_VIEW);
            i.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            i.setDataAndType(Uri.fromFile(file), "application/vnd.android.package-archive");
            ((ContextWrapper) context).startActivity(i);
        } else {
  //          ToastUtils.makeEventToast(context, /*context.getString(R.string.appcenter_install_fail_file_not_exist)*/ "Can not find APK!", false);
        }
    }
    
	/**
     * 卸载应用
     * 
     * @param context
     *            应用上下文
     * @param pkgName
     *            包名
     */
    public static void uninstallApk(Context context, String pkgName) {
        Uri packageURI = Uri.parse("package:" + pkgName);
        Intent uninstallIntent = new Intent(Intent.ACTION_DELETE, packageURI);
        if (!"Xiaomi".equals(Build.BRAND)) {
        	uninstallIntent.setFlags(Intent.FLAG_ACTIVITY_NO_HISTORY);
        }
        context.startActivity(uninstallIntent);
    }
    
    /**
     * 获取用户安装的应用列表
     */
    public static List<PackageInfo> getInstalledApps(Context context) {
        PackageManager pm = context.getPackageManager();
        final String ourPackageName =  context.getPackageName();
        List<PackageInfo> packages = pm.getInstalledPackages(0);
        List<PackageInfo> apps = new ArrayList<PackageInfo>();
        for (PackageInfo info : packages) {
            // 只返回非系统级应用
            if (((info.applicationInfo.flags & ApplicationInfo.FLAG_SYSTEM) == 0)  && !ourPackageName.equals(info.packageName)) {
                apps.add(info);
            }
        }
        return apps;
    }
    
    /**
     * 获取用户所有安装的应用列表
     */
    public static List<PackageInfo> getAllInstalledApps(Context context) {
    	 PackageManager pm = context.getPackageManager();
         return pm.getInstalledPackages(0);
    }
	
	/**
     * 获取app包信息
     * 
     * @param context
     * @param packageName
     *            包名
     * @return
     */
    public static PackageInfo getAppPackageInfo(final Context context, final String packageName) {
        PackageInfo info = null;
        try {
            info = context.getPackageManager().getPackageInfo(packageName, 0);
        } catch (Exception e) {
            info = null;
            //e.printStackTrace();
        }
        return info;
    }
	
	/**
     * 通过包名,安全启动activity.捕获异常
     * @param context
     * @param packageName 要启动的应用包名
     * @return
     */
    public static boolean safeStartActivity(final Context context, String packageName) {
        try {
            if (context != null && !TextUtils.isEmpty(packageName)) {
                //通过包名获取应用Intent
                Intent intent = context.getPackageManager().getLaunchIntentForPackage(packageName);
                if (intent == null) {
                    PackageInfo packageInfo = getAppPackageInfo(context, packageName);
                    if (packageInfo != null && !TextUtils.isEmpty(packageInfo.applicationInfo.className)) {
                        intent = new Intent();
                        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                        intent.setClassName(packageName, packageInfo.applicationInfo.className);
                    }
                }
                if (intent != null) {
                    context.startActivity(intent);
                    return true;
                }
            }
        } catch (ActivityNotFoundException e) {
        } catch (Exception e) {
        }
        return false;
    }
    
	/**
	 * 安全启动activity，捕获异常
	 * @param context
	 * @param intent 启动的intent
	 * @return
	 */
	public static void safeStartActivity(final Context context, Intent intent) {
		try {
			if (context != null) {
				context.startActivity(intent);
			}
		} catch (ActivityNotFoundException e) {
			LogUtils.e("matt", "saveStartActivity err " + e.getMessage());
		} catch (SecurityException e) {
			LogUtils.e("matt", "saveStartActivity err " + e.getMessage());
		} catch (Exception e) {
			LogUtils.e("matt", "saveStartActivity err " + e.getMessage());
		}
	}

	/**
	 * 安全启动activity for result，捕获异常
	 * 
	 * @param context
	 * @param intent
	 *            启动的intent
	 * @return
	 */
	public static void safeStartActivityForResult(final Activity activity, Intent intent,
			int requestCode) {
		try {
			if (activity != null) {
				activity.startActivityForResult(intent, requestCode);
			}
		} catch (ActivityNotFoundException e) {
			LogUtils.e("matt", "saveStartActivityForResult err " + e.getMessage());
		} catch (SecurityException e) {
			LogUtils.e("matt", "saveStartActivityForResult err " + e.getMessage());
		}
	}

	private static String sCurrProcessName = null;
    /**
     * 获取当前进程名称
     * @param context
     * @return
     */
    public static String getCurrProcessName(Context context) {
        if (sCurrProcessName == null) {
            sCurrProcessName = getCurrProcessNameSystem(context);
        }
        return sCurrProcessName;
    }
    
	/**
	 * 获取当前进程名称
	 * @param context
	 * @return
	 */
	private static String getCurrProcessNameSystem(Context context) {
        //todo debug 更高效的方式：http://www.rØogerblog.cn/2016/03/17/android-proess/
		int myPid = android.os.Process.myPid();
		ActivityManager activityManager = (ActivityManager) context
				.getSystemService(Context.ACTIVITY_SERVICE);
		List<RunningAppProcessInfo> appProcessList = activityManager.getRunningAppProcesses();
		if (appProcessList != null) {
			for (ActivityManager.RunningAppProcessInfo appProcess : appProcessList) {
				if (appProcess.pid == myPid) {
					return appProcess.processName;
				}
			}
		}

		java.lang.Process process = null;
		BufferedReader reader = null;
		try {
			process = Runtime.getRuntime().exec("ps " + myPid);
			reader = new BufferedReader(new InputStreamReader(process.getInputStream()));
			String line = reader.readLine();
			String[] texts = null;
			if (line != null) { //第一行为标题
				if ((line = reader.readLine()) != null) { //第二行才是数据
					texts = line.split("\\s+", Integer.MAX_VALUE);
					String name = texts[texts.length - 1];
					return name;
				}
			}
		} catch (Throwable t) {
			t.printStackTrace();
		} finally {
			if (reader != null) {
				try {
					reader.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			if (process != null) {
				process.destroy();
			}
		}
		return null;
	}
	
    /**
     * 判断当前程序是否运行于前台 
     * @param context
     * @param pkgName
     * @return
     * @备注 5.0以上判断会有问题
     */
    @Deprecated 
    public static boolean isAppRunningInForground(Context context, String pkgName) {
        if (Build.VERSION.SDK_INT >= 21) {
        	//SDK5.0或以上
            return isForgroundApp(context, pkgName);
        } else {
            return isTopActivity(context, pkgName);
        }
    }
    
    /**
     * 获取屏幕最上层应用包名
     * @param context
     * @return
     * @备注 5.0以上判断会有问题
     */
    @Deprecated 
    public static String getTopAppPackageName(Context context) {
        try {
            ActivityManager activityManager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
            List<RunningTaskInfo> tasksInfo = activityManager.getRunningTasks(1);
            if (tasksInfo.size() > 0) {
                // Activity位于堆栈的顶层,如果Activity的类为空则判断的是当前应用是否在前台
                return tasksInfo.get(0).topActivity.getPackageName();
            }
        } catch (Exception e) {
        }
        return null;
    }
    
    /**
     * 根据包名判断是否为屏幕最上层应用
     * @param context
     * @param packageName 应用包名
     * @return
     * @备注 5.0以上判断会有问题
     */
    @Deprecated 
    private static boolean isTopActivity(Context context, String packageName) {
        try {
            ActivityManager activityManager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
            List<RunningTaskInfo> tasksInfo = activityManager.getRunningTasks(1);
            if (tasksInfo.size() > 0) {
                // Activity位于堆栈的顶层,如果Activity的类为空则判断的是当前应用是否在前台
                if (packageName.equals(tasksInfo.get(0).topActivity.getPackageName())) {
                    return true;
                }
            }
        } catch (Exception e) {
        }
        return false;
    }
    
    /**
     * 应用是否运行在前台
     * @param context
     * @param packageName 应用包名
     * @return
     * @备注 5.0以上判断会有问题
     */
    @Deprecated 
    private static boolean isForgroundApp(Context context, String packageName) {
        try {
        	 //获取当前正在运行进程列表
            ActivityManager activityManager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
            List<RunningAppProcessInfo> appProcesses = activityManager.getRunningAppProcesses(); 
            if (appProcesses == null) {
                return false; 
            }
            for (RunningAppProcessInfo appProcess : appProcesses) { 
                // 通过进程名及进程所用到的包名来进行查找
                if (appProcess.importance == RunningAppProcessInfo.IMPORTANCE_FOREGROUND) {
                    if (appProcess.processName.equals(packageName) || Arrays.asList(appProcess.pkgList).contains(packageName)) {
                        return true; 
                    }
                } 
            } 
        } catch (Exception e) {
        }
        return false; 
    }
	
	@Deprecated
	public static String getTopAppPkgName(Context context) {
		if (Machine.IS_SDK_ABOVE_L) {
			return null;
		}
		try {
			ActivityManager activityManager = (ActivityManager) context
					.getSystemService(Context.ACTIVITY_SERVICE);
			List<RunningTaskInfo> tasksInfo = activityManager.getRunningTasks(1);
			if (tasksInfo.size() > 0) {
				// Activity位于堆栈的顶层,如果Activity的类为空则判断的是当前应用是否在前台
				return tasksInfo.get(0).topActivity.getPackageName();
			}
		} catch (Exception e) {
		}
		return null;
	}
	
	/**
	 * 判断一个应用是否已经停止运行.<br>
	 * 
	 * @param context
	 * @param packageName
	 * @return
	 */
	@TargetApi(Build.VERSION_CODES.HONEYCOMB_MR1)
	public static boolean isAppStop(Context context, String packageName) {
		boolean isStop = false;
		try {
			ApplicationInfo app = context.getPackageManager()
					.getApplicationInfo(packageName, 0);
			isStop = (app.flags & ApplicationInfo.FLAG_STOPPED) != 0;
		} catch (Exception e) {
			isStop = true; // 通常是程序不存在了吧...
			e.printStackTrace();
		}
		return isStop;
	}
	
	/**
	 * 应用是否正在运行 
	 */
	public static boolean isAppRunning(Context context, String packageName) {
		Set<String> runningApps =  getRunningApps(context);
		return runningApps != null && runningApps.contains(packageName);
	}
	
	/**
	 * 获取正在运行的应用列表
	 * @备注 完美支持6.0版本
	 */
	public static Set<String> getRunningApps(Context context) {
		if (Build.VERSION.SDK_INT >= 21) {
			return ProcessHelperUtil.getRunningAppProcesses(context).keySet();
		} else {
			return getRunningAppsBelowApiLevel20(context);
		}
	}
	
	/**
	 * api level 20及以下系统，获取正在运行的应用列表
	 */
	private static Set<String> getRunningAppsBelowApiLevel20(Context context) {
		Set<String> runningApps = new HashSet<String>();
		ActivityManager am = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
		List<RunningAppProcessInfo> list = am.getRunningAppProcesses();
		for (RunningAppProcessInfo info : list) {
			if (info.pkgList != null) {
				for (String pkg : info.pkgList) {
					runningApps.add(pkg);
				}
			}
		}
		return runningApps;
	}

	/**
	 * 根据包名获取应用的uid
	 * @param context
	 * @param packageName
	 * @return
	 */
	public static int getUidByPkgName(final Context context, final String packageName) {
		int uid = 0;
		try {
			ApplicationInfo info = context.getPackageManager().getApplicationInfo(packageName, PackageManager.GET_ACTIVITIES);
			uid = info.uid;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return uid;
	}

	/**
	 * 服务是否正在运行
	 * 
	 * @param context
	 * @param packageName
	 *            包名
	 * @param serviceName
	 *            服务名
	 * @return
	 */
	public static boolean isServiceRunning(Context context, String packageName, String serviceName) {
		ActivityManager activityManager = (ActivityManager) context
				.getSystemService(Context.ACTIVITY_SERVICE);
		return isServiceRunning(activityManager, packageName, serviceName);
	}

	public static boolean isServiceRunning(ActivityManager activityManager, String packageName,
			String serviceName) {
		List<RunningServiceInfo> serviceTasks = activityManager.getRunningServices(Integer.MAX_VALUE);
		int sz = null == serviceTasks ? 0 : serviceTasks.size();
		for (int i = 0; i < sz; i++) {
			RunningServiceInfo info = serviceTasks.get(i);
			if (null != info && null != info.service) {
				final String pkgName = info.service.getPackageName();
				final String className = info.service.getClassName();

				if (pkgName != null && pkgName.contains(packageName) && className != null
						&& className.contains(serviceName)) {
					LogUtils.i("Notification", "package = " + info.service.getPackageName()
							+ " class = " + info.service.getClassName());
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * 浏览器直接访问uri
	 * 
	 * @param uriString
	 * @return 成功打开返回true
	 */
	public static boolean gotoBrowser(Context context, String uriString) {
		boolean ret = false;
		if (uriString == null) {
			return ret;
		}
		Uri browserUri = Uri.parse(uriString);
		if (null != browserUri) {
			Intent browserIntent = new Intent(Intent.ACTION_VIEW, browserUri);
			browserIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
			try {
				context.startActivity(browserIntent);
				ret = true;
			} catch (ActivityNotFoundException e) {
				LogUtils.i("matt", "gotoBrowser error, uri = " + uriString);
			} catch (Exception e) {
				LogUtils.i("matt", "gotoBrowser error, uri = " + uriString);
			}
		}
		return ret;
	}
	
	/**
	 * 获取在功能菜单出现的程序列表
	 * 
	 * @param context
	 *            上下文
	 * @return 程序列表，类型是 List<ResolveInfo>
	 */
	public static List<ResolveInfo> getLauncherApps(Context context, String action) {
		List<ResolveInfo> infos = null;
		PackageManager packageMgr = context.getPackageManager();
		Intent intent = new Intent(action);
		intent.addCategory(Intent.CATEGORY_LAUNCHER);
		try {
			infos = packageMgr.queryIntentActivities(intent, 0);
		} catch (OutOfMemoryError e) {
			e.printStackTrace();
//			OutOfMemoryHandler.handle();
		} catch (Exception e) {
			e.printStackTrace();
		}
		packageMgr = null;
		return infos;
	}

	/**
	 * 卸载程序
	 * 
	 * @param context
	 *            上下文
	 * @param packageURI
	 *            需要卸载的程序的Uri
	 */
	public static void uninstallApp(Activity context, Uri packageURI) {
		Intent uninstallIntent = new Intent(Intent.ACTION_DELETE, packageURI);
//		uninstallIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
		context.startActivity(uninstallIntent);
		uninstallIntent = null;
	}

	/**
	 * 卸载包
	 * 
	 * @param context
	 *            上下文
	 * @param packageURI
	 *            需要卸载的程序的Uri
	 */
	public static void uninstallPackage(Activity context, String pkgName) {
		Uri packageURI = Uri.parse("package:" + pkgName);
//		GoAppUtils.uninstallApp(context, packageURI);
		uninstallApp(context, packageURI);
	}
	
	/**
	 * 判断go桌面是否是默认桌面
	 * @param context
	 * @return
	 */
	public static boolean isDefaultLauncher(Context context) {
		if (context.getPackageName().equals(getDefaultLauncher(context))) {
			return true;
		}
		return false;
	}


	public static void showAppDetails(Context context, String packageName) {
	    final String scheme = "package";
		/**
		 * 调用系统InstalledAppDetails界面所需的Extra名称(用于Android 2.1及之前版本)
		 */
		final String appPkgName21 = "com.android.settings.ApplicationPkgName";
		/**
		 * 调用系统InstalledAppDetails界面所需的Extra名称(用于Android 2.2)
		 */
		final String appPkgName22 = "pkg";
		/**
		 * InstalledAppDetails所在包名
		 */
		final String appDetailsPackageName = "com.android.settings";
		/**
		 * InstalledAppDetails类名
		 */
		final String appDetailsClassName = "com.android.settings.InstalledAppDetails";

		Intent intent = new Intent();
		final int apiLevel = Build.VERSION.SDK_INT;
		if (apiLevel >= 9) {
			// 2.3（ApiLevel 9）以上，使用SDK提供的接口
			intent.setAction(ACTION_SETTINGS);
			Uri uri = Uri.fromParts(scheme, packageName, null);
			intent.setData(uri);
		} else {
			// 2.3以下，使用非公开的接口（查看InstalledAppDetails源码）
			// 2.2和2.1中，InstalledAppDetails使用的APP_PKG_NAME不同。
			final String appPkgName = apiLevel == 8 ? appPkgName22 : appPkgName21;
			intent.setAction(Intent.ACTION_VIEW);
			intent.setClassName(appDetailsPackageName, appDetailsClassName);
			intent.putExtra(appPkgName, packageName);
		}

		try {
			context.startActivity(intent);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * 判断应用是否安装在手机内存里
	 * 
	 * @author kingyang
	 * @param context
	 * @param intent
	 * @return
	 */
	public static boolean isInternalApp(Context context, Intent intent) {
		if (context != null) {
			PackageManager pkgMgr = context.getPackageManager();
			try {
				String internalPath = Environment.getDataDirectory().getAbsolutePath();
				String dir = pkgMgr.getActivityInfo(intent.getComponent(), 0).applicationInfo.publicSourceDir;
				if (dir != null && dir.length() > 0) {
					return dir.startsWith(internalPath);
				}
			} catch (NameNotFoundException e) {
				e.printStackTrace();
			}
		}
		return false;
	}

	/**
	 * 判断应用是否是系统应用
	 * 
	 * @author kingyang
	 * @param context
	 * @param intent
	 * @return
	 */
	public static boolean isSystemApp(Context context, Intent intent) {
		boolean isSystemApp = false;
		if (context != null) {
			PackageManager pkgMgr = context.getPackageManager();
			try {
				ApplicationInfo applicationInfo = pkgMgr.getActivityInfo(intent.getComponent(), 0).applicationInfo;
				if (applicationInfo != null) {
					isSystemApp = ((applicationInfo.flags & ApplicationInfo.FLAG_SYSTEM) != 0)
							|| ((applicationInfo.flags & ApplicationInfo.FLAG_UPDATED_SYSTEM_APP) != 0);
				}
			} catch (NameNotFoundException e) {
				e.printStackTrace();
			}
		}
		return isSystemApp;
	}
	
	/**
	 * 是否是系统app
	 * @param context
	 * @param pkgName
	 * @return
	 */
	public static boolean isSystemApp(Context context, String pkgName) {
		ApplicationInfo applicationInfo = getApplicationInfo(context, pkgName);
		if (null == applicationInfo) {
			return false;
		}
		boolean isSystemApp = ((applicationInfo.flags & ApplicationInfo.FLAG_SYSTEM) != 0)
				|| ((applicationInfo.flags & ApplicationInfo.FLAG_UPDATED_SYSTEM_APP) != 0);
		return isSystemApp;
	}

	/**
	 * 获取指定应用的Context
	 * @param context
	 * @param packageName
	 * @return
	 */
	public static Context getAppContext(Context context, String packageName) {
		Context ctx = null;
		try {
			ctx = context.createPackageContext(packageName, Context.CONTEXT_IGNORE_SECURITY);
		} catch (NameNotFoundException e) {
			e.printStackTrace();
		}
		return ctx;
	}

	/**
	 * Check if the installed Gmail app supports querying for label information.
	 * 
	 * @param c
	 *            an application Context
	 * @return true if it's safe to make label API queries
	 */
	public static boolean canReadGmailLabels(Context c) {
		/**
		 * Permission required to access this
		 * {@link android.content.ContentProvider}
		 */
		final String permission = "com.google.android.gm.permission.READ_CONTENT_PROVIDER";
		/**
		 * Authority for the Gmail content provider.
		 */
		final String authority = "com.google.android.gm";
		String gmailPackageName = "com.google.android.gm";

		boolean supported = false;

		try {
			final PackageInfo info = c.getPackageManager().getPackageInfo(gmailPackageName,
					PackageManager.GET_PROVIDERS | PackageManager.GET_PERMISSIONS);
			boolean allowRead = false;
			if (info.permissions != null) {
				for (int i = 0, len = info.permissions.length; i < len; i++) {
					final PermissionInfo perm = info.permissions[i];
					if (permission.equals(perm.name)
							&& perm.protectionLevel < PermissionInfo.PROTECTION_SIGNATURE) {
						allowRead = true;
						break;
					}
				}
			}
			if (allowRead && info.providers != null) {
				for (int i = 0, len = info.providers.length; i < len; i++) {
					final ProviderInfo provider = info.providers[i];
					if (authority.equals(provider.authority)
							&& TextUtils.equals(permission, provider.readPermission)) {
						supported = true;
					}
				}
			}
		} catch (NameNotFoundException e) {
			// Gmail app not found
		}
		return supported;
	}
	
	public static boolean isBrowser(Context context, String packageName) {
		if (packageName == null) {
			return false;
		}
        Intent intent = new Intent(Intent.ACTION_VIEW);
        intent.addCategory(Intent.CATEGORY_DEFAULT);
        intent.addCategory(Intent.CATEGORY_BROWSABLE);
        intent.setData(Uri.parse("http://3g.cn"));
        intent.setPackage(packageName);

        PackageManager pm = context.getPackageManager();
        List<ResolveInfo> infos = pm.queryIntentActivities(intent, 0);
        if (infos != null && infos.size() > 0) {
            return true;
        }
        return false;
    }
	
    /**
	 * <br>功能简述:在列表里过滤删除指定包名的应用
	 * <br>功能详细描述:
	 * <br>注意:
	 * @param resolveList
	 * @param pkgs
	 * @return
	 */
	public static List<ResolveInfo> filterPkgs(List<ResolveInfo> resolveList, String[] pkgs) {
		if (resolveList == null || resolveList.size() == 0 || pkgs == null || pkgs.length == 0) {
			return resolveList;
		}
		
		ArrayList<ResolveInfo> list = new ArrayList<ResolveInfo>(resolveList);
		
		for (ResolveInfo resolveInfo : resolveList) {
			if (resolveInfo != null && resolveInfo.activityInfo != null && resolveInfo.activityInfo.packageName != null) {
				for (String pkg : pkgs) {
					if (resolveInfo.activityInfo.packageName.equals(pkg)) {
						list.remove(resolveInfo);
						break;
					}
				}
			}
		}
		
		return list;
	}
	
	public static ServiceInfo getServiceInfo(Context context, ComponentName component) {
		try {
			PackageManager packageManager = context.getPackageManager();
			return packageManager.getServiceInfo(component, PackageManager.GET_SERVICES);
		} catch (NameNotFoundException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public static ApplicationInfo getApplicationInfo(Context context, String pkgName) {
		if (pkgName == null) {
			return null;
		}
		try {
			PackageManager pkManager = context.getPackageManager();
			return pkManager.getApplicationInfo(pkgName, 0);
		} catch (NameNotFoundException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public static Drawable getAppIcon(final Context context, final String pkgName) {
		if (pkgName == null) {
			return null;
		}
		
		Drawable icon = null;
		try {
			PackageManager pkManager = context.getPackageManager();
			ApplicationInfo info = pkManager.getApplicationInfo(pkgName, 0);
			if (info != null) {
				icon = info.loadIcon(pkManager);
			}
		} catch (NameNotFoundException e) {
			e.printStackTrace();
		} catch (OutOfMemoryError e) {
            e.printStackTrace();
        }
		return icon;
	}
	
	public static String getAppLable(final Context context, final String pkgName) {
		if (pkgName == null) {
			return null;
		}
		
		String label = null;
		try {
			PackageManager pkManager = context.getPackageManager();
			ApplicationInfo info = pkManager.getApplicationInfo(pkgName, 0);
			if (info != null) {
				label = pkManager.getApplicationLabel(info).toString();
			}
		} catch (NameNotFoundException e) {
			e.printStackTrace();
		}

		return label;
	}
	
	/**
	 * 获取已安装的所有应用信息
	 * @return
	 */
	public static List<ApplicationInfo> getInstallAppInfo(Context context) {
		if (context == null) {
			return null;
		}
        PackageManager pkgManager = context.getPackageManager();  
        List<ApplicationInfo> appInfoList = pkgManager.getInstalledApplications(PackageManager.GET_UNINSTALLED_PACKAGES);  
        //排序  
        //Collections.sort(appInfoList, new ApplicationInfo.DisplayNameComparator(mypm));  
        return appInfoList;  
    }  
	
	/**
	 * 获取apk安装包的大小
	 * @param packageManager
	 * @param pkgName
	 * @return
	 */
	public static long getApkSize(PackageManager packageManager, String packageName) {
		if (TextUtils.isEmpty(packageName) || packageManager == null) {
			return 0;
		}
		try {
			ApplicationInfo applicationInfo = packageManager.getApplicationInfo(packageName, 0);
			if (applicationInfo != null && applicationInfo.sourceDir != null) {
				return new File(applicationInfo.sourceDir).length();
			}
		} catch (NameNotFoundException e) {
			e.printStackTrace();
		}
		return 0;
	}
	
	/**
	* 判断包名所对应的应用是否安装在SD卡上
	* @param packageName
	* @return, true if install on SD card 
	*/
	public static boolean isInstallOnSDCard(PackageManager packageManager, String packageName) {
		if (TextUtils.isEmpty(packageName) || packageManager == null) {
			return false;
		}
		try {
			ApplicationInfo appInfo = packageManager.getApplicationInfo(packageName, 0);
			if ((appInfo.flags & ApplicationInfo.FLAG_EXTERNAL_STORAGE) != 0) {
				return true;
			}
		} catch (NameNotFoundException e) {
			e.printStackTrace();
		}
		return false;
	}
}
