package com.jb.ga0.commerce.util;

import java.util.ArrayList;

/**
 * 广播器
 *
 * @author matt
 * @date: 2014年12月23日
 *
 */
public class Broadcaster {
	
	/**
	 * 广播器观察者
	 *
	 * @author matt
	 * @date: 2014年12月23日
	 *
	 */
	public static interface BroadcastObserver {
		
		/**
		 * 消息广播回调
		 * @param msgId
		 * @param param
		 * @param objects
		 */
		public void onBroadcastMsg(int msgId, int param, Object ...objects);
	}
	
	private ArrayList<BroadcastObserver> mObservers;
	private Object mLocker = new Object();

	/**
	 * 注册观察者
	 * @param oberver
	 */
	public void registerObserver(BroadcastObserver oberver) {
		if (oberver == null) {
			return;
		}
		
		synchronized (mLocker) {
			
			if (mObservers == null) {
				mObservers = new ArrayList<BroadcastObserver>();
			}
			
			try {
				if (mObservers.indexOf(oberver) < 0) {
					mObservers.add(oberver);
				}
			} catch (Exception e) {
				e.printStackTrace();
			} catch (OutOfMemoryError e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * 反注册观察者
	 * @param observer 观察者
	 * @return 是否去注册成功 true:成功 false:不成功或不存在此观察者
	 */
	public boolean unregisterObserver(BroadcastObserver observer) {
		synchronized (mLocker) {
			if (null == mObservers) {
				return false;
			}
			
			return mObservers.remove(observer);
		}
	}
	
	/**
	 * 清理所有的观察者
	 */
	public void clearAllObserver() {
		synchronized (mLocker) {
			if (mObservers != null) {
				mObservers.clear();
				mObservers = null;
			}
		}
	}

	/**
	 * 广播
	 * 
	 * @param msgId
	 * @param param
	 * @param object
	 * @param objects
	 */
	@SuppressWarnings("unchecked")
	public void broadcast(int msgId, int param, Object ...objects) {
		ArrayList<BroadcastObserver> clone = null;
		synchronized (mLocker) {
			if (mObservers != null) {
				clone = (ArrayList<BroadcastObserver>) mObservers.clone();
			}
		}
		if (null == clone) {
			return;
		}
		for (BroadcastObserver observer : clone) {
			if (observer != null) {
				observer.onBroadcastMsg(msgId, param, objects);
			}
		}
	}
}
