package com.jb.ga0.commerce.util;

import java.util.HashMap;

import android.content.Context;
import android.text.TextUtils;

/**
 * 自定义闹钟管理器，管理多个闹钟（多个SDK同时使用时，可以各自管理自己的闹钟）
 *
 * @author matt
 * @date: 2015年1月30日
 */
public class CustomAlarmManager {
	private static final String ALARM_ACTION_MIDDLE = ".commerce.action.alarm."; 
	
	private static CustomAlarmManager sInstance;
	private Context mContext;
	/**
	 * <moduleName, CustomAlarm>
	 */
	private HashMap<String, CustomAlarm> mAlarms = new HashMap<String, CustomAlarm>();

	private CustomAlarmManager(Context context) {
		context = context.getApplicationContext();
		mContext = context;
	}
	
	public static CustomAlarmManager getInstance(Context context) {
		if (sInstance == null) {
			sInstance = new CustomAlarmManager(context);
		}
		return sInstance;
	}
	
	/**
	 * 获取闹钟
	 * @param moduleName 模块名称(每个SDK有一个唯一的模块名称，形如"tokencoin", "chargelocker"等)
	 * @return
	 */
	public CustomAlarm getAlarm(String moduleName) {
		if (TextUtils.isEmpty(moduleName)) {
			return null;
		}
		synchronized (mAlarms) {
			CustomAlarm alarm = mAlarms.get(moduleName);
			if (alarm == null) {
				alarm = new CustomAlarm(mContext, mContext.getPackageName() + ALARM_ACTION_MIDDLE + moduleName);
				mAlarms.put(moduleName, alarm);
			}
			return alarm;
		}
	}
	
}
