package com.jb.ga0.commerce.util;

import android.os.Bundle;
import android.os.Environment;
import android.text.TextUtils;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

/**
 * 开发助手相关，读取开发开关相关配置的工具类
 * Created by xiexingwu on 2016/6/3.
 * v1
 */
public class DevHelper {
    private static final String sDEV_HELPER_FILE_NAME = "devhelper";
    private static String sPATH = Environment.getExternalStorageDirectory().getAbsolutePath()
            + File.separator + sDEV_HELPER_FILE_NAME + File.separator;
    private static final String sDEBUG_FILENAME_SUFFIX_DEBUG = "debug";
    /**
     * 通用配置文件名字，只能针对没有设置包名的应用
     */
    public static final String sCOMMON = "common";
    private static final String sSPLIT_KEY = "=";
    public static final String sVALUE_TRUE = "true";
    public static final String sVALUE_FALSE = "false";
    private static final String sPOINT = ".";
    private static final String sUNDER_LINE = "_";
    private static String sPACKAGE_CONFIG = sCOMMON;
    private Bundle mBundle = null;
    private File mDebugFile = null;
    private String mPackageName;

    /**
     * 默认通用配置，所有其他应用通用
     */
    public DevHelper() {
        this(sPACKAGE_CONFIG);
    }

    /**
     * 指定包名配置
     *
     * @param packageName
     */
    public DevHelper(String packageName) {
        setRegister(packageName);
        mPackageName = packageName;
        mDebugFile = getDeBugFileByName(sPACKAGE_CONFIG);
        mBundle = new Bundle();
        buid(mDebugFile);
    }

    /**
     * 刷新LOG开关路径
     */
    public static void setRegister(String packageName) {
        sPACKAGE_CONFIG = packageName.replace(sPOINT, sUNDER_LINE);
    }

    /**
     * 获取当前对应控制文件的apk包名
     *
     * @return 通用情况返回:{@link #sCOMMON}，其他情况返回包名
     */
    public String getPackageName() {
        return TextUtils.isEmpty(mPackageName) ? sPACKAGE_CONFIG : mPackageName;
    }

    /**
     * 根据开关名字打开开关
     *
     * @param switchKeyName 开关名字
     */
    public void open(String switchKeyName) {
        setValueByKey(switchKeyName, true);
    }

    /**
     * 根据开关名字关闭开关
     *
     * @param switchKeyName 开关名字
     */
    public void close(String switchKeyName) {
        setValueByKey(switchKeyName, false);
    }

    /**
     * 设置对于key的value值
     *
     * @param key
     * @param switchValue
     */
    public void setValueByKey(String key, boolean switchValue) {
        saveValue(key, switchValue ? sVALUE_TRUE : sVALUE_FALSE);
    }

    /**
     * 设置对于key的value值
     *
     * @param key
     * @param value
     */
    public void setValueByKey(String key, int value) {
        saveValue(key, value + "");
    }

    /**
     * 设置对于key的value值
     *
     * @param key
     * @param value
     */
    public void setValueByKey(String key, String value) {
        saveValue(key, value);
    }

    /**
     * 保存更改的键值对
     *
     * @param key
     * @param newValue
     */
    private void saveValue(String key, String newValue) {
        if (TextUtils.isEmpty(key) || TextUtils.isEmpty(key = key.trim())
                || TextUtils.isEmpty(newValue) || TextUtils.isEmpty(newValue = newValue.trim())) {
            return;
        }
        if (mBundle == null) {
            buid(mDebugFile);
        }
        String value = mBundle.getString(key);
        if (!TextUtils.isEmpty(value) && newValue.equalsIgnoreCase(value)) {
            //值一致，不需要改动
            return;
        }
        mBundle.putString(key, newValue);
        out(mDebugFile);
    }

    /**
     * 移除指定键值对
     *
     * @param key
     */
    public void delete(String key) {
        if (TextUtils.isEmpty(key) || TextUtils.isEmpty(key = key.trim())) {
            return;
        }
        if (mBundle == null) {
            return;
        }
        mBundle.remove(key);
        out(mDebugFile);
    }

    /***
     * 根据开关名字判断开关是否打开
     *
     * @param switchKeyName
     * @return 所有的默认开关为false
     */
    public boolean isOpen(String switchKeyName) {
        String value = getValueByKey(switchKeyName);
        return isOpenValue(value);
    }

    /**
     * 判断值是不是对应开关打开的值
     *
     * @return
     */
    public static boolean isOpenValue(String value) {
        return !TextUtils.isEmpty(value) && Boolean.parseBoolean(value.trim());
    }

    /***
     * 根据开关名字判断开关是否关闭
     *
     * @param switchKeyName
     * @return 所有的默认开关为false
     */
    public boolean isClose(String switchKeyName) {
        String value = getValueByKey(switchKeyName);
        return isCloseValue(value);
    }

    /**
     * 判断值是不是对应开关关闭的值
     *
     * @return
     */
    public static boolean isCloseValue(String value) {
        return !TextUtils.isEmpty(value) && sVALUE_FALSE.equalsIgnoreCase(value.trim());
    }

    /***
     * @param key
     * @return 是否是开关项, true是开关，false不是开关
     */
    public boolean isSwitch(String key) {
        String value = getValueByKey(key);
        return isOpenValue(value) || isCloseValue(value);
    }

    /**
     * @param fileName
     * @return
     */
    private static File getDeBugFileByName(String fileName) {
        return new File(sPATH + fileName + sPOINT + sDEBUG_FILENAME_SUFFIX_DEBUG);
    }

    /**
     * @return
     */
    public static File getDeBugFileByName() {
        return getDeBugFileByName(sPACKAGE_CONFIG);
    }

    /**
     * 用key获取Value
     *
     * @return
     */
    public String getValueByKey(String switchKeyName) {
        if (mBundle == null) {
            return null;
        }
        String value = mBundle.getString(switchKeyName);
        return value;
    }

    /**
     * 从SD卡读取构建Debug配置
     *
     * @param file
     */
    private void buid(File file) {
        if (file == null || !file.isFile() || !file.exists()) {
            return;
        }
        try {
            BufferedReader reader = new BufferedReader(new FileReader(file));
            String tempString = null;
            while ((tempString = reader.readLine()) != null) {
                tempString = tempString.trim();
                if (!TextUtils.isEmpty(tempString) && tempString.contains(sSPLIT_KEY)) {
                    String[] spstr = tempString.split(sSPLIT_KEY);
                    if (spstr.length > 1 && !TextUtils.isEmpty(spstr[0].trim())
                            && !TextUtils.isEmpty(spstr[1])) {
                        mBundle.putString(spstr[0].trim(), spstr[1]);
                    }
                }
            }
            reader.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void outOnDismiss() {
        if (!mDebugFile.exists()) {
            try {
                createFileInUnExists(mDebugFile);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * debug配置Bundle序列化到SD卡，buid()的反向操作
     */
    private void out(File file) {
        if (mBundle == null || mBundle.isEmpty() || file == null) {
            return;
        }
        BufferedWriter bw = null;
        try {
            Set<String> keySet = mBundle.keySet();
            file.deleteOnExit();
            createFileInUnExists(file);
            bw = new BufferedWriter(new FileWriter(file));
            Iterator<String> iterator = keySet.iterator();
            while (iterator.hasNext()) {
                String key = iterator.next();
                String value = mBundle.getString(key);
                if (!TextUtils.isEmpty(key) && !TextUtils.isEmpty(value)) {
                    bw.write(key + sSPLIT_KEY + value);
                    bw.newLine();
                }
            }
            bw.flush();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (bw != null) {
                try {
                    bw.close();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
        }
    }

    /**
     * 获取有dev文件的app包名
     *
     * @return
     */
    public static List<String> getDevApps() {
        List<String> list = new ArrayList<String>();
        File devs = new File(sPATH);
        if (devs.exists()) {
            File[] files = devs.listFiles();
            if (files != null && files.length > 0) {
                for (File file : files) {
                    String name = file.getName();
                    if (name.endsWith(sPOINT + sDEBUG_FILENAME_SUFFIX_DEBUG) && name.contains(sUNDER_LINE) && !name.startsWith(sCOMMON)) {
                        name = name.replace(sPOINT + sDEBUG_FILENAME_SUFFIX_DEBUG, "");
                        name = name.replace(sUNDER_LINE, sPOINT);
                        list.add(name);
                    }
                }
            }
        }
        return list;
    }

    /**
     * 获取所有键值对的keySet，已读取的文件（在内存）
     *
     * @return
     */
    public Set<String> keySet() {
        if (mBundle == null || mBundle.isEmpty()) {
            return null;
        }
        return mBundle.keySet();
    }

    /**
     * 获取所有键值对的keySet,重新读取文件
     *
     * @return
     */
    public Set<String> keySetBuid() {
        buid(mDebugFile);
        return keySet();
    }

    /**
     * 新建文件（不是文件夹），当父路径没有，也会自动创建
     */
    private static void createFileInUnExists(File file) throws IOException {
        if (file == null) {
            return;
        }
        file.getParentFile().mkdirs();
        file.createNewFile();
    }

    /**
     * 提取包名方法（方法体可以优化，最好用正则）
     *注意：动态加载有问题
     * @return
     */
    private static String peepPackageName() {
        try {
            List<String> names = new ArrayList<>();
            ClassLoader classLoader = DevHelper.class.getClassLoader();
            String s = classLoader.toString();
            if (!TextUtils.isEmpty(s) && s.contains("\"")) {
                String[] strings = s.split("\"");

                for (String s1 : strings) {
                    if (!TextUtils.isEmpty(s1) && s1.endsWith(".apk")) {
                        names.add(s1);
                    }
                }
            }

            for (String name : names) {
                if (!TextUtils.isEmpty(name) && name.startsWith("/data/app/") && name.endsWith(".apk")) {
                    name = name.replace("/data/app/", "").replace(".apk", "");
                    if (name.contains("-")) {
                        String[] ns = name.split("-");
                        if (ns.length > 0) {
                            return ns[0];
                        }
                    }
                }
            }
            return null;
        } catch (Exception e) {
        }

        return null;
    }
}
