package com.jb.ga0.commerce.util;

import java.lang.reflect.Field;
import java.lang.reflect.Method;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Point;
import android.os.Build;
import android.util.DisplayMetrics;
import android.view.Display;
import android.view.View;
import android.view.ViewConfiguration;
import android.view.WindowManager;

/**
 * 绘制工具类
 *
 * @author matt
 * @date: 2015年2月3日
 *
 */
public class DrawUtils {
	
	public static float sDensity = 1.0f;
	public static int sDensityDpi;
	public static float sFontDensity;
	public static int sWidthPixels;
	public static int sHeightPixels;
	public static int sStatusHeight = -1; //系统状态栏高度
	private final static int STATUS_BAR_DEFAULT_HEIGHT_DP = 25;
	// 点击的最大识别距离，超过即认为是移动
	public static int sTouchSlop = 15; 
	private static Point sOutSize = new Point();
	
	/** ========sdk19(KITKAT) 及以上android版本用======begin===== */
	public static final int NAVBAR_LOCATION_RIGHT = 1;
	public static final int NAVBAR_LOCATION_BOTTOM = 2;
	public static int sNavBarLocation;
	private static int sRealWidthPixels;
	private static int sRealHeightPixels;
	private static int sNavBarWidth; // 虚拟键宽度
	private static int sNavBarHeight; // 虚拟键高度
	public static int sTabletStatusHeight; // 平板中底边的状态栏高度
	private static Class sDisplayClass = null;
	private static Method sMethodForWidth = null;
	private static Method sMethodForHeight = null;
	/** ========sdk19(KITKAT) 及以上android版本用======end===== */

	/**
	 * dip/dp转像素
	 * 
	 * @param dipValue
	 *            dip或 dp大小
	 * @return 像素值
	 */
	public static int dip2px(float dipVlue) {
		return (int) (dipVlue * sDensity + 0.5f);
	}

	/**
	 * 像素转dip/dp
	 * 
	 * @param pxValue
	 *            像素大小
	 * @return dip值
	 */
	public static int px2dip(float pxValue) {
		final float scale = sDensity;
		return (int) (pxValue / scale + 0.5f);
	}

	/**
	 * sp 转 px
	 * 
	 * @param spValue
	 *            sp大小
	 * @return 像素值
	 */
	public static int sp2px(float spValue) {
		final float scale = sDensity;
		return (int) (scale * spValue);
	}

	/**
	 * px转sp
	 * 
	 * @param pxValue
	 *            像素大小
	 * @return sp值
	 */
	public static int px2sp(float pxValue) {
		final float scale = sDensity;
		return (int) (pxValue / scale);
	}

	@SuppressLint("NewApi")
	public static void resetDensity(Context context) {
		if (context != null && null != context.getResources()) {
			DisplayMetrics metrics = context.getResources().getDisplayMetrics();
			sDensity = metrics.density;
			sFontDensity = metrics.scaledDensity;
			sDensityDpi = metrics.densityDpi;
			
			WindowManager wm = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
			Display display = wm.getDefaultDisplay();
			if (Machine.IS_ICS) {
				display.getSize(sOutSize);
				sWidthPixels = sOutSize.x;
				sHeightPixels = sOutSize.y;
			} else {
				sWidthPixels = display.getWidth();
				sHeightPixels = display.getHeight();
			}
			
			try {
				Class<?> clazz = Class.forName("android.view.Display");
				Point realSize = new Point();
				Method method = clazz.getMethod("getRealSize", Point.class);
				method.invoke(display, realSize);
				sRealWidthPixels = realSize.x;
				sRealHeightPixels = realSize.y;
			} catch (Throwable e) {
				sRealWidthPixels = sWidthPixels;
				sRealHeightPixels = sHeightPixels;
			}
			
			try {
				final ViewConfiguration configuration = ViewConfiguration.get(context);
				if (null != configuration) {
					sTouchSlop = configuration.getScaledTouchSlop();
				}
			} catch (Throwable e) {
				e.printStackTrace();
			}
			resetNavBarHeight(context);
			if (sStatusHeight < 0) { //使用<0来作为是否初始化过的标志
				resetStatusHeight(context);
			}
		}
	}
	
	/**
	 * 获取状态栏高度
	 * @param context
	 * @return
	 * @备注 这个方法必须在初始化@{ #sDensity }后调用
	 */
	private static void resetStatusHeight(Context context) {
//		int defaultStatusBarHeight = context.getResources().getInteger(R.integer.status_bar_height);
		int defaultStatusBarHeight = 25;
		Class<?> clazz = null;
		Object obj = null;
		Field field = null;
		try {
			clazz = Class.forName("com.android.internal.R$dimen");
			obj = clazz.newInstance();
			if (Machine.isMeizu()) {
				try {
					field = clazz.getField("status_bar_height_large");
				} catch (Throwable t) {
					t.printStackTrace();
				}
			}
			if (field == null) {
				field = clazz.getField("status_bar_height");
			}
		} catch (Throwable t) {
			t.printStackTrace();
		}
		if (field != null && obj != null) {
			try {
				int id = Integer.parseInt(field.get(obj).toString());
				sStatusHeight = context.getResources().getDimensionPixelSize(id);
			} catch (Throwable t) {
				t.printStackTrace();
			}
		}

		if (Machine.isTablet(context)
				&& sStatusHeight > DrawUtils.dip2px(STATUS_BAR_DEFAULT_HEIGHT_DP)) {
			//状态栏高度大于25dp的平板，状态栏通常在下方
			sStatusHeight = 0;
		} else {
			if (sStatusHeight <= 0
					|| sStatusHeight > DrawUtils.dip2px(STATUS_BAR_DEFAULT_HEIGHT_DP * 2)) {
				//安卓默认状态栏高度为25dp，如果获取的状态高度大于2倍25dp的话，这个数值可能有问题，用回桌面定义的值从新获取。出现这种可能性较低，只有小部分手机出现
//				if (DrawUtils.sVirtualDensity == -1) {
					sStatusHeight = DrawUtils.dip2px(defaultStatusBarHeight);
//				} else {
//					sStatusHeight = (int) (mDefaultStatusBarHeight * DrawUtils.sVirtualDensity + 0.5f);
//				}
			}
		}
	}
	
	public static void resetNavBarHeight(Context context) {
		sNavBarWidth = DrawUtils.sRealWidthPixels - DrawUtils.sWidthPixels;
		sNavBarHeight = DrawUtils.sRealHeightPixels - DrawUtils.sHeightPixels;
		sNavBarLocation = getNavBarLocation();
	}
	
	public static int getNavBarLocation() {
		if (DrawUtils.sRealWidthPixels > DrawUtils.sWidthPixels) {
			return NAVBAR_LOCATION_RIGHT;
		}
		return NAVBAR_LOCATION_BOTTOM;
	}
	
	public static int getTabletScreenWidth(Context context) {
		int width = 0;
		if (context != null) {
			try {
				WindowManager wm = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
				Display display = wm.getDefaultDisplay();
				if (sDisplayClass == null) {
					sDisplayClass = Class.forName("android.view.Display");
				}
				if (sMethodForWidth == null) {
					sMethodForWidth = sDisplayClass.getMethod("getRealWidth");
				}
				width = (Integer) sMethodForWidth.invoke(display);
			} catch (Exception e) {
			}
		}

		// Rect rect= new Rect();
		// ((Activity)
		// context).getWindow().getDecorView().getWindowVisibleDisplayFrame(rect);
		// int statusbarHeight = height - rect.bottom;
		if (width == 0) {
			width = sWidthPixels;
		}

		return width;
	}

	public static int getTabletScreenHeight(Context context) {
		int height = 0;
		if (context != null) {
			WindowManager wm = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
			Display display = wm.getDefaultDisplay();
			try {
				if (sDisplayClass == null) {
					sDisplayClass = Class.forName("android.view.Display");
				}
				if (sMethodForHeight == null) {
					sMethodForHeight = sDisplayClass.getMethod("getRealHeight");
				}
				height = (Integer) sMethodForHeight.invoke(display);
			} catch (Exception e) {
			}
		}

		// Rect rect= new Rect();
		// ((Activity)
		// context).getWindow().getDecorView().getWindowVisibleDisplayFrame(rect);
		// int statusbarHeight = height - rect.bottom;
		if (height == 0) {
			height = sHeightPixels;
		}

		return height;
	}
	
	public static int getRealWidth() {
		if (Machine.IS_SDK_ABOVE_KITKAT) {
			return sRealWidthPixels;
		}
		return sWidthPixels;
	}
	
	public static int getRealHeight() {
		if (Machine.IS_SDK_ABOVE_KITKAT) {
			return sRealHeightPixels;
		}
		return sHeightPixels;
	}
	

	/**
	 * 虚拟键在下面时
	 * @return
	 */
	public static int getNavBarHeight() {
		if (Machine.IS_SDK_ABOVE_KITKAT && Machine.canHideNavBar()) {
			return sNavBarHeight;
		}
		return 0;
	}
	
	/**
	 * 横屏，虚拟键在右边时
	 * @return
	 */
	public static int getNavBarWidth() {
		if (Machine.IS_SDK_ABOVE_KITKAT && Machine.canHideNavBar()) {
			return sNavBarWidth;
		}
		return 0;
	}
	
	/**
	 * 判断虚拟导航键是否存在
	 * 需要注意存在用户用root的方式隐藏了虚拟导航键，然后用手势返回或呼出home的情况
	 * 这时候sRealHeightPixels = sHeightPixels，sRealWidthPixels = sWidthPixels
	 * @return
	 */
	public static boolean isNavBarAvailable() {
		if (sRealHeightPixels > sHeightPixels || sRealWidthPixels > sWidthPixels) {
			return true;
		}
		return false;
	}
	
	@SuppressLint("NewApi")
	public static void setAlpha(View view, float alpha) {
		if (Build.VERSION.SDK_INT >= 11) { 
			view.setAlpha(alpha);
		} else {
			view.getBackground().setAlpha((int) (alpha * 255));
		}
	}
	public static boolean isLandScreen() {
		if (sRealHeightPixels > sRealWidthPixels) {
			return true;
		}
		return false;
	}
	
}
