package com.jb.ga0.commerce.util;

import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.telephony.TelephonyManager;
import android.text.TextUtils;

import org.apache.http.HttpHost;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.CoreConnectionPNames;
import org.apache.http.protocol.BasicHttpContext;
import org.apache.http.protocol.ExecutionContext;
import org.apache.http.protocol.HttpContext;

import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;

/**
 * 网络工具类
 *
 * @author matt
 * @date: 2015年1月28日
 *
 */
public class NetUtil {
	/**
	 * 检测网络是否可用
	 * @param context
	 * @return
	 */
	public static boolean isNetWorkAvailable(Context context) {
		boolean result = false;
		if (context != null) {
			ConnectivityManager connectivityManager = (ConnectivityManager) context
					.getSystemService(Context.CONNECTIVITY_SERVICE);
			if (connectivityManager != null) {
				NetworkInfo networkInfo = null;
				/**
				 * 防止这个crash
				 * Caused by: android.os.DeadSystemException
				 ... 11 more
				 android.os.DeadSystemException
				 at android.net.ConnectivityManager.getActiveNetworkInfo(ConnectivityManager.java:876)

				 */
				try {
					networkInfo = connectivityManager.getActiveNetworkInfo();
				} catch (Exception e) {
					e.printStackTrace();
				}
				if (networkInfo != null && networkInfo.isAvailable()) {
					result = true;
				}
			}
		}
		return result;
	}
	
	/**
	 * 检测WIFI是否可用
	 * @param context
	 * @return
	 */
	public static boolean isWifiEnable(Context context) {
		boolean result = false;
		if (context != null) {
			ConnectivityManager connectivityManager = (ConnectivityManager) context
					.getSystemService(Context.CONNECTIVITY_SERVICE);
			if (connectivityManager != null) {
				NetworkInfo networkInfo = null;
				try {
					networkInfo = connectivityManager.getActiveNetworkInfo();
				} catch (Exception e) {
					e.printStackTrace();
				}
				if (networkInfo != null && networkInfo.isConnected()
						&& networkInfo.getType() == ConnectivityManager.TYPE_WIFI) {
					result = true;
				}
			}
		}
		return result;
	}
	
	/**
	 * 网络类型定义
	 *
	 * @author matt
	 * @date: 2015年1月30日
	 *
	 */
	public static enum NetworkType {
		unknown,
		wifi,
		mobile2g,
		mobile3g4g,
	}
	/**
	 * 获取当前网络状态，wifi，GPRS，3G，4G
	 * 
	 * @param context
	 * @return
	 */
	public static NetworkType getNetworkType(Context context) {
		NetworkType ret = NetworkType.unknown;
		try {
			ConnectivityManager manager = (ConnectivityManager) context
					.getSystemService(Context.CONNECTIVITY_SERVICE);
			NetworkInfo networkinfo = manager.getActiveNetworkInfo();
			if (networkinfo.getType() == ConnectivityManager.TYPE_WIFI) {
				ret = NetworkType.wifi;
			} else if (networkinfo.getType() == ConnectivityManager.TYPE_MOBILE) {
				int subtype = networkinfo.getSubtype();
				switch (subtype) {
				case TelephonyManager.NETWORK_TYPE_1xRTT:
				case TelephonyManager.NETWORK_TYPE_CDMA:
				case TelephonyManager.NETWORK_TYPE_EDGE:
				case TelephonyManager.NETWORK_TYPE_GPRS:
				case TelephonyManager.NETWORK_TYPE_IDEN:
					// 2G
					ret = NetworkType.mobile2g;
					break;
				case TelephonyManager.NETWORK_TYPE_EVDO_0:
				case TelephonyManager.NETWORK_TYPE_EVDO_A:
				case TelephonyManager.NETWORK_TYPE_HSDPA:
				case TelephonyManager.NETWORK_TYPE_HSPA:
				case TelephonyManager.NETWORK_TYPE_HSUPA:
				case TelephonyManager.NETWORK_TYPE_UMTS:
					// 3G,4G
					ret = NetworkType.mobile3g4g;
					break;
				case TelephonyManager.NETWORK_TYPE_UNKNOWN:
				default:
					// unknown
					break;
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return ret;
	}
	
	/**
	 * 获取本地IP函数
	 * @return
	 */
	public static String getLocalIPAddress() {
		try {
			for (Enumeration<NetworkInterface> mEnumeration = NetworkInterface
					.getNetworkInterfaces(); mEnumeration.hasMoreElements();) {
				NetworkInterface intf = mEnumeration.nextElement();
				for (Enumeration<InetAddress> enumIPAddr = intf.getInetAddresses(); enumIPAddr
						.hasMoreElements();) {
					InetAddress inetAddress = enumIPAddr.nextElement();
					//如果不是回环地址
					if (!inetAddress.isLoopbackAddress()) {
						//直接返回本地IP地址
						return inetAddress.getHostAddress();
					}
				}
			}
		} catch (SocketException ex) {
		}
		return null;
	}
	
	/** 获取重定向之后的网址信息
	 * HttpClient缺省会自动处理客户端重定向
	 * 即访问网页A后,假设被重定向到了B网页,那么HttpClient将自动返回B网页的内容
	 * 若想取得B网页的地址,就需要借助HttpContext对象,HttpContext实际上是客户端用来在多次请求响应的交互中,保持状态信息的
	 * 可以利用HttpContext来存放一些我们需要的信息,以便下次请求的时候能够取出这些信息来使用
	*/
	public static String getHttpRedirectUrl(String url) {
		String resultUrl = null;
		DefaultHttpClient httpClient = new DefaultHttpClient();
		HttpContext httpContext = new BasicHttpContext();
		HttpGet httpGet = new HttpGet(url);
		try {
			// 设置超时的时间为30秒
			httpClient.getParams().setParameter(
					CoreConnectionPNames.CONNECTION_TIMEOUT, 30000);
			// 读取数据超时时间30秒
			httpClient.getParams().setParameter(
					CoreConnectionPNames.SO_TIMEOUT, 30000);
			// 将HttpContext对象作为参数传给execute()方法,则HttpClient会把请求响应交互过程中的状态信息存储在HttpContext中
			httpClient.execute(httpGet, httpContext);
			// 获取重定向之后的主机地址信息
			HttpHost targetHost = (HttpHost) httpContext
					.getAttribute(ExecutionContext.HTTP_TARGET_HOST);
			// 获取实际的请求对象的URI
			HttpUriRequest realRequest = (HttpUriRequest) httpContext
					.getAttribute(ExecutionContext.HTTP_REQUEST);
			// host + request 就是最终需要的跳转地址
			String targetHostUrl = targetHost.toURI().toString();
			if (TextUtils.isEmpty(targetHostUrl)) {
				return null;
			}
			String realRequestUrl = realRequest.getURI().toString();
			if (TextUtils.isEmpty(realRequestUrl)) {
				return null;
			}
			resultUrl = targetHostUrl + realRequestUrl;
		} catch (Exception e) {
			e.printStackTrace();
		} catch (OutOfMemoryError e) {
			return null;
		} finally {
			httpClient.getConnectionManager().shutdown();
		}
		return resultUrl;
	}
	
}
