package com.jb.ga0.commerce.util;

import android.util.Log;

/**
 * 计时日志打印，用于查看代码执行耗时
 *
 * @author matt
 * @date: 2015年4月11日
 *
 */
public class PassTimeLogger {
	//开始计时点
	private long mLogStartTime;
	//累计耗时
	private long mTotalTime = 0;
	//累计计时次数
	private int mAddCount = 0;

	@Deprecated
	public PassTimeLogger() {
	}

	/**
	 * 重新开始计时
	 */
	public void resetStartTime() {
		mLogStartTime = System.currentTimeMillis();
	}

	/**
	 * 终止计时并打印耗时，用于一次性计时
	 * @param taskName
	 */
	public void logEndTime(String tag, String taskName) {
		log(tag, String.format("[%s] 耗时：%.3fs", taskName,
				(System.currentTimeMillis() - mLogStartTime) / 1000.0f));
		resetStartTime();
	}

	/**
	 * 终止计时，用于累计计时
	 */
	public void endTime() {
		mTotalTime += System.currentTimeMillis() - mLogStartTime;
		mAddCount++;
	}

	/**
	 * 打印耗时，用于累计计时
	 * @param taskName
	 */
	public void logTotalTime(String tag, String taskName) {
		log(tag, String.format("[%s] 耗时：%.3fs, 执行次数:%d", taskName,
				mTotalTime / 1000.0f, mAddCount));
		resetStartTime();
	}

	/**
	 * 打印信息，用于说明(正在做什么事)
	 * @param content
	 */
	private void log(String tag, String content) {
		Log.d(tag, content);
	}
}
