package com.jb.ga0.commerce.util.encrypt;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

import com.jb.ga0.commerce.util.io.FileUtil;

/**
 * 文件加密器
 *
 * @author matt
 * @date: 2015年1月15日
 *
 */
public class FileCryptor {
	//加密字节数
	private static final int ENCRYPT_BYTE_NUM = 20;
	
	private IEncrypt mEncrypt;
	
	public FileCryptor() {
		mEncrypt = new XORCrypt();
	}
	
	public FileCryptor(IEncrypt encrypt) {
		mEncrypt = encrypt;
	}
	
	public void setEncrypt(IEncrypt encrypt) {
		mEncrypt = encrypt;
	}
	
	public void encrypt(String filePath) {
		if (!FileUtil.isFileExist(filePath) || new File(filePath).length() < ENCRYPT_BYTE_NUM) {
			return;
		}
		
		RandomAccessFile randownFile = null;
		try {
			randownFile = new RandomAccessFile(filePath, "rw");

			byte[] bytes = new byte[ENCRYPT_BYTE_NUM];
			randownFile.seek(0);
			randownFile.read(bytes);

			bytes = mEncrypt.encrypt(bytes);
			randownFile.seek(0);
			randownFile.write(bytes);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if (randownFile != null) {
				try {
					randownFile.close();
				} catch (IOException e) {
				}
				randownFile = null;
			}
		}
	}

	public void decrypt(String filePath) {
		if (!FileUtil.isFileExist(filePath) || new File(filePath).length() < ENCRYPT_BYTE_NUM) {
			return;
		}
		
		RandomAccessFile randownFile = null;
		try {
			randownFile = new RandomAccessFile(filePath, "rw");

			byte[] bytes = new byte[ENCRYPT_BYTE_NUM];
			randownFile.seek(0);
			randownFile.read(bytes);

			bytes = mEncrypt.decrypt(bytes);
			randownFile.seek(0);
			randownFile.write(bytes);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if (randownFile != null) {
				try {
					randownFile.close();
				} catch (IOException e) {
				}
				randownFile = null;
			}
		}
	}
}
