package com.jb.ga0.commerce.util.http;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.json.JSONException;
import org.json.JSONObject;

import com.gau.utils.net.operator.IHttpOperator;
import com.gau.utils.net.request.THttpRequest;
import com.gau.utils.net.response.BasicResponse;
import com.gau.utils.net.response.IResponse;
import com.jb.ga0.commerce.util.io.StringUtils;
import com.jb.ga0.commerce.util.zip.ZipUtils;

/**
 * 网络请求Json数据操作类
 * @author caoyaming
 *
 */
public class AdvertJsonOperator implements IHttpOperator {
	//是否需要解压数据
	private boolean mIsZipData = false;
	private boolean mIsDecode = false;
	
	public AdvertJsonOperator() {
		
	}
	
	public AdvertJsonOperator(boolean isZipData) {
		mIsZipData = isZipData;
	}
	/**
	 * @param isZipData 是否需要解压数据
	 */
	public AdvertJsonOperator(boolean isZipData, boolean isDecode) {
		mIsZipData = isZipData;
		mIsDecode = isDecode;
	}
	@Override
	public IResponse operateHttpResponse(THttpRequest request, HttpResponse response) throws IllegalStateException, IOException {
		HttpEntity entity = response.getEntity();
		InputStream is = entity.getContent();
		JSONObject json = parseData(is, mIsZipData, mIsDecode);
		BasicResponse iResponse = new BasicResponse(IResponse.RESPONSE_TYPE_JSONOBJECT, json);
		return iResponse;
	}
	
	public static JSONObject parseData(final InputStream in, final boolean isZipData, final boolean isDecode) {
		if (in == null) {
			return null;
		}
		try {
			String jsonString = null;
			if (isZipData) {
				jsonString = ZipUtils.unzip(in);
			} else {
				ByteArrayOutputStream buffer = new ByteArrayOutputStream();
				byte[] buff = new byte[1024];
				int len = -1;
				try {
					while ((len = in.read(buff)) != -1) {
						buffer.write(buff, 0, len);
					}
				} catch (IOException e) {
					e.printStackTrace();
					return null;
				}
				jsonString = new String(buffer.toByteArray());
			}
			if (jsonString != null) {
				if (isDecode) {
					try {
						jsonString = StringUtils.encode(jsonString);
					} catch (Exception e) {					
						e.printStackTrace();
					}
				}
			
				return new JSONObject(jsonString);
			}
		} catch (JSONException e) {
			e.printStackTrace();
		} finally {
			try {
				if (in != null) {
					in.close();
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return null;
	}
}
