package com.jb.ga0.commerce.util.http;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.SharedPreferences;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Build;
import android.os.Environment;
import android.os.SystemClock;
import android.provider.Settings;
import android.telephony.TelephonyManager;
import android.text.TextUtils;
import android.util.DisplayMetrics;
import android.view.WindowManager;

import com.gau.go.gostaticsdk.StatisticsManager;
import com.jb.ga0.commerce.util.AppUtils;
import com.jb.ga0.commerce.util.GoogleMarketUtils;
import com.jb.ga0.commerce.util.Machine;
import com.jb.ga0.commerce.util.io.FileUtil;

import java.util.Locale;
import java.util.Random;


/**
 * 用于获取http协议头phead常用的参数
 *
 * @author matt
 * @date: 2015年11月17日
 *
 */
public class GoHttpHeadUtil {

	private static String sAndroidId;
	/**
	 * <br>功能简述:获取Android ID的方法
	 * <br>功能详细描述:
	 * <br>注意:
	 * @return
	 */
	public static String getAndroidId(Context context) {
		if (sAndroidId == null && context != null) {
			sAndroidId = Settings.Secure.getString(context.getContentResolver(),
					Settings.Secure.ANDROID_ID);
		}
		return sAndroidId;
	}
	
	/**
	 * 采用随机数的形式模拟IMEI号,避免申请权限.
	 * 
	 * @author huyong
	 * @param context
	 * @return
	 */
	public static String getVirtualIMEI(Context context) {
		/**
		 * 内部类
		 */
		class VirtualImeiGenerator {
			//存入sharedPreference中的key
			private final static String RANDOM_DEVICE_ID = "random_device_id"; 
			//默认随机IMEI
			private final static String DEFAULT_RANDOM_DEVICE_ID = "0000000000000000"; 
			private final static String DEVICE_ID_SP_FILE = "commerce_device_id";
			private final String mDeviceIdSdcardFile = Environment.getExternalStorageDirectory().getPath() + "/commerce/statistics/deviceId" + ".txt";
			private String mDeviceId;

			/**
			 * 采用随机数的形式模拟imei号，避免申请权限。
			 * 
			 * @author huyong
			 * @param context
			 * @return
			 */
			public String getVirtualIMEI(Context context) {
				if (mDeviceId != null) {
					return mDeviceId;
				}
				String deviceidString = getDeviceIdFromSharedpreference(context);
				// Sharedpreference中没有找到，就从SDcard获取，如果还没有则自动生成一个，并保存下来
				if (deviceidString != null && deviceidString.equals(DEFAULT_RANDOM_DEVICE_ID)) {
					deviceidString = getDeviceIdFromSDcard();
					// 如果SD卡上拿到数据的话就视为旧用户，保存到Sharedpreference,如果拿不到数据就是新用户，随机生成。
					try {
						if (deviceidString == null) {
							long randomDeviceid = SystemClock.elapsedRealtime();
							// 获取随机数，并保存至sharedpreference
							Random rand = new Random();
							long randomLong = rand.nextLong();
							while (randomLong == Long.MIN_VALUE) {
								randomLong = rand.nextLong();
							}
							randomDeviceid += Math.abs(randomLong);
							deviceidString = String.valueOf(randomDeviceid);
							rand = null;

							saveDeviceIdToSDcard(deviceidString);
						} else {
							//TODO 用户之前已经使用过GO桌面的标志。
							//StatisticsData.saveIsOldUser(context);
						}
						saveDeviceIdToSharedpreference(context, deviceidString);
					} catch (Exception e) {
						e.printStackTrace();
					}
					//Sharedpreference找到了deviceID但是SDcard上没有，就保存到SD卡上。
				} else if (getDeviceIdFromSDcard() == null) {
					saveDeviceIdToSDcard(deviceidString);
				}
				mDeviceId = deviceidString;
				return deviceidString;
			}

			/**
			 * 从sdcard获取随机生成的IMEI的方法
			 * @return
			 */
			private String getDeviceIdFromSDcard() {
				return getStringFromSDcard(mDeviceIdSdcardFile);
			}
			
			private String getStringFromSDcard(String filePath) {
				String string = null;
				try {
					//判断sd卡是否存在
					if (Machine.isSDCardExist()) {
						byte[] bs = FileUtil.readByteFromSDFile(filePath);
						string = new String(bs);
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
				return string;
			}

			/**
			 * 保存随机生成的IMEI到sdcard上的方法
			 * @param deviceId
			 */
			private void saveDeviceIdToSDcard(String deviceId) {
				writeToSDCard(deviceId, mDeviceIdSdcardFile);
			}
			
			/**
			 * 保存deviceId到Sharedpreference
			 * @param context
			 * @param deviceId
			 */
			@SuppressLint("InlinedApi")
			private void saveDeviceIdToSharedpreference(Context context, String deviceId) {
				SharedPreferences sp;
				if (Build.VERSION.SDK_INT >= 11) {
					sp = context.getSharedPreferences(DEVICE_ID_SP_FILE, Context.MODE_MULTI_PROCESS);
				} else {
					sp = context.getSharedPreferences(DEVICE_ID_SP_FILE, Context.MODE_PRIVATE);
				}		
				sp.edit().putString(RANDOM_DEVICE_ID, deviceId).commit();
			}

			/**
			 * 从Sharedpreference获取上次保存的主题包名
			 * 
			 * @author huyong
			 * @return
			 */
			@SuppressLint("InlinedApi")
			private String getDeviceIdFromSharedpreference(Context context) {
				SharedPreferences sp;
				if (Build.VERSION.SDK_INT >= 11) {
					sp = context.getSharedPreferences(DEVICE_ID_SP_FILE, Context.MODE_MULTI_PROCESS);
				} else {
					sp = context.getSharedPreferences(DEVICE_ID_SP_FILE, Context.MODE_PRIVATE);
				}	
				return sp.getString(RANDOM_DEVICE_ID, DEFAULT_RANDOM_DEVICE_ID);
			}

			/**
			 * 把字符串写到SD卡的方法
			 * 
			 * @param data
			 */
			private void writeToSDCard(String data, String filePath) {
				if (data != null) {
					try {
						//判断sd卡是否存在
						if (Machine.isSDCardExist()) {
							FileUtil.saveByteToSDFile(data.getBytes(), filePath);
						}
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}
		}
		return new VirtualImeiGenerator().getVirtualIMEI(context);
	}
	
	private static int sVersionCode = -1;
	public static int getVersionCode(Context context) {
		if (sVersionCode == -1) {
			sVersionCode = AppUtils.getAppVersionCode(context);
		}
		return sVersionCode;
	}
	
	private static String sVersionName = null;
	/**
	 * 获取桌面版本号
	 * 
	 * @author xiedezhi
	 * 
	 * @param context
	 * @return
	 */
	public static String getVersionName(Context context) {
		if (sVersionName == null) {
			sVersionName = AppUtils.getAppVersionName(context);
		}
		return sVersionName;
	}
	
	/**
	 * 从res/raw/uid.txt文件中获取渠道id
	 * 
	 * @author huyong
	 * @param context
	 * @return
	 */
	public static String getUid(Context context) {
		return "200"; // uid默认为200
	}
	
	/**
	 * 获取国家，返回结果为大写字母
	 * 
	 * @param context
	 * @return 
	 * @备注 先获取sim卡的国家，如果没有sim卡，取本地语言代表的国家
	 */
	public static String getCountry(Context context) {
		String ret = null;
		try {
			TelephonyManager telManager = (TelephonyManager) context
					.getSystemService(Context.TELEPHONY_SERVICE);
			if (telManager != null) {
				ret = telManager.getSimCountryIso().toUpperCase();
			}
		} catch (Throwable e) {
		}
		if (TextUtils.isEmpty(ret)) {
			ret = Locale.getDefault().getCountry().toUpperCase();
		}

		return TextUtils.isEmpty(ret) ? "ZZ" : ret;
	}

	/**
	 * 获取国家，返回结果为大写字母
	 * 
	 * @param context
	 * @return 
	 * @备注 先获取sim卡的国家，如果没有sim卡，取本地语言代表的国家
	 */
	public static String getLocal(Context context) {
		return getCountry(context);
	}
	
	/**
	 * 获取语言，返回小写字母
	 * 
	 * @return
	 */
	public static String getLanguage(Context context) {
		return Locale.getDefault().getLanguage().toLowerCase();
	}
	
	/**
	 * 获取用户运营商代码
	 * @param context
	 * @return
	 */
	public static String getImsi(Context context) {
		String simOperator = "000";
		try {
			if (context != null) {
				// 从系统服务上获取了当前网络的MCC(移动国家号)，进而确定所处的国家和地区
				TelephonyManager manager = (TelephonyManager) context
						.getSystemService(Context.TELEPHONY_SERVICE);
				simOperator = manager.getSimOperator();
			}
		} catch (Throwable e) {
			// TODO: handle exception
		}
		
		//当客户端运营商编码不存在时，simOperator可能被设置为空，服务器端不会受理
		//故如果simOperator字符串为空，则置为“000”
		return TextUtils.isEmpty(simOperator) ? "000" : simOperator;
	}
	/**
	 * 获取用户运营商代码
	 * @param context
	 * @return
	 */
	public static String getCnUser(Context context) {
		return getImsi(context);
	}
	
	/**
	 * 获取设备宽高320*480
	 * 
	 * @param context
	 * @return
	 */
	public static String getDeviceDIPString(Context context) {
		WindowManager wMgr = (WindowManager) context
				.getSystemService(Context.WINDOW_SERVICE);
		return wMgr.getDefaultDisplay().getWidth() + "*"
				+ wMgr.getDefaultDisplay().getHeight();
	}
	
	/**
	 * 获取设备DIP
	 * @param context
	 * @return
	 */
	public static int getDeviceDIP(Context context) {
		DisplayMetrics dm = new DisplayMetrics();
		WindowManager wMgr = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
		wMgr.getDefaultDisplay().getMetrics(dm);
		return dm != null ? dm.densityDpi : 0;
	}
	
	/**
	 * 是否存在google电子市场
	 * @param context
	 * @return
	 */
	public static boolean isExistGoogleMarket(Context context) {
		return GoogleMarketUtils.isMarketExist(context);
	}
	
	/**
	 * 获取当前网络类型，wifi，GPRS，3G，4G
	 * 
	 * @param context
	 * @return
	 */
	public static String getNetType(Context context) {
		// build Network conditions
		String ret = "";
		try {
			ConnectivityManager manager = (ConnectivityManager) context
					.getSystemService(Context.CONNECTIVITY_SERVICE);
			NetworkInfo networkinfo = manager.getActiveNetworkInfo();
			if (networkinfo != null && networkinfo.getType() == ConnectivityManager.TYPE_WIFI) {
				ret = "WIFI";
			} else if (networkinfo != null && networkinfo.getType() == ConnectivityManager.TYPE_MOBILE) {
				int subtype = networkinfo.getSubtype();
				switch (subtype) {
					case TelephonyManager.NETWORK_TYPE_1xRTT :
					case TelephonyManager.NETWORK_TYPE_CDMA :
					case TelephonyManager.NETWORK_TYPE_EDGE :
					case TelephonyManager.NETWORK_TYPE_GPRS :
					case TelephonyManager.NETWORK_TYPE_IDEN :
						// 2G
						ret = "2G" /*+ "(typeid = " + networkinfo.getType() + "  typename = "
									+ networkinfo.getTypeName() + "  subtypeid = "
									+ networkinfo.getSubtype() + "  subtypename = "
									+ networkinfo.getSubtypeName() + ")"*/;
						break;
					case TelephonyManager.NETWORK_TYPE_EVDO_0 :
					case TelephonyManager.NETWORK_TYPE_EVDO_A :
					case TelephonyManager.NETWORK_TYPE_HSDPA :
					case TelephonyManager.NETWORK_TYPE_HSPA :
					case TelephonyManager.NETWORK_TYPE_HSUPA :
					case TelephonyManager.NETWORK_TYPE_UMTS :
						// 3G,4G
						ret = "3G/4G" /*+ "(typeid = " + networkinfo.getType() + "  typename = "
										+ networkinfo.getTypeName() + "  subtypeid = "
										+ networkinfo.getSubtype() + "  subtypename = "
										+ networkinfo.getSubtypeName() + ")"*/;
						break;
					case TelephonyManager.NETWORK_TYPE_UNKNOWN :
					default :
						// unknow
						ret = "UNKNOW" /*+ "(typeid = " + networkinfo.getType() + "  typename = "
										+ networkinfo.getTypeName() + "  subtypeid = "
										+ networkinfo.getSubtype() + "  subtypename = "
										+ networkinfo.getSubtypeName() + ")"*/;
						break;
				}
			} else {
				ret = "UNKNOW" /*+ "(typeid = " + networkinfo.getType() + "  typename = "
								+ networkinfo.getTypeName() + ")"*/;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return ret;
	}
	
	public static String getGoId(Context context) {
		return StatisticsManager.getGOID(context);
	}
}
