package com.jb.ga0.commerce.util.http;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.zip.GZIPInputStream;

import org.apache.http.HttpResponse;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * HTTP公共方法
 * @author matt
 *
 */
public class HttpUtil {
	/**
	 * 将键值对转换成url,形式 "x1=y1&x2=y2...xn=yn"
	 * @param params
	 * @return
	 * @备注： 用于构建httpget的url
	 */
	public static String convert2Url(Map<String, String> params) {
		StringBuffer sb = new StringBuffer();

		boolean isAtFirst = true;
		Set<Map.Entry<String, String>> set = params.entrySet();
		for (Iterator<Map.Entry<String, String>> it = set.iterator(); it.hasNext(); ) {
			Map.Entry<String, String> entry = it.next();

			if (!isAtFirst) {
				sb.append("&");
			} else {
				isAtFirst = false;
			}
			sb.append(entry.getKey()).append("=").append(entry.getValue());
		}

		return sb.toString();
	}

	/**
	 * 将键值对添加进JSONObject
	 * @param params 入参
	 * @param jsonObj 出参，若为空，则new一个
	 * @return
	 */
	public static JSONObject putInJSONObject(Map<String, Object> params, JSONObject jsonObj) {
		if (null == jsonObj) {
			jsonObj = new JSONObject();
		}

		if (params != null) {
			for (Map.Entry<String, Object> entry : params.entrySet()) {
				try {
					jsonObj.put(entry.getKey(), entry.getValue());
				} catch (JSONException e) {
					e.printStackTrace();
				}
			}
		}

		return jsonObj;
	}

	/**
	 * gzip压缩json，输出字符串
	 * @param json
	 * @return
	 */
	public static String gzip(JSONObject json) {
		if (null == json) {
			return null;
		}
		return json.toString();
//		String zipped = null;
//		try {
//			ZipCompress compress = new ZipCompress();
//			byte[] data = compress.compress(json.toString().getBytes(
//					"UTF-8"));
//			zipped = new String(data, "ISO-8859-1");
//		} catch (UnsupportedEncodingException e) {
//			e.printStackTrace();
//		}
//
//		return zipped;
	}
	
	public static String getContentString(HttpResponse response, boolean isDataGzipped) {
		if (null == response || response.getStatusLine().getStatusCode() != HttpURLConnection.HTTP_OK) {
			return null;
		}
		
		try {
			return toString(response.getEntity().getContent(), isDataGzipped);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
	 /**
	  * 从HttpResponse获取json对象
	  * @param response
	  * @param isDataGzipped 数据是否被压缩
	  * @return
	  */
	public static JSONObject getJsonObject(HttpResponse response, boolean isDataGzipped) {
		try {
			String jsonString = getContentString(response, isDataGzipped);
			return new JSONObject(jsonString);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
	private static String toString(InputStream inStream, boolean isDataGzipped) {
		try {
			byte[] bytes = toByteArray(inStream);
			if (isDataGzipped) {
				bytes = ungzip(bytes);
			}
			return new String(bytes, "utf-8");
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return null;
	}
	
	/**
	 * 从inputStream读数据
	 * @param input
	 * @return
	 * @throws IOException
	 */
	private static byte[] toByteArray(InputStream input) throws IOException {
		ByteArrayOutputStream output = new ByteArrayOutputStream();
		copy(input, output);
		return output.toByteArray();
	}
	
	/**
	 * 解压
	 * @param bs
	 * @return
	 * @throws Exception
	 */
	private static byte[] ungzip(byte[] bs) throws Exception {
		GZIPInputStream gzin = null;
		ByteArrayInputStream bin = null;
		try {
			bin = new ByteArrayInputStream(bs);
			gzin = new GZIPInputStream(bin);
			return toByteArray(gzin);
		} catch (Exception e) {
			throw e;
		} finally {
			if (bin != null) {
				bin.close();
			}
			if (gzin != null) {
				gzin.close();
			}
		}
	}
	
	/**
	 * 将inputStream的数据拷贝到outputStream
	 * @param input
	 * @param output
	 * @return
	 * @throws IOException
	 */
	private static int copy(InputStream input, OutputStream output)
			throws IOException {
		byte[] buffer = new byte[1024 * 4];
		int count = 0;
		int n = 0;
		while (-1 != (n = input.read(buffer))) {
			output.write(buffer, 0, n);
			count += n;
		}
		return count;
	}
	
	/**
	 * JSONArray转换为String[]，并返回
	 * @param jsonArray
	 * @return
	 */
	public static String[] jsonArray2StringArray(JSONArray jsonArray) {
		if (jsonArray == null || jsonArray.length() <= 0) {
			return null;
		}
		String[] ret = new String[jsonArray.length()];
		for (int i = 0; i < jsonArray.length(); ++i) {
			ret[i] = jsonArray.optString(i);
		}
		return ret;
	}
	
	/**
	 * String[]转换为JSONArray，并返回
	 * @param strArray
	 * @return
	 */
	public static JSONArray stringArray2JSONArray(String[] strArray) {
		JSONArray jsonArray = new JSONArray();
		
		if (strArray != null && strArray.length > 0) {
			for (String attachment : strArray) {
				jsonArray.put(attachment);
			}
		}
		
		return jsonArray;
	}
}
