package com.jb.ga0.commerce.util.io;

import java.util.ArrayList;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteQueryBuilder;

import com.jb.ga0.commerce.util.LogUtils;

/**
 * 数据库具体操作
 * @author matt
 *
 */
public abstract class DataBaseHelper extends SQLiteOpenHelper {
	public final static String TYPE_NUMERIC = "NUMERIC";
	public final static String TYPE_INTEGER = "INTEGER";
	public final static String TYPE_TEXT = "TEXT";
	public final static String NOT_NULL = "NOT NULL";
	public final static String DEFAULT = "DEFAULT";
	public final static String PRIMARY_KEY = "PRIMARY KEY";
	public final static String AUTOINCREMENT = "AUTOINCREMENT";
	public final static String IF_NOT_EXISTS = "IF NOT EXISTS";
	public final static String FOR_EACH_ROW = "FOR EACH ROW";
	public final static String UNIQUE = "UNIQUE";
	public final static String INSERT_INTO = "INSERT INTO";
	public final static String INSERT_OR_REPLACE_INTO = "INSERT OR REPLACE INTO";
	public final static String VALUES = "VALUES";
	public final static String INNER_JOIN = "INNER JOIN";
	
    private final static int DB_VERSION_ONE = 1;

    private final Context mContext;
    // just for test
    private boolean mIsNewDB = false;
    // 可以执行多表关联查询
    private SQLiteQueryBuilder msqlQB = null;
    // 更新数据库结果，默认是成功的
    private boolean mUpdateResult = true; 

    public DataBaseHelper(Context context, String dataBaseName, int dataBaseVersion) {
        super(context, dataBaseName, null, dataBaseVersion);
        mContext = context;
        // just for test
        msqlQB = new SQLiteQueryBuilder();
        SQLiteDatabase db = null;
        try {
            db = getWritableDatabase();
            if (!mUpdateResult) {
                // 更新失败，则删除数据库，再行创建。
                if (db != null) {
                    db.close();
                }
                mContext.deleteDatabase(getDbName());
                // mContext.openOrCreateDatabase(DATABASE_NAME,
                // Context.MODE_PRIVATE, null);
                getWritableDatabase();
            }
        } catch (SQLiteException ex) {
            // add begin 2011-06-30 rongjinsong
//            PackageManager pm = context.getPackageManager();
//            pm.clearPackagePreferredActivities(context.getPackageName());
//            int icon = android.R.drawable.stat_notify_error;
//            Uri packageUri = Uri.parse("package:" + context.getPackageName());
//            CharSequence tickerText = mContext.getText(R.string.db_dataException);
//            CharSequence contentTitle = mContext.getText(R.string.crash_notif_title);
//            CharSequence contentText = mContext.getText(R.string.crash_notif_uninstall_text);
//            Intent intent = new Intent(Intent.ACTION_DELETE, packageUri);
//            AppUtils.sendNotification(mContext, intent, icon, tickerText, contentTitle,
//                    contentText, INotificationId.GOTO_DB_EXCEPTION);
            mContext.deleteDatabase(getDbName());
            // add end 2011-06-30
        } catch (IllegalStateException ie) {
//            GoLauncher.sendMessage(this, IDiyFrameIds.SCHEDULE_FRAME,
//                    IDiyMsgIds.RESTART_GOLAUNCHER, -1, null, null);
        }
    }

    @Override
    public void onCreate(SQLiteDatabase db) {
        mUpdateResult = true;
        mIsNewDB = true;

        db.beginTransaction();
        try { //数据库新建，创建所有表
            onCreateTables(db);

            db.setTransactionSuccessful();
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            db.endTransaction();
        }
    }

    /**
     * 检查指定的表是否存在
     * 
     * 
     * @param tableName
     * @return
     */
    public boolean isExistTable(final SQLiteDatabase db, String tableName) {
        boolean result = false;
        Cursor cursor = null;
        String where = "type='table' and name='" + tableName + "'";
        try {
            cursor = db.query("sqlite_master", null, where, null, null, null, null);
            if (cursor != null && cursor.moveToFirst()) {
                result = true;
            }
        } catch (SQLiteException e) {
            e.printStackTrace();
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }
        return result;
    }

    /**
     * 检查表中是否存在该字段
     * 
     * 
     * @param db
     * @param tableName
     * @param columnName
     * @return
     */
    private boolean isExistColumnInTable(SQLiteDatabase db, String tableName, String columnName) {
        boolean result = false;
        Cursor cursor = null;
        try {
            // 查询列数
            String columns[] = { columnName };
            cursor = db.query(tableName, columns, null, null, null, null, null);
            if (cursor != null && cursor.getColumnIndex(columnName) >= 0) {
                result = true;
            }
        } catch (Exception e) {
            // e.printStackTrace();
            LogUtils.i("DatabaseHelper", "isExistColumnInTable has exception");
            result = false;
        } finally {
            if (cursor != null) {
                cursor.close();
            }
        }

        return result;
    }

    /**
     * 新添加字段到表中
     * 
     * 
     * @param db
     * @param tableName
     *            : 修改表名
     * @param columnName
     *            ：新增字段名
     * @param columnType
     *            ：新增字段类型
     * @param defaultValue
     *            ：新增字段默认值。为null，则不提供默认值
     */
    public void addColumnToTable(SQLiteDatabase db, String tableName, String columnName,
            String columnType, String defaultValue) {
        if (!isExistColumnInTable(db, tableName, columnName)) {
            db.beginTransaction();
            try {
                // 增加字段
                String updateSql = "ALTER TABLE " + tableName + " ADD " + columnName + " "
                        + columnType;
                db.execSQL(updateSql);

                // 提供默认值
                if (defaultValue != null) {
                    if (columnType.equals(TYPE_TEXT)) {
                        // 如果是字符串类型，则需加单引号
                        defaultValue = "'" + defaultValue + "'";
                    }

                    updateSql = "update " + tableName + " set " + columnName + " = " + defaultValue;
                    db.execSQL(updateSql);
                }

                db.setTransactionSuccessful();
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                db.endTransaction();
            }
        }
    }

    // 只针对安卓3.0系统以上
    public void onDowngrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        // 默认支持向下兼容。（oldVersion = 2, newVersion = 1）
        // 后期在做版本降级处理时，在此可根据需要做相应处理
        LogUtils.i("DatabaseHelper", "onDowngrade oldVersion=" + oldVersion + ", newVersion="
                + newVersion);
        return;
    }

    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        // TODO:根据版本号，更新数据库表结构
        if (oldVersion < DB_VERSION_ONE || oldVersion > newVersion || newVersion > getDbCurrentVersion()) {
            LogUtils.i("testDataBase", "onUpgrade() false oldVersion = " + oldVersion
                    + ", newVersion = " + newVersion);
            // note：由于新修改数据仅仅是增加字段，未删除或重命名以前数据库中表字段，因此不必重新生成数据库，直接用当下数据库即可
            // 后期，若需要修改老版数据库中字段或删除字段，则必须将该字段字段设为false.
            // 以上规则对现已发布1.50及以后的版本均有效。1.50-2.02对应数据库版本号为7th，2.10-2.12对应数据库版本号为8th，
            // 2.13对应数据库版本号为9th.
            // mUpdateResult = false;
            return;
        }
        ArrayList<UpgradeDB> upgradeDBFuncS = new ArrayList<UpgradeDB>();
        //这里添加升级节点
        onAddUpgrades(upgradeDBFuncS);
        for (int i = oldVersion - 1; i < newVersion - 1; i++) {
            mUpdateResult = upgradeDBFuncS.get(i).onUpgradeDB(db);
            if (!mUpdateResult) {
                // 中间有任何一次升级失败，则直接返回
                break;
            }
        }
        upgradeDBFuncS.clear();
    }
    
    /**
     * 执行原生sql语句
     * @param sql
     */
    public void execSql(String sql) {
        SQLiteDatabase db = getWritableDatabase();
        try {
            db.execSQL(sql);
        } catch (Exception e) {
            e.printStackTrace();
        } 
    }
 
    /**
     * 使用原生sql语句查询
     * @param sql
     * @param selectionArgs
     * @return
     */
    public Cursor rawQuery(String sql, String[] selectionArgs) {
        SQLiteDatabase db = getWritableDatabase();
        try {
            return db.rawQuery(sql, selectionArgs);
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        return null;
    }

    /**
     * 用于单表查询
     */
    public Cursor query(String tableName, String[] projection, String selection,
            String[] selectionArgs, String sortOrder) {

        return query(tableName, projection, selection, selectionArgs, null, null, sortOrder);
    }

    /**
     * 用于单表查询
     */
    public Cursor query(String tableName, String[] projection, String selection,
            String[] selectionArgs, String groupBy, String having, String sortOrder) {
        Cursor result = null;
        try {
            SQLiteDatabase db = getReadableDatabase();
            result = db.query(tableName, projection, selection, selectionArgs, groupBy, having,
                    sortOrder);
        } catch (SQLException e) {
            LogUtils.i("data", "SQLException when query in " + tableName + ", " + selection);
        } catch (IllegalStateException e) {
            LogUtils.i("data", "IllegalStateException when query in " + tableName + ", " + selection);
        }
        return result;
    }

    /**
     * 用于多表查询
     */
    public Cursor queryCrossTables(String tableName, String[] projection, String selection,
            String[] selectionArgs, String sortOrder) {
        Cursor result = null;
        synchronized (msqlQB) {
            msqlQB.setTables(tableName);
            try {
                SQLiteDatabase db = getReadableDatabase();
                result = msqlQB.query(db, projection, selection, selectionArgs, null, null,
                        sortOrder);
            } catch (SQLException e) {
                // e.printStackTrace();
                LogUtils.i("data", "SQLException when query in " + tableName + ", " + selection);
            } catch (IllegalStateException e) {
                // e.printStackTrace();
                LogUtils.i("data", "IllegalStateException when query in " + tableName + ", " + selection);
            }
        }
        return result;
    }

    public long insert(String tableName, ContentValues initialValues) throws DatabaseException {
        SQLiteDatabase db = getWritableDatabase();
        long rowId = 0;
        try {
            rowId = db.insert(tableName, null, initialValues);
        } catch (Exception e) {
            LogUtils.i("data", "Exception when insert in " + tableName);
            throw new DatabaseException(e);
        }
        return rowId;
    }

    public int delete(String tableName, String selection, String[] selectionArgs)
            throws DatabaseException {
        SQLiteDatabase db = getWritableDatabase();
        int count = 0;
        try {
            count = db.delete(tableName, selection, selectionArgs);
        } catch (Exception e) {
            LogUtils.i("data", "Exception when delete in " + tableName + ", " + selection);
            throw new DatabaseException(e);
        }
        return count;
    }

    public int update(String tableName, ContentValues values, String selection,
            String[] selectionArgs) throws DatabaseException {
        SQLiteDatabase db = getWritableDatabase();
        int count = 0;
        try {
            count = db.update(tableName, values, selection, selectionArgs);
        } catch (Exception e) {
            LogUtils.i("data", "Exception when update in " + tableName + ", " + selection);
            throw new DatabaseException(e);
        }
        return count;
    }
    
    public int updateOrInsert(String tableName, ContentValues values, String selection,
            String[] selectionArgs) throws DatabaseException {
        SQLiteDatabase db = getWritableDatabase();
        int count = 0;
        try {
            count = db.update(tableName, values, selection, selectionArgs);
            if (count <= 0) {
				db.insert(tableName, null, values);
			}
        } catch (Exception e) {
            LogUtils.i("data", "Exception when updateOrInsert in " + tableName + ", " + selection);
            throw new DatabaseException(e);
        }
        return count;
    }

    /**
     * 
     * @param sql
     * @throws DatabaseException
     */
    public void exec(String sql) throws DatabaseException {
        SQLiteDatabase db = getWritableDatabase();
        try {
            db.execSQL(sql);
        } catch (SQLException e) {
            LogUtils.i("data", "Exception when exec " + sql);
            throw new DatabaseException(e);
        }
    }

    public boolean isNewDB() {
        return mIsNewDB;
    }

    /**
     * <br>
     * 功能简述: <br>
     * 功能详细描述: <br>
     * 注意:
     */
    public boolean openDBWithWorldReadable() {
        try {
            close();
            if (mContext.openOrCreateDatabase(getDbName(), Context.MODE_WORLD_READABLE, null) == null) {
                return false;
            }
        } catch (Exception e) {
            // TODO: handle exception
            close();
            return false;
        }
        return true;
    }

    public void beginTransaction() {
        try {
            SQLiteDatabase db = getWritableDatabase();
            if (null != db) {
                db.beginTransaction();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void setTransactionSuccessful() {
        try {
            SQLiteDatabase db = getWritableDatabase();
            if (null != db) {
                db.setTransactionSuccessful();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void endTransaction() {
        try {
            SQLiteDatabase db = getWritableDatabase();
            db.endTransaction();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    //================升级相关=====BEGIN==================
    /**
     * 抽象升级类，只在每个连续的数据库版本间作升级
     * 
     * 
     * 
     */
    abstract public class UpgradeDB {
        abstract public boolean onUpgradeDB(SQLiteDatabase db);
    }
    //================升级相关=====END==================
    
    /**
     * 获取当前数据库版本
     */
    abstract public int getDbCurrentVersion(); 
    
    abstract public String getDbName();
    
    /**
     * 在这里创建所有表
     */
    abstract public void onCreateTables(SQLiteDatabase db);
    
    /**
     * 在这里添加所有升级节点
     * @param upgrades
     */
    abstract public void onAddUpgrades(ArrayList<UpgradeDB> upgrades);
}