package com.jb.ga0.commerce.util.io;

import android.app.Application;
import android.content.Context;
import android.content.res.Resources;
import android.content.res.XmlResourceParser;
import android.graphics.drawable.Drawable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;

import com.jb.ga0.commerce.util.LogUtils;

/**
 * @author fanwenda
 * 
 */
// CHECKSTYLE:OFF
public class ResourcesProvider {
	private static ResourcesProvider mInstance;
	private String mPkgName;;
	private Resources mResources;
	private LayoutInflater mInflater;

	public static synchronized void freeInstance() {
		if (null != mInstance) {
			mInstance = null;
		}
	}

	public static synchronized ResourcesProvider getInstance(Context context) {
		if (null == mInstance) {
			mInstance = new ResourcesProvider(context);
		}
		return mInstance;
	}

	private ResourcesProvider(Context context) {
		mPkgName = context.getPackageName();
		mResources = context.getResources();
		mInflater = LayoutInflater.from(context);
	}

	public int getId(String res) {
		int id = 0;
		if (id == 0) {
			id = mResources.getIdentifier(res, "id", mPkgName);
			if (id == 0) {
				LogUtils.e("ResourcesProvider", "id:" + res + " is not found");
			}
		}
		return id;
	}

	public int getLayoutId(String res) {
		int id = 0;
		if (id == 0) {
			id = mResources.getIdentifier(res, "layout", mPkgName);
			if (id == 0) {
				LogUtils.e("ResourcesProvider", "layout:" + res + " is not found");
			}
		}
		return id;
	}

	public int getDrawableId(String res) {
		int id = 0;
		if (id == 0) {
			id = mResources.getIdentifier(res, "drawable", mPkgName);
			if (id == 0) {
				LogUtils.e("ResourcesProvider", "drawable:" + res + " is not found");
			}
		}
		return id;
	}

	public int getColor(String res) {
		int id = mResources.getIdentifier(res, "color", mPkgName);
		if (id == 0) {
			LogUtils.e("ResourcesProvider", "color:" + res + " is not found");
		}
		return mResources.getColor(id);
	}

	public int getInteger(String res) {
		int id = mResources.getIdentifier(res, "integer", mPkgName);
		if (id == 0) {
			LogUtils.e("ResourcesProvider", "integer:" + res + " is not found");
		}
		return mResources.getInteger(id);
	}

	public float getDimension(String res) {
		int id = mResources.getIdentifier(res, "dimen", mPkgName);
		if (id == 0) {
			LogUtils.e("ResourcesProvider", "dimen:" + res + " is not found");
		}
		return mResources.getDimension(id);
	}

	public int getDimensionPixelSize(String res) {
		int id = mResources.getIdentifier(res, "dimen", mPkgName);
		if (id == 0) {
			LogUtils.e("ResourcesProvider", "dimen:" + res + " is not found");
		}
		return mResources.getDimensionPixelOffset(id);
	}

	public CharSequence getText(String res) {
		int id = mResources.getIdentifier(res, "string", mPkgName);
		if (id == 0) {
			LogUtils.e("ResourcesProvider", "string:" + res + " is not found");
		}
		return mResources.getText(id);
	}

	public String getString(String res) {
		int id = mResources.getIdentifier(res, "string", mPkgName);
		if (id == 0) {
			LogUtils.e("ResourcesProvider", "string:" + res + " is not found");
		}
		return mResources.getString(id);
	}

	public Drawable getDrawable(String res) {
		int id = mResources.getIdentifier(res, "drawable", mPkgName);
		if (id == 0) {
			LogUtils.e("ResourcesProvider", "drawable:" + res + " is not found");
		}
		return mResources.getDrawable(id);
	}

	public View getView(String res, ViewGroup root) {
		int id = mResources.getIdentifier(res, "layout", mPkgName);
		if (id == 0) {
			LogUtils.e("ResourcesProvider", "layout:" + res + " is not found");
		}
		return mInflater.inflate(id, root);
	}

	public View getView(String res, ViewGroup root, boolean attachToRoot) {
		int id = mResources.getIdentifier(res, "layout", mPkgName);
		if (id == 0) {
			LogUtils.e("ResourcesProvider", "layout:" + res + " is not found");
		}
		return mInflater.inflate(id, root, attachToRoot);
	}

	public Animation getAnimation(Application context, String res) {
		int id = mResources.getIdentifier(res, "anim", mPkgName);
		if (id == 0) {
			LogUtils.e("ResourcesProvider", "anim:" + res + " is not found");
		}
		return AnimationUtils.loadAnimation(context, id);
	}

	public XmlResourceParser getXml(String res) {
		int id = mResources.getIdentifier(res, "xml", mPkgName);
		if (id == 0) {
			LogUtils.e("ResourcesProvider", "xml:" + res + " is not found");
		}
		return mResources.getXml(id);
	}

	public boolean getBoolean(String res) {
		int id = mResources.getIdentifier(res, "bool", mPkgName);
		if (id == 0) {
			LogUtils.e("ResourcesProvider", "bool:" + res + " is not found");
		}
		return mResources.getBoolean(id);
	}
}
