package com.jb.ga0.commerce.util.retrofit;

import android.content.Context;

import com.jb.ga0.commerce.util.retrofit.Interceptor.GunzipResponseInterceptor;
import com.jb.ga0.commerce.util.retrofit.Interceptor.LogInterceptor;
import com.jb.ga0.commerce.util.retrofit.Interceptor.RepeatRequestInterceptor;
import com.jb.ga0.commerce.util.retrofit.Interceptor.RetryAfterNetOkInterceptor;

import java.util.concurrent.TimeUnit;

import okhttp3.OkHttpClient;

/**
 * Created by matt on 2017/10/19.
 */
public class ClientProvider {
    private static ClientProvider sInstance;
    private Context mContext;
    private OkHttpClient mClient;

    private ClientProvider(Context context) {
        mContext = context.getApplicationContext();

        OkHttpClient.Builder builder = new OkHttpClient.Builder()
                .connectTimeout(15, TimeUnit.SECONDS)
                .readTimeout(15, TimeUnit.SECONDS)
                .writeTimeout(15, TimeUnit.SECONDS)
                 // 打印网络请求信息
                .addInterceptor(LogInterceptor.getsInstance(mContext))
//                //请求压缩
//                .addInterceptor(new GzipRequestInterceptor()))
                //响应解压缩
                .addInterceptor(new GunzipResponseInterceptor())
                //todo debug by matt
//                .addNetworkInterceptor(new StethoInterceptor())
                //网络失败重试
                .addInterceptor(new RetryAfterNetOkInterceptor(mContext))
                //定时请求
                .addInterceptor(new RepeatRequestInterceptor())
                ;

        mClient = builder.build();
    }

    public static ClientProvider getInstance(Context context) {
        if (sInstance == null) {
            synchronized (ClientProvider.class) {
                if (sInstance == null) {
                    sInstance = new ClientProvider(context);
                }
            }
        }
        return sInstance;
    }

    /**
     * 返回的OkHttpClient，只能添加拦截器，不应修改其它配置
     * @return
     */
    public OkHttpClient get() {
        return mClient;
    }

    /**
     * 设置http请求打印的log开关
     * @param log
     */
    public void setLog(boolean log) {
        LogInterceptor.getsInstance(mContext).setLog(log);
    }
}