package com.jb.ga0.commerce.util.retrofit.Interceptor;

import android.content.Context;
import android.util.Log;

import com.jb.ga0.commerce.util.LogUtils;
import com.jb.ga0.commerce.util.retrofit.HttpConstants;

import java.io.IOException;

import okhttp3.Interceptor;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.ResponseBody;
import okio.Buffer;

/**
 * 网络失败时，取缓存数据
 * 有网的情况下直接从网络中获取数据，无网络的情况下才走缓存
 * Created by matt on 2017/10/19.
 */
public class LogInterceptor implements Interceptor {
    private static final String TAG = HttpConstants.LogTag.TAG;

    private static LogInterceptor sInstance = new LogInterceptor(null);
    private boolean mIsLog = false;

    private LogInterceptor(Context context) {
    }

    public static LogInterceptor getsInstance(Context context) {
        return sInstance;
    }

    public void setLog(boolean log) {
        mIsLog = log;
    }

    @Override
    public Response intercept(Chain chain) throws IOException {
        LogUtils.i(TAG, "LogInterceptor#intercept() enter");

        Request request = chain.request();
        if (!mIsLog) {
            return chain.proceed(request);
        }
        //todo debug by matt
        request = request.newBuilder().removeHeader("Accept-Encoding").build();

        long t1 = System.nanoTime();//请求发起的时间

        LogUtils.i(TAG, String.format("%d发送请求 %s %nbody: %s %non connection: %s %n报文heads:%n%s ",
                request.hashCode(), request.url(), getRequestBodyString(request), chain.connection(), request.headers()));

        Response response = null;
        try {
            response = chain.proceed(request);
        } catch (IOException e) {
            Log.w(TAG, "[LogInterceptor#intercept] network fail-->", e);
            throw e;
        }

        long t2 = System.nanoTime();//收到响应的时间

        //这里不能直接使用response.body().string()的方式输出日志
        //因为response.body().string()之后，response中的流会被关闭，程序会报错，我们需要创建出一
        //个新的response给应用层处理
        ResponseBody responseBody = response.peekBody(1024 * 1024);

        LogUtils.i(TAG, String.format("%d接收响应: [%s] %n响应码：%d %n耗时：%.1fms, from %s %n报文headers：%n%s",
                request.hashCode(),
                response.request().url(),
                response.code(),
                (t2 - t1) / 1e6d,
                response.cacheResponse() != null ? "cache" : "network",
                response.headers()));
        LogUtils.i(TAG, String.format("%d接收响应: [%s] %n返回数据:【%s】",
                request.hashCode(),
                response.request().url(),
                responseBody.string()));

        return response;
    }

    private static String getRequestBodyString(Request request){
        try {
            final Request copy = request.newBuilder().build();
            final Buffer buffer = new Buffer();
            copy.body().writeTo(buffer);
            return buffer.readUtf8();
        } catch (final IOException e) {
            return "did not work";
        }
    }
}
