package com.jb.ga0.commerce.util.retrofit.Interceptor;

import android.content.SharedPreferences;
import android.util.Log;

import com.jb.ga0.commerce.util.CustomAlarm;

import java.util.HashMap;

import static com.jb.ga0.commerce.util.retrofit.HttpConstants.LogTag.TAG;

/**
 * Created by matt on 2017/10/31.
 */
public class RepeatRequestCtrl {
    public static final String REPEAT_REQUEST_KEY = "repeat_request_key";

    private static RepeatRequestCtrl sInstance;
    private HashMap<String, RepeatRequestParam> mParamHashMap = new HashMap<>();

    private RepeatRequestCtrl() {

    }

    public static RepeatRequestCtrl getInstance() {
        if (sInstance == null) {
            synchronized (RepeatRequestCtrl.class) {
                if (sInstance == null) {
                    sInstance = new RepeatRequestCtrl();
                }
            }
        }
        return sInstance;
    }

    /**
     * 发送定时http请求命令
     * <br/>在调用此方法的同时，需要在header里添加repeat_request_key, 值同@param requestKey
     * <br/>Retrofit设置形如 @Headers("repeat_request_key: 你的requestKey")，示例如下：
     * <br/>@Headers("repeat_request_key: https://api.douban.com/v2/movie/repeat")
     *
     * @param sp sp文件，用于储存请求成功时间
     * @param alarm 定时器，通过 {@link com.jb.ga0.commerce.util.CustomAlarmManager#getAlarm(String)}获取
     * @param alarmId 定时器id，在同一个alarm module里，alarmId不能重复
     * @param triggerDelay 定时器第1次触发时间
     * @param interval 定时器触发间隔
     * @param requestKey 请求key，唯一标志该请求，建议使用 baseurl+suffix
     * @param r 执行http请求的runnable
     */
    public void callRepeat(SharedPreferences sp, CustomAlarm alarm, int alarmId, long triggerDelay, long interval, String requestKey, final Runnable r) {
        if (mParamHashMap.containsKey(requestKey)) {
            Log.w(TAG, "[RepeatRequestCtrl#callRepeat] requestKey已存在，请检查是否equestKey定义重复！", new Throwable());
        }

        mParamHashMap.put(requestKey, new RepeatRequestParam(sp, alarm, alarmId));
        alarm.alarmRepeatInRealTime(sp, alarmId, triggerDelay, interval, true, new CustomAlarm.OnAlarmListener() {
            @Override
            public void onAlarm(int i) {
                r.run();
            }
        });
    }

    void recordSuccess(String requestKey) {
        RepeatRequestParam param = mParamHashMap.get(requestKey);
        if (param != null) {
            param.saveTriggerTime();
        }
    }

    class RepeatRequestParam {
        SharedPreferences mSp;
        CustomAlarm mAlarm;
        int mAlarmId;

        RepeatRequestParam(SharedPreferences sp, CustomAlarm alarm, int alarmId) {
            mSp = sp;
            mAlarm = alarm;
            mAlarmId = alarmId;
        }

        void saveTriggerTime() {
            mAlarm.saveTriggerTime(mSp, mAlarmId);
        }
    }
}