package com.jb.ga0.commerce.util.retrofit;

import org.json.JSONObject;

import java.util.Map;

import okhttp3.FormBody;
import okhttp3.MediaType;
import okhttp3.RequestBody;

/**
 * RequestBody构建器
 * Created by matt on 2018/1/3.
 */
public class RequestBodyBuilder {
    /**
     * json格式的request body，形如"{"k1":"v1", "j2":{"k2":"v2"}}"
     * @param jsonObject
     * @return
     */
    public static RequestBody fromJsonObject(JSONObject jsonObject) {
        if (jsonObject == null) {
            return null;
        }
        return RequestBody.create(MediaType.parse("application/json; charset=utf-8"), jsonObject.toString());
    }

    /**
     * 键值对格式的request body, 形如"k1=v1&k2=v2..."
     * @param fieldMap
     * @return
     */
    public static RequestBody fromFieldMap(Map<String, String> fieldMap) {
        if (fieldMap == null) {
            return null;
        }

        FormBody.Builder builder = new FormBody.Builder();
        for (Map.Entry<String, String> entry : fieldMap.entrySet()) {
            builder.add(entry.getKey(), entry.getValue());
        }
        return builder.build();
    }

    /**
     * 文本格式的request body
     * @param text
     * @return
     */
    public static RequestBody fromText(String text) {
        if (text == null) {
            return null;
        }

        return RequestBody.create(MediaType.parse("text/plain"), text);
    }
}