package com.jb.ga0.commerce.util.retrofit;

import android.util.Log;

import com.jb.ga0.commerce.util.LogUtils;

import java.io.IOException;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;


/**
 * Created by matt on 2017/10/17.
 */
public class RetrofitProxy {
    private static final String TAG = HttpConstants.LogTag.TAG;

    /**
     * callback接口
     */
    public static interface HttpCallback<T> {
        /**
         * 请求数据成功
         * @param call
         * @param response
         */
        void onHttpSuccess(Call<T> call, Response<T> response);

        /**
         * 请求数据失败
         * @param call
         * @param response 仅{@link HttpErrorCode#RemoteError}时不为null
         * @param throwable
         * @param errorCode
         */
        void onHttpFailure(Call<T> call, Response<T> response, Throwable throwable, HttpErrorCode errorCode);
    }

    /**
     * 错误码
     */
    public static enum HttpErrorCode {
        //无网络或网络连接失败(没有Response)
        NetFail,
        //服务器错误（有Response，但有Response#isSuccessful()为false）
        RemoteError,
        //解析错误 （onFailure）
        ParseFail,
        //被取消（onFailure）
        Canceled,
    }

    /**
     * retrofit Callback封装
     */
    public static class RetrofitCallback<T> implements Callback {
        private HttpCallback<T> mHttpCallback;

        public RetrofitCallback(HttpCallback<T> httpCallback) {
            mHttpCallback = httpCallback;
        }

        @Override
        public void onResponse(Call call, Response response) {
            LogUtils.i(TAG, "RetrofitCallback#onResponse() url = " + call.request().url());

            if (response != null && response.isSuccessful()) {
                mHttpCallback.onHttpSuccess(call, response);
            } else {
                //服务器返回错误码
                mHttpCallback.onHttpFailure(call, response, new IOException("服务器错误"), HttpErrorCode.RemoteError);
            }
        }

        @Override
        public void onFailure(Call call, Throwable t) {
            Log.w(TAG, "[RetrofitProxy#onFailure] ", t);

            if (t instanceof IOException) {
                mHttpCallback.onHttpFailure(call, null, t, call.isCanceled() ? HttpErrorCode.Canceled : HttpErrorCode.NetFail);
            } else {
                mHttpCallback.onHttpFailure(call, null, t, HttpErrorCode.ParseFail);
            }
        }
    }
}
