package com.jb.ga0.commerce.util.retrofit.test;

import android.content.Context;
import android.content.SharedPreferences;
import android.util.Log;

import com.jb.ga0.commerce.util.CustomAlarm;
import com.jb.ga0.commerce.util.CustomAlarmManager;
import com.jb.ga0.commerce.util.LogUtils;
import com.jb.ga0.commerce.util.retrofit.ClientProvider;
import com.jb.ga0.commerce.util.retrofit.HttpConstants;
import com.jb.ga0.commerce.util.retrofit.Interceptor.LogInterceptor;
import com.jb.ga0.commerce.util.retrofit.Interceptor.RepeatRequestCtrl;
import com.jb.ga0.commerce.util.retrofit.Interceptor.RetryAfterNetOkCtrl;
import com.jb.ga0.commerce.util.retrofit.RequestBodyBuilder;
import com.jb.ga0.commerce.util.retrofit.RetrofitRequest;
import com.jb.ga0.commerce.util.retrofit.cache.CustomCacheInterceptor;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.net.UnknownHostException;
import java.util.List;
import java.util.concurrent.TimeUnit;

import okhttp3.CacheControl;
import okhttp3.OkHttpClient;
import okhttp3.ResponseBody;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;
import retrofit2.Retrofit;
import retrofit2.converter.gson.GsonConverterFactory;

/**
 * Created by matt on 2017/10/18.
 */
public class Test {
    static String url = "http://abtest.goforandroid.com/abtestcenter/ab?sid=292&gzip=0&utm_source=unknown_buychannel&isupgrade=2&aid=4ff9050001bb056f&local=US&cversion=500&user_from=-1&pkgname=com.commerce.chargelockerdemo&cid=6&cdays=1&entrance=999";
    static String baseUrl = "http://abtest.goforandroid.com/abtestcenter/";
    static String path = "ab";
    static String query = "?sid=292&gzip=0&utm_source=unknown_buychannel&isupgrade=2&aid=4ff9050001bb056f&local=US&cversion=500&user_from=-1&pkgname=com.commerce.chargelockerdemo&cid=6&cdays=1&entrance=999";

    private static final String TAG = HttpConstants.LogTag.TAG;

    public static void test(final Context context) {
        LogInterceptor.getsInstance(context).setLog(true);
//        testRetryAfterNetOk(context);
//        testCache(context);
//        testWrapperCache(context);
//        testWrapperRepeatRequest(context);
//        testWrapperRetryAfterNetOk(context);
        testWrapperPostJson(context);
    }

    /**
     *
     * @param context
     */
    public static void testWrapperPostJson(Context context) {
        /**
         * 协议参考：http://wiki.3g.net.cn/pages/viewpage.action?pageId=21530052
         */

        String url = "http://adviw.goforandroid.com/api/v1/func";
        retrofit2.Callback callback = new retrofit2.Callback<ResponseBody>() {
            @Override
            public void onResponse(Call<ResponseBody> call, Response<ResponseBody> response) {
            }

            @Override
            public void onFailure(Call<ResponseBody> call, Throwable t) {
            }
        };

        LogInterceptor.getsInstance(context).setLog(true);
        LogUtils.setShowLog(true);
        new RetrofitRequest(url, RetrofitRequest.Method.post)
                .setRequestBody(RequestBodyBuilder.fromJsonObject(getPostData()))
                .setCallback(callback)
                .enqueue(context);
    }
    private static JSONObject getPostData() {
        JSONObject postData = new JSONObject();
        try {
            JSONObject phead = new JSONObject();
            phead.put("vcode", 1);
            phead.put("vname", "1");
            phead.put("country", "US");
            phead.put("lang", "en");
            phead.put("aid", "a73f5c1f7985181a");
            phead.put("imei", "");
            phead.put("sys", "5.5");
            phead.put("net", "4G");
            phead.put("hasmarket", 1);
            phead.put("adid", "8ef191c6-26d6-4f80-8ef3-a3da9a80d2d8");
            phead.put("pkgname", "com.jb.tube");

            postData.put("phead", phead);
            postData.put("accessKey", "QV6KSEODASGEY71LJ7BKNZA3ZCY39I9A");
            postData.put("prodKey", "RCMFKQ276HU75G2Q77WGML4I");
        } catch (JSONException e) {
            e.printStackTrace();
        }

        return postData;
    }

    private String getFeaturesUrl() {
        return "http://adviw.goforandroid.com/api/v1/func";
    }

    /**
     * 测试失败重试接口
     *
     * @param context
     */
    public static void testWrapperRetryAfterNetOk(final Context context) {

        String requestKey = "https://api.douban.com/v2/movie/retry";

        String url = "https://api.douban.com/v2/movie/top250?start=0&count=3";
        RetrofitRequest.RequestRetryAfterNetOk retryAfterNetOk = new RetrofitRequest.RequestRetryAfterNetOk(
                requestKey, 5
        );

        Callback<ResponseBody> callback = new Callback<ResponseBody>() {
            @Override
            public void onResponse(Call<ResponseBody> call, Response<ResponseBody> response) {
                Log.i(TAG, "[Test#onHttpSuccess] netResponse:" + response.raw().networkResponse());
                Log.i(TAG, "[Test#onHttpSuccess] cacheResponse:" + response.raw().cacheResponse());
                try {
                    Log.i(TAG, "[Test#onHttpSuccess] " + response.body().string());
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onFailure(Call<ResponseBody> call, Throwable t) {
                if (t instanceof UnknownHostException) {
                    Log.w(TAG, "[Test#onHttpFailure] " + t);
                } else {
                    Log.w(TAG, "Test#onHttpFailure() called", t);
                }
            }
        };
        new RetrofitRequest(url, RetrofitRequest.Method.get)
                .setRequestRetryAfterNetOk(retryAfterNetOk)
                .setCallback(callback)
                .enqueue(context);

    }

    /**
     * 测试定时请求接口
     *
     * @param context
     */
    public static void testWrapperRepeatRequest(final Context context) {

        SharedPreferences sp = context.getSharedPreferences("testRepeatRequest", Context.MODE_PRIVATE);
        CustomAlarm alarm = CustomAlarmManager.getInstance(context).getAlarm("mymode");
        int alarmId = 1;
        long delay = 0;
        long interval = 60 * 60 * 1000; //1小时
        String requestKey = "https://api.douban.com/v2/movie/repeat";

        String url = "https://api.douban.com/v2/movie/top250?start=0&count=3";
        RetrofitRequest.RequestRepeat requestRepeat = new RetrofitRequest.RequestRepeat(
                sp, alarm, alarmId, delay, interval, requestKey
        );

        Callback<ResponseBody> callback = new Callback<ResponseBody>() {
            @Override
            public void onResponse(Call<ResponseBody> call, Response<ResponseBody> response) {
                Log.i(TAG, "[Test#onHttpSuccess] netResponse:" + response.raw().networkResponse());
                Log.i(TAG, "[Test#onHttpSuccess] cacheResponse:" + response.raw().cacheResponse());
                try {
                    Log.i(TAG, "[Test#onHttpSuccess] " + response.body().string());
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onFailure(Call<ResponseBody> call, Throwable t) {
                if (t instanceof UnknownHostException) {
                    Log.w(TAG, "[Test#onHttpFailure] " + t);
                } else {
                    Log.w(TAG, "Test#onHttpFailure() called", t);
                }
            }
        };
        new RetrofitRequest(url, RetrofitRequest.Method.get)
                .setRequestRepeat(requestRepeat)
                .setCallback(callback)
                .enqueue(context);

    }

    /**
     * 测试缓存接口
     *
     * @param context
     */
    public static void testWrapperCache(Context context) {
        String url = "https://api.douban.com/v2/movie/top250?start=0&count=3";

        RetrofitRequest.RequestCache requestCache = new RetrofitRequest.RequestCache(
                CustomCacheInterceptor.CacheType.cache_both,
                new CacheControl.Builder().maxAge(3600, TimeUnit.SECONDS).build(),
                "https://api.douban.com/v2/movie");
        Callback<ResponseBody> callback = new Callback<ResponseBody>() {
            @Override
            public void onResponse(Call<ResponseBody> call, Response<ResponseBody> response) {
                Log.i(TAG, "[Test#onHttpSuccess] netResponse:" + response.raw().networkResponse());
                Log.i(TAG, "[Test#onHttpSuccess] cacheResponse:" + response.raw().cacheResponse());
                try {
                    Log.i(TAG, "[Test#onHttpSuccess] " + response.body().string());
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onFailure(Call<ResponseBody> call, Throwable t) {
                if (t instanceof UnknownHostException) {
                    Log.w(TAG, "[Test#onHttpFailure] " + t);
                } else {
                    Log.w(TAG, "Test#onHttpFailure() called", t);
                }
            }
        };
        new RetrofitRequest(url, RetrofitRequest.Method.get)
                .setRequestCache(requestCache)
                .setCallback(callback)
                .enqueue(context);
    }

    /**
     * 测试失败重试接口
     *
     * @param context
     */
    public static void testRetryAfterNetOk(final Context context) {

        String requestKey = "https://api.douban.com/v2/movie/retry";

        //发起可失败重试的请求
        RetryAfterNetOkCtrl.getInstance(context).call(requestKey, 5, new Runnable() {
            @Override
            public void run() {
                String url = "https://api.douban.com/v2/movie/top250?start=0&count=3";
                Retrofit retrofit = buildRetrofit(context, url, null);

                Call<HttpResult<List<ASubject>>> call = retrofit.create(TestService.class).getMovieRetry(url);

                call.enqueue(new Callback<HttpResult<List<ASubject>>>() {
                    @Override
                    public void onResponse(Call<HttpResult<List<ASubject>>> call, Response<HttpResult<List<ASubject>>> response) {
                        Log.i(TAG, "[Test#onHttpSuccess] netResponse:" + response.raw().networkResponse());
                        Log.i(TAG, "[Test#onHttpSuccess] cacheResponse:" + response.raw().cacheResponse());
                        Log.i(TAG, "[Test#onHttpSuccess] " + response.body().getSubjects().get(0));
                    }

                    @Override
                    public void onFailure(Call<HttpResult<List<ASubject>>> call, Throwable t) {
                        if (t instanceof UnknownHostException) {
                            Log.w(TAG, "[Test#onHttpFailure] " + t);
                        } else {
                            Log.w(TAG, "Test#onHttpFailure() called", t);
                        }
                    }
                });
            }
        });

    }

    /**
     * 测试定时请求接口
     *
     * @param context
     */
    public static void testRepeatRequest(final Context context) {

        SharedPreferences sp = context.getSharedPreferences("testRepeatRequest", Context.MODE_PRIVATE);
        CustomAlarm alarm = CustomAlarmManager.getInstance(context).getAlarm("mymode");
        int alarmId = 1;
        long delay = 0;
        long interval = 60 * 60 * 1000; //1小时
        String requestKey = "https://api.douban.com/v2/movie/repeat";

        //发起定时请求
        RepeatRequestCtrl.getInstance().callRepeat(sp, alarm, alarmId, delay, interval, requestKey, new Runnable() {
            @Override
            public void run() {
                String url = "https://api.douban.com/v2/movie/top250?start=0&count=3";
                Retrofit retrofit = buildRetrofit(context, url, null);

                Call<HttpResult<List<ASubject>>> call = retrofit.create(TestService.class).getMovieRepeat(url);

                call.enqueue(new Callback<HttpResult<List<ASubject>>>() {
                    @Override
                    public void onResponse(Call<HttpResult<List<ASubject>>> call, Response<HttpResult<List<ASubject>>> response) {
                        Log.i(TAG, "[Test#onHttpSuccess] netResponse:" + response.raw().networkResponse());
                        Log.i(TAG, "[Test#onHttpSuccess] cacheResponse:" + response.raw().cacheResponse());
                        Log.i(TAG, "[Test#onHttpSuccess] " + response.body().getSubjects().get(0));
                    }

                    @Override
                    public void onFailure(Call<HttpResult<List<ASubject>>> call, Throwable t) {
                        if (t instanceof UnknownHostException) {
                            Log.w(TAG, "[Test#onHttpFailure] " + t);
                        } else {
                            Log.w(TAG, "Test#onHttpFailure() called", t);
                        }
                    }
                });
            }
        });

    }

    /**
     * 测试缓存接口
     *
     * @param context
     */
    public static void testCache(Context context) {
        String url = "https://api.douban.com/v2/movie/top250?start=0&count=3";
        OkHttpClient client = ClientProvider.getInstance(context).get().newBuilder()
                //这里添加Cache设置
                .addInterceptor(new CustomCacheInterceptor.Builder(
                        context, CustomCacheInterceptor.CacheType.cache_both)
                        .setCacheControl(new CacheControl.Builder().maxAge(3600, TimeUnit.SECONDS).build())
                        .setCacheKey("https://api.douban.com/v2/movie")
                        .build()
                ).build();
        Retrofit retrofit = buildRetrofit(context, url, client);

        Call<HttpResult<List<ASubject>>> call = retrofit.create(TestService.class).getMovie(url);

        call.enqueue(new Callback<HttpResult<List<ASubject>>>() {
            @Override
            public void onResponse(Call<HttpResult<List<ASubject>>> call, Response<HttpResult<List<ASubject>>> response) {
                Log.i(TAG, "[Test#onHttpSuccess] netResponse:" + response.raw().networkResponse());
                Log.i(TAG, "[Test#onHttpSuccess] cacheResponse:" + response.raw().cacheResponse());
                Log.i(TAG, "[Test#onHttpSuccess] " + response.body().getSubjects().get(0));
            }

            @Override
            public void onFailure(Call<HttpResult<List<ASubject>>> call, Throwable t) {
                if (t instanceof UnknownHostException) {
                    Log.w(TAG, "[Test#onHttpFailure] " + t);
                } else {
                    Log.w(TAG, "Test#onHttpFailure() called", t);
                }
            }
        });
    }

    private static Retrofit buildRetrofit(Context context, String url) {
        return buildRetrofit(context, url, null);
    }

    private static Retrofit buildRetrofit(Context context, String url, OkHttpClient client) {
        return new Retrofit.Builder()
                .client(client != null ? client : ClientProvider.getInstance(context).get())
                .addConverterFactory(GsonConverterFactory.create())
//                .addCallAdapterFactory(RxJavaCallAdapterFactory.create())
                .baseUrl(url.substring(0, url.lastIndexOf("/") + 1))
                .build();
    }
}
