package com.jb.ga0.commerce.util.statistics;

import com.jb.ga0.commerce.util.LogUtils;

import android.content.Context;

/**
 * 103统计接口
 *
 * @author chenchongji
 * 2016年3月14日
 */
public class Base103Statistic extends BaseStatistic {
	/**
	 * 统计日志序列号
	 */
	private static final int LOG_ID = 103;

	/**
	 * 上传使用103协议的统计数据。
	 * "功能点ID||统计对象||操作代码||操作结果||入口||Tab分类||位置||关联对象||广告ID||备注"
	 * @param context
	 * @param isGoKeyboard 是否输入法产品
	 * @param params 统计数据
	 * @备注 103统计日志比101多一个字段"广告ID"
	 */
	public static void upload(Context context, boolean isGoKeyboard, Statistic103Params params) {
		StringBuffer buffer = new StringBuffer();
		//判断是否为输入法
		if (isGoKeyboard) {
			//当前时间(输入法需要特殊处理)
			buffer.append(System.currentTimeMillis());
			buffer.append(SEPARATOR);
		}
		// 功能点ID
		buffer.append(params.mFunId);
		buffer.append(SEPARATOR);
		// 统计对象(mapId)
		appendStatisticField(buffer, params.mSender);
		buffer.append(SEPARATOR);
		// 操作代码
		appendStatisticField(buffer, params.mOperationCode);
		buffer.append(SEPARATOR);
		// 操作结果-----0:未成功,1:成功(默认成功)
		appendStatisticField(buffer, params.mOperationResult);
		buffer.append(SEPARATOR);
		// 入口
		appendStatisticField(buffer, params.mEntrance);
		buffer.append(SEPARATOR);
		// Tab分类
		appendStatisticField(buffer, params.mTabCategory);
		buffer.append(SEPARATOR);
		// 位置
		appendStatisticField(buffer, params.mPosition);
		buffer.append(SEPARATOR);
		// 关联对象
		appendStatisticField(buffer, params.mAssociatedObj);
		buffer.append(SEPARATOR);
		// 广告ID
		appendStatisticField(buffer, params.mAdvertId);
		buffer.append(SEPARATOR);
		// 备注
		appendStatisticField(buffer, params.mRemark);
		// 上传统计数据
		uploadStatisticData(context, LOG_ID, params.mFunId, buffer);
		
		if (LogUtils.isShowLog()) {
			LogUtils.v(LOG_TAG, "/功能点ID : " + params.mFunId
					+ "   /统计对象 : " + params.mSender
					+ "   /操作代码 : " + params.mOperationCode
					+ "   /操作结果 : " + params.mOperationResult
					+ "   /入口 : " + params.mEntrance
					+ "   /Tab分类 : " + params.mTabCategory
					+ "   /位置 : " + params.mPosition 
					+ "   /关联对象 : " + params.mAssociatedObj
					+ "   /广告ID : " + params.mAdvertId
					+ "   /备注 : " + params.mRemark
					);
		}
	}

	/**
	 * 103统计参数
	 *
	 * @author chenchongji
	 * 2016年3月14日
	 */
	public static class Statistic103Params {
		/** 功能点ID */
		protected int mFunId;
		/** 统计对象 */
		protected String mSender;
		/** 操作代码 */
		protected String mOperationCode;
		/** 操作结果 */
		protected String mOperationResult;
		/** 入口 */
		protected String mEntrance;
		/** Tab分类 */
		protected String mTabCategory;
		/** 位置 */
		protected String mPosition;
		/** 关联对象 */
		protected String mAssociatedObj;
		/** 广告ID */
		protected String mAdvertId;
		/** 备注 */
		protected String mRemark;

		/**
		 * 功能点ID
		 * @param funId {@link Statistic103Params#mFunId}
		 * @return
		 */
		public Statistic103Params funId(int funId) {
			mFunId = funId;
			return this;
		}
		
		/**
		 * 统计对象
		 * @param sender {@link Statistic103Params#mSender}
		 * @return
		 */
		public Statistic103Params sender(String sender) {
			mSender = sender;
			return this;
		}
		
		/**
		 * 操作代码
		 * @param operationCode {@link Statistic103Params#mOperationCode}
		 * @return
		 */
		public Statistic103Params operationCode(String operationCode) {
			mOperationCode = operationCode;
			return this;
		}
		
		/**
		 * 操作结果
		 * @param operationResult {@link Statistic103Params#mOperationResult}
		 * @return
		 */
		public Statistic103Params operationResult(String operationResult) {
			mOperationResult = operationResult;
			return this;
		}
		
		/**
		 * 入口
		 * @param entrance {@link Statistic103Params#mEntrance}
		 * @return
		 */
		public Statistic103Params entrance(String entrance) {
			mEntrance = entrance;
			return this;
		}
	
		/**
		 * Tab分类 
		 * @param entrance {@link Statistic103Params#mTabCategory}
		 * @return
		 */
		public Statistic103Params tabCategory(String tabCategory) {
			mTabCategory = tabCategory;
			return this;
		}
		
		/**
		 * 位置
		 * @param position {@link Statistic103Params#mPosition}
		 * @return
		 */
		public Statistic103Params position(String position) {
			mPosition = position;
			return this;
		}
		
		/**
		 * 关联对象
		 * @param associatedObj {@link Statistic103Params#mAssociatedObj}
		 * @return
		 */
		public Statistic103Params associatedObj(String associatedObj) {
			mAssociatedObj = associatedObj;
			return this;
		}
		
		/**
		 * 广告ID
		 * @param advertId {@link Statistic103Params#mAdvertId}
		 * @return
		 */
		public Statistic103Params advertId(String advertId) {
			mAdvertId = advertId;
			return this;
		}
		
		/**
		 * 备注
		 * @param remark {@link Statistic103Params#mRemark}
		 * @return
		 */
		public Statistic103Params remark(String remark) {
			mRemark = remark;
			return this;
		}
	}
}
