package com.jb.ga0.commerce.util.statistics;

import com.gau.go.gostaticsdk.StatisticsManager;
import com.gau.go.gostaticsdk.beans.OptionBean;
import com.jb.ga0.commerce.util.LogUtils;
import com.jb.ga0.commerce.util.io.StringUtils;

import android.content.Context;
import android.text.TextUtils;

/**
 * 统计基类
 *
 * @author chenchongji
 * 2016年3月11日
 */
public abstract class BaseStatistic {
	/**
	 * {@link StatisticsManager}需要关注的方法：
	 * a-19协议：{@link StatisticsManager#upLoadBasicInfoStaticData(String, boolean, boolean, String, boolean)}
	 * b-普通协议101~105，是d接口的简化版：{@link StatisticsManager#uploadStaticData(int, int, String)}
	 * c-callback记录：{@link StatisticsManager#uploadRequestUrl(String)}
	 * d-带选项的统计接口：{@link StatisticsManager#uploadStaticDataForOptions(int, int, String, com.gau.go.gostaticsdk.OnInsertDBListener, OptionBean...)}
	 * e-可以自定义参数的接口：{@link StatisticsManager#uploadStaticDataForDiy(int, int, String, com.gau.go.gostaticsdk.OnInsertDBListener, String, String, String, OptionBean...)}
	 * f-高级用法，自己拼接所有log字段：{@link StatisticsManager#upLoadStaticData(String)}
	 * 
	 * ----------------------------------------------------------
	 * 默认只有103和207协议是实时上传，其它协议如需实时上传，需要自己控制
	 */
	//--------------------------------------------------------------
	
	/**
	 * 默认日志TAG
	 */
	protected static final String LOG_TAG = "CommerceStatistic";
	/**
	 *  统计数据分隔符
	 */
	protected static final String SEPARATOR = "||";

	/**
	 * 日志上传策略，可选参数
	 * @param context
	 * @param logId
	 * @param funId
	 * @param data
	 * @param uploadPolicy
	 */
	protected static void uploadStatisticData(Context context, int logId, int funId, StringBuffer data, SatisticsUploadPolicy ...uploadPolicy) {
		if (LogUtils.isShowLog()) {
			StatisticsManager.getInstance(context).enableLog(true);
			LogUtils.d(LOG_TAG, "logId:" + logId + ", funId:" + funId + "-->" + StringUtils.toString(data));
		}
		// 上传统计数据
		if (uploadPolicy.length > 0 && uploadPolicy[0] == SatisticsUploadPolicy.immediately_always) {
			//不受开关影响,立即上传统计数据.
			StatisticsManager.getInstance(context).uploadStaticDataForOptions(logId, funId, StringUtils.toString(data), null, 
				new OptionBean(OptionBean.OPTION_INDEX_IMMEDIATELY_ANYWAY, true));
		} else if (uploadPolicy.length > 0 && uploadPolicy[0] == SatisticsUploadPolicy.immediately_care_switch) {
			//若开关开启,则立即上传统计数据.
			StatisticsManager.getInstance(context).uploadStaticDataForOptions(logId, funId, StringUtils.toString(data), null,
					new OptionBean(OptionBean.OPTION_INDEX_IMMEDIATELY_CARE_SWITCH, (Boolean) true));
		} else {
			//默认策略
			StatisticsManager.getInstance(context).uploadStaticData(logId, funId, StringUtils.toString(data), null);
		}
	}
	
    /**
	 * 上传回调统计(商业化推广需要使用)
	 * @param context
	 * @param callUrl 回调Url
	 */
	public static void uploadRequestUrl(Context context, String callUrl) {
		if (context == null || TextUtils.isEmpty(callUrl)) {
			return;
		}
		//上传统计数据
		StatisticsManager.getInstance(context).uploadRequestUrl(callUrl);
	}

	/**
	 * 上传带Location的统计数据（目前主要只用于添加的特定地理位置上传）
	 * 
	 * @param context
	 * @param logSequence
	 * @param funId
	 * @param data
	 * @param position
	 */
	protected static void uploadStatisticDataWithLocation(Context context, int logSequence, int funId, StringBuffer data, String position) {
		StatisticsManager.getInstance(context).uploadStaticDataForOptions(logSequence, funId, StringUtils.toString(data), null, new OptionBean(OptionBean.OPTION_INDEX_POSITION, position));
	}

	/**
	 * 统计日志上传策略
	 *
	 * @author chenchongji
	 * 2016年3月14日
	 */
	public static enum SatisticsUploadPolicy {
		/**
		 * 默认策略，即103和207实时上传，
		 */
		default_policy,
		/**
		 * 忽略开关，总是实时上传
		 */
		immediately_always,
		/**
		 * 在开关开启的情况下，实时上传
		 */
		immediately_care_switch;
	}
	
	/**
	 * 添加统计字段到buffer，统一处理字段为null的情况
	 * @param sb
	 * @param field
	 * @return
	 */
	public static StringBuffer appendStatisticField(StringBuffer sb, String field) {
		if (field == null) {
			field = "";
		}
		return sb.append(field);
	}
}
