package com.jb.ga0.commerce.util.topApp;

import java.io.BufferedReader;
import java.io.Closeable;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by chenxiqiang on 17-8-10.
 */

public class PsInfo implements Comparable<PsInfo> {

    private static final String SPLIT_REG = "\\s+";
    private static final String FOREGROUND = "fg";
    private static final String CMD_PS = "ps -c -p -P";

    //sample
    //USER      PID   PPID  VSIZE  RSS  CPU PRIO  NICE  RTPRI SCHED  PCY WCHAN              PC  NAME
    //u0_a16    1923  1324  1163484 31508 0  20    0     0     0     fg     ep_poll 0000000000 S com.google.android.googlequicksearchbox:interactor
    //root      1     0     732    588   0  20    0     0     0         ffffffff 00000000 S /init

    public static List<PsInfo> getAll(boolean onlyForeApp) throws IOException {
        Process process = Runtime.getRuntime().exec(CMD_PS);
        List<String> lines = read(process.getInputStream());

        List<PsInfo> list = new ArrayList<>();
        //第0行是title
        for (int i = 1, len = lines.size(); i < len; i++) {
            String line = lines.get(i);
            String[] items = line.split(SPLIT_REG);
            PsInfo info = !onlyForeApp || isForeApp(items) ? new PsInfo(items) : null;
            if (info != null) {
                list.add(info);
            }
        }
        return list;
    }

    public static List<String> getAllPID(boolean onlyForeApp) throws IOException {
        Process process = Runtime.getRuntime().exec(CMD_PS);
        List<String> lines = read(process.getInputStream());

        List<String> list = new ArrayList<>();
        //第0行是title
        for (int i = 1, len = lines.size(); i < len; i++) {
            String[] items = lines.get(i).split(SPLIT_REG);
            String pid = !onlyForeApp || isForeApp(items) ? items[IDX_PID] : null;
            if (pid != null) {
                list.add(pid);
            }
        }
        return list;
    }

    private static boolean isForeApp(String[] items) {
        String name = items[items.length - 1];
        boolean isFore = FOREGROUND.equals(items[IDX_PCY_OR_WCHAN]);
        return isFore/*前台进程*/
                && name.indexOf('.') >= 1/*包名至少含有一个‘.’，并且不以之开头*/
                && !name.contains("-")/*包名不能包含中划线*/
                && (name.contains("/.") || !name.contains("/"))/*/.可能是应用的标志，但是只有文件分隔符为系统脚本进程*/
                ;
    }

    private static List<String> read(InputStream in) throws IOException {
        BufferedReader reader = null;

        List<String> lines = new ArrayList<>();
        String line;
        try {
            reader = new BufferedReader(new InputStreamReader(in));
            while ((line = reader.readLine()) != null) {
                lines.add(line);
            }
        } finally {
            close(reader);
            close(in);
        }

        return lines;
    }

    private static void close(Closeable closeable) {
        if (closeable != null) {
            try {
                closeable.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /*Base*/

    private final String[] items;

    public PsInfo(String psLine) {
        this(psLine.split(SPLIT_REG));
    }

    private PsInfo(String[] items) {
        this.items = items;
    }

    public static final int IDX_USER = 0;
    public static final int IDX_PID = 1;
    public static final int IDX_PPID = 2;
    public static final int IDX_VSIZE = 3;
    public static final int IDX_RSS = 4;
    public static final int IDX_CPU = 5;
    public static final int IDX_PRIO = 6;
    public static final int IDX_NICE = 7;
    public static final int IDX_RTPRI = 8;
    public static final int IDX_SCHED = 9;

    public static final int IDX_PCY_OR_WCHAN = 10;//pcy 可能为空导致错位

    public boolean isForeground() {
        return FOREGROUND.equals(items[IDX_PCY_OR_WCHAN]);
    }

    public String get(int idx) {
        return items[idx];
    }

    public String getName() {
        return items[items.length - 1];
    }

    public String getPID() {
        return items[IDX_PID];
    }

    /*Compare*/

    private Integer weight;

    public Integer getWeight() {
        if (weight == null) {
            weight = 100;
            try {
                int pri = Integer.parseInt(items[IDX_PRIO]);
                int nice = Integer.parseInt(items[IDX_NICE]);
                int cpu = Integer.parseInt(items[IDX_CPU]);
                weight = pri + nice - cpu;
            } catch (NumberFormatException e) {
                e.printStackTrace();
            }
        }
        return weight;
    }

    @Override
    public int compareTo(PsInfo o) {
        return getWeight() - o.getWeight();
    }
}
