package com.jb.ga0.commerce.util.topApp;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Pattern;

import com.jb.ga0.commerce.util.Machine;

import android.annotation.SuppressLint;
import android.annotation.TargetApi;
import android.app.Activity;
import android.app.ActivityManager;
import android.app.ActivityManager.RunningAppProcessInfo;
import android.app.AlarmManager;
import android.app.usage.UsageEvents;
import android.app.usage.UsageStats;
import android.app.usage.UsageStatsManager;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.os.Build;
import android.provider.Settings;
import android.text.TextUtils;
import android.util.Log;

/**
 * 类名称：AppUtils 类描述：应用程序相关操作
 *
 * @version 1.0.0
 */
public class TopHelper {

    static final String TAG = "AppUtils";
    static final String INVALID_PACKAGE_NAME = "invalid_package_name";

    public static Set<String> sNotALauncher = new HashSet<String>(
            Arrays.asList("com.jiubang.goscreenlock"));
    /**
     * 是否需要用新方法获取栈顶，对于5.0以上，6.0以下获取栈顶应用的机型
     */
    static final int GET_TOP_USE_NEW_METHOD_UNKNOW = 0;
    static final int GET_TOP_USE_NEW_METHOD_NO = 1;
    static final int GET_TOP_USE_NEW_METHOD_YEP = 2;
    static int sGetTopUseNewMethod = GET_TOP_USE_NEW_METHOD_UNKNOW;

    /**
     * 获取在功能菜单出现的程序列表
     *
     * @param context 上下文
     * @return 程序列表，类型是 List<ResolveInfo>
     */
    public static List<ResolveInfo> getLauncherApps(Context context) {
        List<ResolveInfo> infos = null;
        PackageManager packageMgr = context.getPackageManager();
        Intent intent = new Intent(Intent.ACTION_MAIN);
        intent.addCategory(Intent.CATEGORY_LAUNCHER);
        try {
            infos = packageMgr.queryIntentActivities(intent, 0);
        } catch (OutOfMemoryError e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
        packageMgr = null;
        return infos;
    }

    /**
     * 获取桌面类应用的包名.<br>
     *
     * @param context
     * @return
     */
    public static List<String> getLauncherPackageNames(Context context) {
        List<String> packages = new ArrayList<String>();
        PackageManager packageManager = context.getPackageManager();
        Intent intent = new Intent(Intent.ACTION_MAIN);
        intent.addCategory(Intent.CATEGORY_HOME);
        List<ResolveInfo> resolveInfo = null;
        try {
            resolveInfo = packageManager.queryIntentActivities(intent,
                    PackageManager.MATCH_DEFAULT_ONLY);
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (resolveInfo != null && !resolveInfo.isEmpty()) {
            for (ResolveInfo info : resolveInfo) {
                // 过滤掉一些名不符实的桌面
                if (!TextUtils.isEmpty(info.activityInfo.packageName)
                        && !sNotALauncher
                        .contains(info.activityInfo.packageName)) {
                    packages.add(info.activityInfo.packageName);
                }
            }
        }
        return packages;
    }

    /**
     * 获取当前前台Activity的应用的包名.<br>
     * 注意:只适用于5.0以下使用.<br>
     *
     * @param context
     * @return 对应的包名或当出错时为""
     */
    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    public static String getFrontActivityAppPackageName(Context context) {
        final ComponentName componentInfo = getTopActivity(context);
        if (componentInfo == null) {
            return "";
        }
        return componentInfo.getPackageName();
    }

    /**
     * 获取当前前台Activity的应用的ComponentName.<br>
     * 注意:只适用于5.0以下使用.<br>
     *
     * @param context
     * @return 对应的包名或当出错时为null
     */
    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    public static ComponentName getTopActivity(Context context) {
        if (Machine.HAS_SDK_LOLLIPOP) {
            throw new IllegalStateException(
                    "getTopActivity() has no mean for above LOLLIPOP!");
        }
        ActivityManager am = (ActivityManager) context
                .getSystemService(Activity.ACTIVITY_SERVICE);
        // get the info from the currently running task
        List<ActivityManager.RunningTaskInfo> taskInfo = null;
        try {
            taskInfo = am.getRunningTasks(1);
        } catch (Throwable thr) {
            thr.printStackTrace();
        }
        if (taskInfo == null || taskInfo.isEmpty()) {
            return null;
        }
        ComponentName componentInfo = taskInfo.get(0).topActivity;
        return componentInfo;
    }

    /**
     * 判断指定的应用的Activity是否位于前台.<br>
     *
     * @param context
     * @param packageName
     * @return
     */
    public static boolean isFrontActivity(Context context, String packageName) {
        if (TextUtils.isEmpty(packageName)) {
            return false;
        }
        boolean isFront = false;
        if (Machine.HAS_SDK_LOLLIPOP) {
            try {
                ActivityManager activityManager = (ActivityManager) context
                        .getSystemService(Context.ACTIVITY_SERVICE);
                List<RunningAppProcessInfo> appProcesses = activityManager
                        .getRunningAppProcesses();
                if (appProcesses == null) {
                    return false;
                }
                for (RunningAppProcessInfo appProcess : appProcesses) {
                    // 通过进程名及进程所用到的包名来进行查找
                    if (appProcess.importance == RunningAppProcessInfo.IMPORTANCE_FOREGROUND) {
                        if (appProcess.processName.equals(packageName)
                                || Arrays.asList(appProcess.pkgList).contains(
                                packageName)) {
                            isFront = true;
                        }
                    }
                }
            } catch (Exception e) {
                Log.e("wbq", e.toString());
            }
        } else {
            isFront = getFrontActivityAppPackageName(context).equals(
                    packageName);
        }
        return isFront;
    }

    /**
     * 判断是否返回到桌面.<br>
     *
     * @param context
     * @return
     */
    public static boolean isBackToPkg(Context context, List<String> launcherPackageNames) {
//		List<String> launcherPackageNames = getLauncherPackageNames(context);
        for (String packageName : launcherPackageNames) {
            if (isFrontActivity(context, packageName)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 判断是否返回到桌面.<br>
     * 适配了5.0及其以上和6.0以下系统，请保证在非主线程下调用
     *
     * @param context
     * @return
     */
    @SuppressLint("DefaultLocale")
    public static boolean isBackToPkgAboveLolipop(Context context, List<String> launcherPackageNames, boolean isHome) {
        if (!Machine.HAS_SDK_LOLLIPOP) {
            // 小于5.0
            return isBackToPkg(context, launcherPackageNames);
        }
        ComponentName topActivity = null;
        // 5.1或以上
        if (Machine.HAS_SDK_5_1_1) {
            if (TopHelper
                    .isPermissionPackageUsageStatsGrandedLollipopMr1(context)) {
                topActivity = TopHelper.getFrontActivityLollipopMr1(context);
            }
        }
        // 5.0
        else if (Machine.HAS_SDK_LOLLIPOP) {
            if (TopHelper
                    .isPermissionPackageUsageStatsGrandedOnLollipop(context)) {
                topActivity = TopHelper.getFrontActivityOnLollipop(context);
            }
        }
        if (topActivity != null) {
            final String topPkgname = topActivity.getPackageName();
            if (!TextUtils.isEmpty(topPkgname)
                    && !INVALID_PACKAGE_NAME.equals(topPkgname)) {
//				List<String> launcherPackageNames = getLauncherPackageNames(context);
                for (String packageName : launcherPackageNames) {
                    if (topPkgname.equals(packageName)) {
                        return true;
                    }
                }
            }
        }
        return isBackToPkgAboveLolipopByTrick(context, launcherPackageNames, isHome);
    }

    /**
     * 对于5.0+通过特殊方式判断是否回到桌面, 不一定有效，作为替补方法使用<br>
     *
     * @param context
     * @return
     */
    public static boolean isBackToPkgAboveLolipopByTrick(Context context, List<String> launcherPackageNames, boolean isHome) {
        if (Machine.HAS_SDK_LOLLIPOP) { //&& !Machine.HAS_SDK_6) {
            // 5.0及其以上和6.0以下
            if (sGetTopUseNewMethod == GET_TOP_USE_NEW_METHOD_UNKNOW) {
                ActivityManager actMgr = (ActivityManager) context
                        .getSystemService(Context.ACTIVITY_SERVICE);
                List<RunningAppProcessInfo> rApps = actMgr
                        .getRunningAppProcesses();
                List<ActivityManager.RunningTaskInfo> taskInfo = actMgr
                        .getRunningTasks(1);
                if ((rApps != null && rApps.size() >= 5)
                        || (taskInfo != null && taskInfo.size() >= 3)) {
                    sGetTopUseNewMethod = GET_TOP_USE_NEW_METHOD_NO;
                } else {
                    sGetTopUseNewMethod = GET_TOP_USE_NEW_METHOD_YEP;
                }
            }
            if (sGetTopUseNewMethod == GET_TOP_USE_NEW_METHOD_NO) {
                return isBackToPkg(context, launcherPackageNames);
            }
            try {
                ProcessHelperBean topAppBean = isHome ? ProcessHelperUtil
                        .getTopAppOnlyLauncher(context) : ProcessHelperUtil.getTopApp(context);
                if (topAppBean != null) {
                    String[] packageNames = topAppBean.getPkgLists();
                    String processName = topAppBean.getProcessName();
                    String topPkgname = "";
                    int i = 0;
                    if (packageNames != null) {
                        for (i = 0; i < packageNames.length; i++) {
                            if (processName.toLowerCase().contains(
                                    packageNames[i].toLowerCase())) {
                                topPkgname = packageNames[i];
                                break;
                            }
                        }
                        if (i >= packageNames.length) {
                            topPkgname = packageNames[0];
                        }
                        Log.d("wbq", "栈顶： " + topPkgname);
//						List<String> launcherPackageNames = getLauncherPackageNames(context);
                        for (String packageName : launcherPackageNames) {
                            if (topPkgname.equals(packageName)) {
                                return true;
                            }
                        }
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            } catch (Error e) {
                e.printStackTrace();
            }
        }
        return false;
    }

    /**
     * 对于5.0+通过特殊方式判断是否回到桌面, 不一定有效，作为替补方法使用<br>
     *
     * @param context
     * @return
     */
    public static String getTopPkgAboveLolipopByTrick(Context context) {
        if (Machine.HAS_SDK_LOLLIPOP) { // && !Machine.HAS_SDK_6) {
            // 5.0及其以上和6.0以下
            try {
                // 5.0及其以上和6.0以下
                if (sGetTopUseNewMethod == GET_TOP_USE_NEW_METHOD_UNKNOW) {
                    ActivityManager actMgr = (ActivityManager) context
                            .getSystemService(Context.ACTIVITY_SERVICE);
                    List<RunningAppProcessInfo> rApps = actMgr
                            .getRunningAppProcesses();
                    List<ActivityManager.RunningTaskInfo> taskInfo = actMgr
                            .getRunningTasks(1);
                    if ((rApps != null && rApps.size() >= 5)
                            || (taskInfo != null && taskInfo.size() >= 3)) {
                        sGetTopUseNewMethod = GET_TOP_USE_NEW_METHOD_NO;
                    } else {
                        sGetTopUseNewMethod = GET_TOP_USE_NEW_METHOD_YEP;
                    }
                }
                if (sGetTopUseNewMethod == GET_TOP_USE_NEW_METHOD_NO) {
                    return getTopAppPkgAboveL(context);
                }
//				ProcessHelperBean topAppBean = ProcessHelperUtil
//						.getTopAppOnlyLauncher(context); // TODO
                ProcessHelperBean topAppBean =
                        ProcessHelperUtil.getTopApp(context);
                if (topAppBean != null) {
                    String[] packageNames = topAppBean.getPkgLists();
                    String processName = topAppBean.getProcessName();
                    String topPkgname = "";
                    int i = 0;
                    if (packageNames != null) {
                        for (i = 0; i < packageNames.length; i++) {
                            if (processName.toLowerCase().contains(
                                    packageNames[i].toLowerCase())) {
                                topPkgname = packageNames[i];
                                break;
                            }
                        }
                        if (i >= packageNames.length) {
                            topPkgname = packageNames[0];
                        }
                        Log.d("wbq", "栈顶： " + topPkgname);
                        return topPkgname;
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            } catch (Error e) {
                e.printStackTrace();
            }
        }
        return "";
    }

    /**
     * 对于5.0+通过特殊方式判断是否回到桌面, 不一定有效，作为替补方法使用<br>
     * <p>
     * 由于使用ps命令先获取可行的进程id后再读取proc文件，性能会相对{@link #getTopPkgAboveLolipopByPsTrick(Context)}高一些。
     * <p>
     * 如果获取的结果为空的话，则重定向使用旧的方法来处理。
     *
     * @param context
     * @return
     */
    public static String getTopPkgAboveLolipopByPsTrick(Context context) {
        if (Machine.HAS_SDK_LOLLIPOP) { // && !Machine.HAS_SDK_6) {
            // 5.0及其以上和6.0以下
            try {
                // 5.0及其以上和6.0以下
                if (sGetTopUseNewMethod == GET_TOP_USE_NEW_METHOD_UNKNOW) {
                    ActivityManager actMgr = (ActivityManager) context
                            .getSystemService(Context.ACTIVITY_SERVICE);
                    List<RunningAppProcessInfo> rApps = actMgr
                            .getRunningAppProcesses();
                    List<ActivityManager.RunningTaskInfo> taskInfo = actMgr
                            .getRunningTasks(1);
                    if ((rApps != null && rApps.size() >= 5)
                            || (taskInfo != null && taskInfo.size() >= 3)) {
                        sGetTopUseNewMethod = GET_TOP_USE_NEW_METHOD_NO;
                    } else {
                        sGetTopUseNewMethod = GET_TOP_USE_NEW_METHOD_YEP;
                    }
                }
                if (sGetTopUseNewMethod == GET_TOP_USE_NEW_METHOD_NO) {
                    return getTopAppPkgAboveL(context);
                }
//				ProcessHelperBean topAppBean = ProcessHelperUtil
//						.getTopAppOnlyLauncher(context); // TODO
                ProcessHelperBean topAppBean = ProcessHelperUtil.getTopAppByPs(context);
                if (topAppBean == null) {
                    topAppBean = ProcessHelperUtil.getTopApp(context);
                }
                if (topAppBean != null) {
                    String[] packageNames = topAppBean.getPkgLists();
                    String processName = topAppBean.getProcessName();
                    String topPkgname = "";
                    int i = 0;
                    if (packageNames != null) {
                        for (i = 0; i < packageNames.length; i++) {
                            if (processName.toLowerCase().contains(
                                    packageNames[i].toLowerCase())) {
                                topPkgname = packageNames[i];
                                break;
                            }
                        }
                        if (i >= packageNames.length) {
                            topPkgname = packageNames[0];
                        }
                        Log.d("wbq", "栈顶： " + topPkgname);
                        return topPkgname;
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            } catch (Error e) {
                e.printStackTrace();
            }
        }
        return "";
    }

    private static String getTopAppPkgAboveL(Context context) {
        try {
            //获取当前正在运行进程列表
            ActivityManager activityManager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
            List<ActivityManager.RunningAppProcessInfo> appProcesses = activityManager.getRunningAppProcesses();
            if (appProcesses == null) {
                return null;
            }
            for (ActivityManager.RunningAppProcessInfo appProcess : appProcesses) {
                // 通过进程名及进程所用到的包名来进行查找
                if (appProcess.importance == ActivityManager.RunningAppProcessInfo.IMPORTANCE_FOREGROUND) {
                    if (appProcess.pkgList != null && appProcess.pkgList.length > 0) {
                        return appProcess.pkgList[0];
                    }
                }
            }
        } catch (Exception e) {
        }
        return null;
    }

    /**
     * 判断某个包名是否是桌面
     *
     * @param context
     * @param pkgName
     * @return
     */
    public static boolean isPkgNameHome(Context context, String pkgName) {
        List<String> launcherPackageNames = getLauncherPackageNames(context);
        for (String packageName : launcherPackageNames) {
            if (packageName.equals(pkgName)) {
                return true;
            }
        }

        return false;
    }

    /**
     * 在使用Intent试图打开其它软件(尤其是第三方)前, 应该先进行判断是否有支持该打开该Intent的Activity
     *
     * @param context
     * @param intent
     * @return
     */
    public static boolean isIntentSafe(Context context, Intent intent) {
        final PackageManager packageManager = context.getPackageManager();
        List<ResolveInfo> list = packageManager.queryIntentActivities(intent,
                PackageManager.MATCH_DEFAULT_ONLY);
        return list.size() > 0;
    }

    /**
     * 获取系统应用信息管理实例UsageStatsManager<br>
     * 只建议于5.0或以上使用<br>
     *
     * @param context
     * @return UsageStatsManager or null when error occur
     */
    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    public static UsageStatsManager getSystemServiceUsageStatsManager(Context context) {
        UsageStatsManager usageStatsManager = null;
        if (Machine.HAS_SDK_5_1_1) {
            usageStatsManager = (UsageStatsManager) context
                    .getSystemService(Context.USAGE_STATS_SERVICE);
        } else {
            try {
                usageStatsManager = (UsageStatsManager) context
                        .getSystemService("usagestats");
            } catch (Throwable e) {
                e.printStackTrace();
            }
        }
        return usageStatsManager;
    }

//	/**
//	 * 判断是否获取了android.permission.PACKAGE_USAGE_STAT权限<br>
//	 * 该权限是系统级别的权限, 不授予第三方应用, 但是第三方应用可以让用户主动授权该权限<br>
//	 * @see #openUsageAccess
//	 * @param context
//	 * @return
//	 */
//	public static boolean isPermissionPackageUsageStatsGranded(Context context) {
//		// 检查是否授权成功
//		boolean isGranded = false;
//		if (Machine.HAS_SDK_5_1_1) {
//			// 5.1或以上
//			isGranded = AppUtils
//					.isPermissionPackageUsageStatsGrandedLollipopMr1(ZBoostApplication
//							.getAppContext());
//		} else if (Machine.HAS_SDK_LOLLIPOP) {
//			// 5.0
//			isGranded = AppUtils
//					.isPermissionPackageUsageStatsGrandedOnLollipop(ZBoostApplication
//							.getAppContext());
//		}
//		return isGranded;
//	}

    /**
     * 判断是否获取了android.permission.PACKAGE_USAGE_STAT权限<br>
     * 该权限是系统级别的权限, 不授予第三方应用, 但是第三方应用可以让用户主动授权该权限<br>
     * 用于5.1版本或以上版本<br>
     *
     * @param context
     * @return
     * @see #openUsageAccess
     */
    @TargetApi(Build.VERSION_CODES.LOLLIPOP_MR1)
    public static boolean isPermissionPackageUsageStatsGrandedLollipopMr1(Context context) {
        if (Machine.HAS_SDK_5_1_1) {
            return isPermissionPackageUsageStatsGranded(getSystemServiceUsageStatsManager(context));
        }
        return false;
    }

    /**
     * 判断是否获取了android.permission.PACKAGE_USAGE_STAT权限<br>
     * 该权限是系统级别的权限, 不授予第三方应用, 但是第三方应用可以让用户主动授权该权限<br>
     * 用于5.0版本<br>
     *
     * @param context
     * @return
     * @see #openUsageAccess
     */
    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    public static boolean isPermissionPackageUsageStatsGrandedOnLollipop(Context context) {
        if (Machine.HAS_SDK_LOLLIPOP) {
            return isPermissionPackageUsageStatsGranded(getSystemServiceUsageStatsManager(context));
        }
        return false;
    }

    /**
     * 判断是否获取了android.permission.PACKAGE_USAGE_STAT权限<br>
     * 该权限是系统级别的权限, 不授予第三方应用, 但是第三方应用可以让用户主动授权该权限<br>
     *
     * @param usageStatsManager
     * @return
     * @see #openUsageAccess
     */
    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    private static boolean isPermissionPackageUsageStatsGranded(UsageStatsManager usageStatsManager) {
        if (usageStatsManager != null && Machine.HAS_SDK_LOLLIPOP) {
            long endTime = System.currentTimeMillis();
            long beginTime = endTime - AlarmManager.INTERVAL_DAY;
            List<UsageStats> usageStatses = null;
            try {
                usageStatses = usageStatsManager.queryUsageStats(
                        UsageStatsManager.INTERVAL_BEST, beginTime, endTime);
            } catch (Throwable e) {
                e.printStackTrace();
            }
            return usageStatses != null && !usageStatses.isEmpty();
        }
        return false;
    }

    /**
     * 打开"有权查看使用情况的应用"的窗口 该权限的授权窗口在 设置 -> 安全 -> 高级 -> 有权查看使用情况的应用(Apps with
     * usage access)<br>
     *
     * @param context
     * @see #isPermissionPackageUsageStatsGranded
     */
    public static void openUsageAccess(Context context) {
        final String[] pages = new String[]{Settings.ACTION_USAGE_ACCESS_SETTINGS, Settings.ACTION_SECURITY_SETTINGS,
                Settings.ACTION_SETTINGS};
        for (int i = 0; i < pages.length; i++) {
            Intent intent = new Intent(pages[i]);
            if (!(context instanceof Activity)) {
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            }
            intent.addFlags(Intent.FLAG_ACTIVITY_EXCLUDE_FROM_RECENTS);
            intent.addFlags(Intent.FLAG_ACTIVITY_NO_HISTORY);
            if (isIntentSafe(context, intent)) {
                context.startActivity(intent);
                break;
            }
        }
    }

    /**
     * 获取栈顶应用程序
     *
     * @param context
     * @return
     */
    public static ComponentName getTopPackageName(Context context) {
        ComponentName topActivity = null;
        // 5.1或以上
        if (Machine.HAS_SDK_5_1_1) {
            if (TopHelper.isPermissionPackageUsageStatsGrandedLollipopMr1(context)) {
                topActivity = TopHelper.getFrontActivityLollipopMr1(context);
            }
        }
        // 5.0
        else if (Machine.HAS_SDK_LOLLIPOP) {
            if (TopHelper.isPermissionPackageUsageStatsGrandedOnLollipop(context)) {
                topActivity = TopHelper.getFrontActivityOnLollipop(context);
            }
        }
        // 5.0以下
        else {
            topActivity = TopHelper
                    .getTopActivity(context);
        }
        if (topActivity == null) {
            topActivity = new ComponentName("invalid_package_name", "invalid_activity_name");
        }
        return topActivity;
    }

    /**
     * 获取栈顶的应用的ComponentName<br>
     * 注意只适用于5.1或以上<br>
     *
     * @param context
     * @return ComponentName or null when error
     */
    @TargetApi(Build.VERSION_CODES.LOLLIPOP_MR1)
    public static ComponentName getFrontActivityLollipopMr1(Context context) {
        return getFrontActivityLollipop(getSystemServiceUsageStatsManager(context));
    }

    /**
     * 获取栈顶的应用的ComponentName<br>
     * 注意只适用于5.0<br>
     *
     * @param context
     * @return ComponentName or null when error
     */
    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    public static ComponentName getFrontActivityOnLollipop(Context context) {
        ComponentName frontActivity = getFrontActivityLollipop(getSystemServiceUsageStatsManager(context));
//		if (frontActivity == null) {
//			frontActivity = getFrontActivityOnLollipopByTrick(context);
//		}
        return frontActivity;
    }

    /**
     * 在5.0系统使用的获取栈顶应用的补充方法<br>
     *
     * @param context
     * @return
     */
    private static ComponentName getFrontActivityOnLollipopByTrick(Context context) {
        if (Build.VERSION.SDK_INT == Build.VERSION_CODES.LOLLIPOP) {
            ActivityManager activityManager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
            List<ActivityManager.RunningAppProcessInfo> pis = activityManager.getRunningAppProcesses();
            if (pis != null) {
                for (ActivityManager.RunningAppProcessInfo pi : pis) {
                    if (pi.pkgList.length == 1) {
                        return new ComponentName(pi.pkgList[0], "");
                    }
                }
            }
        }
        return new ComponentName(getForegroundAppByProcFiles(context), "");
    }

    /**
     * 用来存储最后一次usageEvent对应的componentName，这样我们可以在
     * com.gto.zero.zboost.util.AppUtils#getFrontActivityLollipop(android.app.usage.UsageStatsManager)
     * 中取相对较短的时间间隔
     */
    private static ComponentName sBackupComponentNameForFrontActivityLollipop;

    /**
     * 获取栈顶的应用的ComponentName<br>
     * 注意只适用于5.0或以上<br>
     *
     * @param usageStatsManager
     * @return ComponentName or null when error
     */
    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    private static ComponentName getFrontActivityLollipop(UsageStatsManager usageStatsManager) {
        if (usageStatsManager == null) {
            return null;
        }
        if (Machine.HAS_SDK_LOLLIPOP) {
            long endTime = System.currentTimeMillis();
            long beginTime = endTime - 10000; // 获取10秒内的事件
            UsageEvents.Event event = new UsageEvents.Event();
            long lastEvent = 0;
            String packageName = null;
            String className = null;
            UsageEvents usageEvents = null;
            // XXX 按照usageStatsManager.queryEvents的方法说明，最近几分钟内的事件应该是获取不到的；
            // 然而APP LOCK经实际使用来看是可以获取到最近的事件的，或许以后这个API会做到如其方法所说，
            // 这样可能这个方法就不适用我们所期望的用途了
            try {
                usageEvents = usageStatsManager.queryEvents(beginTime, endTime);
            } catch (Throwable e) {
                e.printStackTrace();
            }

            if (usageEvents != null) {
                while (usageEvents.hasNextEvent()) {
                    usageEvents.getNextEvent(event);
                    if (event.getEventType() == UsageEvents.Event.MOVE_TO_FOREGROUND) {
                        final long timeStamp = event.getTimeStamp();
                        // XXX 按照applocker的经验，以这个时间判断是否最新事件并不合适，只取最后一个事件就可以了
                        // if (timeStamp > lastEvent) {
                        lastEvent = timeStamp;
                        packageName = event.getPackageName();
                        className = event.getClassName();
                        // }
                    }
                }
            }
            // XXX 事件太久远了就不好使了吧...
//			if (System.currentTimeMillis() - lastEvent > 10 * 1000) {
//				// 用这个方法我们就没有className了
//				packageName = getFrontActivityPackageNameByQueryAndAggregateUsageStats(usageStatsManager);
//				className = "";
//			}

            if (TextUtils.isEmpty(className)) {
                className = "";
            }
            if (!TextUtils.isEmpty(packageName)) {
                sBackupComponentNameForFrontActivityLollipop = new ComponentName(packageName, className);
                return sBackupComponentNameForFrontActivityLollipop;
            }
        }

        return sBackupComponentNameForFrontActivityLollipop;
    }

    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    private static String getFrontActivityPackageNameByQueryAndAggregateUsageStats(UsageStatsManager usageStatsManager) {
        if (Machine.HAS_SDK_LOLLIPOP) {
            final long endTime = System.currentTimeMillis();
            final long beginTime = endTime - 10000;
            Map<String, UsageStats> queryUsageStats = null;
            try {
                queryUsageStats = usageStatsManager.queryAndAggregateUsageStats(beginTime, endTime);
            } catch (Throwable e) {
                e.printStackTrace();
            }

            if (queryUsageStats == null || queryUsageStats.isEmpty()) {
                return "";
            }

            String lastAppPkgName = null;
            long lastAppTime = 0;

            final Iterator<Map.Entry<String, UsageStats>> iterator = queryUsageStats.entrySet().iterator();
            while (iterator.hasNext()) {
                Map.Entry<String, UsageStats> next = iterator.next();
                UsageStats value = next.getValue();
                long useTime = Math.max(value.getFirstTimeStamp(), value.getLastTimeUsed());
                useTime = Math.max(useTime, value.getLastTimeStamp());
                if (useTime > lastAppTime) {
                    lastAppTime = useTime;
                    lastAppPkgName = value.getPackageName();
                }
            }

            return lastAppPkgName == null ? "" : lastAppPkgName;
        }

        return "";
    }


    private final static FilenameFilter FILENAME_FILTER_NUMS = new FilenameFilter() {

        /**
         * 匹配模式，只要数字
         */
        private Pattern mPattern = Pattern.compile("^[0-9]+$");

        @Override
        public boolean accept(File dir, String filename) {
            return mPattern.matcher(filename).matches();
        }
    };

    /**
     * first app user
     */
    private static final int AID_APP = 10000;

    /**
     * offset for uid ranges for each user
     */
    private static final int AID_USER = 100000;

    /**
     * 获取正在前台运行的应用的包名<br>
     * 通过分析进程文件估算，可靠性有待考量, 可作为补充使用<br>
     *
     * @param context
     * @return
     */
    public static String getForegroundAppByProcFiles(Context context) {
        File[] files = new File("/proc").listFiles(FILENAME_FILTER_NUMS);
        if (files == null || files.length == 0) {
            return "";
        }
        int lowestOomScore = Integer.MAX_VALUE;
//        String foregroundProcess = null;

        int foregroundProcessUid = -1;

        for (File file : files) {
            if (!file.isDirectory()) {
                continue;
            }

            int pid;
            try {
                pid = Integer.parseInt(file.getName());
            } catch (NumberFormatException e) {
                continue;
            }

            try {
                String cgroup = readFile(String.format("/proc/%d/cgroup", pid));

                String[] lines = cgroup.split("\n");

                if (lines.length != 2) {
                    continue;
                }

                String cpuSubsystem = lines[0];
                String cpuAcctSubsystem = lines[1];

                if (!cpuAcctSubsystem.endsWith(Integer.toString(pid))) {
                    // not an application process
                    continue;
                }

                if (cpuSubsystem.endsWith("bg_non_interactive")) {
                    // background policy
                    continue;
                }

//                String cmdline = read(String.format("/proc/%d/cmdline", pid));
//
//                if (cmdline.contains("com.android.systemui")) {
//                    continue;
//                }

                int uid = Integer.parseInt(cpuAcctSubsystem.split(":")[2].split("/")[1].replace("uid_", ""));
                if (uid >= 1000 && uid <= 1038) {
                    // system process
                    continue;
                }

                int appId = uid - AID_APP;
//                int userId = 0;
                // loop until we get the correct user id.
                // 100000 is the offset for each user.
                while (appId > AID_USER) {
                    appId -= AID_USER;
//                    userId++;
                }

                if (appId < 0) {
                    continue;
                }

                // u{user_id}_a{app_id} is used on API 17+ for multiple user account support.
                // String uidName = String.format("u%d_a%d", userId, appId);

                File oomScoreAdj = new File(String.format("/proc/%d/oom_score_adj", pid));
                if (oomScoreAdj.canRead()) {
                    int oomAdj = Integer.parseInt(readFile(oomScoreAdj.getAbsolutePath()));
                    if (oomAdj != 0) {
                        continue;
                    }
                }

                int oomScore = Integer.parseInt(readFile(String.format("/proc/%d/oom_score", pid)));
                if (oomScore < lowestOomScore) {
                    lowestOomScore = oomScore;
//                    foregroundProcess = cmdline;
                    foregroundProcessUid = uid;
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        if (foregroundProcessUid != -1) {
            return context.getPackageManager().getPackagesForUid(foregroundProcessUid)[0];
        }
        return "";
    }

    private static String readFile(String path) throws IOException {
        StringBuilder output = new StringBuilder();
        BufferedReader reader = new BufferedReader(new FileReader(path));
        output.append(reader.readLine());
        for (String line = reader.readLine(); line != null; line = reader.readLine()) {
            output.append('\n').append(line);
        }
        reader.close();
        return output.toString();
    }

    static void checkMethod(Context context) {
        if (sGetTopUseNewMethod == GET_TOP_USE_NEW_METHOD_UNKNOW) {
            ActivityManager actMgr = (ActivityManager) context
                    .getSystemService(Context.ACTIVITY_SERVICE);
            List<RunningAppProcessInfo> rApps = actMgr
                    .getRunningAppProcesses();
            List<ActivityManager.RunningTaskInfo> taskInfo = actMgr
                    .getRunningTasks(1);
            if ((rApps != null && rApps.size() >= 5)
                    || (taskInfo != null && taskInfo.size() >= 3)) {
                sGetTopUseNewMethod = GET_TOP_USE_NEW_METHOD_NO;
            } else {
                sGetTopUseNewMethod = GET_TOP_USE_NEW_METHOD_YEP;
            }
        }
    }
}
