package com.jb.ga0.commerce.util.topApp;

import java.util.ArrayList;
import java.util.List;

import com.jb.ga0.commerce.util.Machine;

import android.content.ComponentName;
import android.content.Context;
import android.text.TextUtils;

/**
 * @author wubinqi
 * 2016年4月7日
 */
public class TopStackUtils {
	
	/**
	 * 判断当前是否是桌面
	 * @param context
	 * @param launcherPackageNames
	 * @return
	 */
	public static boolean isHome(Context context) {
		List<String> launcherPackageNames = TopHelper.getLauncherPackageNames(context);
		if (null == launcherPackageNames || launcherPackageNames.isEmpty()) {
			return false;
		}
		return TopHelper.isBackToPkgAboveLolipop(context, launcherPackageNames, true);
	}
	
	/**
	 * {@linkplain #isTopStack(Context, List)}
	 */
	public static boolean isTopStack(Context context, String packageName) {
		if (TextUtils.isEmpty(packageName) || TextUtils.isEmpty(packageName.trim())) {
			return false;
		}
		ArrayList<String> pkgs = new ArrayList<String>();
		pkgs.add(packageName);
		return TopHelper.isBackToPkgAboveLolipop(context, pkgs, false);
	}
	
	/**
	 * 判断当前栈底应用是否在{@code launcherPackageNames}中
	 * @param context
	 * @param launcherPackageNames
	 * @return
	 */
	public static boolean isTopStack(Context context, List<String> launcherPackageNames) {
		if (null == launcherPackageNames || launcherPackageNames.isEmpty()) {
			return false;
		}
		return TopHelper.isBackToPkgAboveLolipop(context, launcherPackageNames, false);
	}
	
	/**
	 * 获取当前栈底包名
	 * @param context
	 * @return
	 */
	public static String getTopStackPackage(Context context) {
		if (!Machine.HAS_SDK_LOLLIPOP) {
			// 小于5.0
			return TopHelper.getFrontActivityAppPackageName(context);
		}
		ComponentName topActivity = getTopByUsageState(context);
		if (topActivity != null) {
			final String topPkgname = topActivity.getPackageName();
			if (!TextUtils.isEmpty(topPkgname)
					&& !TopHelper.INVALID_PACKAGE_NAME.equals(topPkgname)) {
				return topPkgname;
			}
		}
		return TopHelper.getTopPkgAboveLolipopByTrick(context);
	}
	
	/**
	 * 获取当前栈顶的应用
	 * {@literal 5.0以上需要Usage权限}
	 * @param context
	 * @return
	 */
	public static ComponentName getTopStack(Context context) {
		ComponentName topActivity = getTopByUsageState(context);
		// 5.0以下
		if (!Machine.HAS_SDK_LOLLIPOP) {
			topActivity = TopHelper.getTopActivity(context);
		}
		return topActivity;
	}
	
	/**
	 * 是否用读proc方式的检测栈底
	 */
	public static boolean isUseReadProcMethod(Context context) {
		if (!Machine.HAS_SDK_LOLLIPOP) {
			// 小于5.0
			return false;
		}
		TopHelper.checkMethod(context);
		if (TopHelper.GET_TOP_USE_NEW_METHOD_NO == TopHelper.sGetTopUseNewMethod) {
			return false;
		}
		ComponentName topActivity = getTopByUsageState(context);
		if (topActivity != null) {
			final String topPkgname = topActivity.getPackageName();
			if (!TextUtils.isEmpty(topPkgname)
					&& !TopHelper.INVALID_PACKAGE_NAME.equals(topPkgname)) {
				return false;
			}
		}
		return true;
	}
	
	static ComponentName getTopByUsageState(Context context) {
		ComponentName topActivity = null;
		// 5.1或以上
		if (Machine.HAS_SDK_5_1_1) {
			if (TopHelper
					.isPermissionPackageUsageStatsGrandedLollipopMr1(context)) {
				topActivity = TopHelper.getFrontActivityLollipopMr1(context);
			}
		}
		// 5.0
		else if (Machine.HAS_SDK_LOLLIPOP) {
			if (TopHelper
					.isPermissionPackageUsageStatsGrandedOnLollipop(context)) {
				topActivity = TopHelper.getFrontActivityOnLollipop(context);
			}
		}
		return topActivity;
	}
}
