package com.jiubang.commerce.buychannel;

import android.app.Application;
import android.content.Context;
import android.content.SharedPreferences;

import com.appsflyer.AppsFlyerConversionListener;
import com.appsflyer.AppsFlyerLib;
import com.gau.go.gostaticsdk.StatisticsManager;
import com.gau.go.gostaticsdk.utiltool.Machine;
import com.jb.ga0.commerce.util.LogUtils;
import com.jb.ga0.commerce.util.thread.CustomThreadExecutorProxy;
import com.jiubang.commerce.buychannel.buyChannel.Interface.SetBuyChannelListener;
import com.jiubang.commerce.buychannel.buyChannel.Statistics.StatisticsDebug;
import com.jiubang.commerce.buychannel.buyChannel.Statistics.UserStatistics;
import com.jiubang.commerce.buychannel.buyChannel.bean.UserTagParam;
import com.jiubang.commerce.buychannel.buyChannel.bean.UserTypeInfo;
import com.jiubang.commerce.buychannel.buyChannel.manager.GpVersionHelper;
import com.jiubang.commerce.buychannel.buyChannel.manager.ReferrerManager;
import com.jiubang.commerce.buychannel.buyChannel.manager.ServerCheckHelper;
import com.jiubang.commerce.buychannel.buyChannel.utils.BuyChannelUtils;
import com.jiubang.commerce.buychannel.buyChannel.utils.TextUtils;

import org.json.JSONObject;

import java.util.List;
import java.util.Map;
import java.util.Set;

import static com.jiubang.commerce.buychannel.buyChannel.bean.UserTypeInfo.FirstUserType.organic;

/**
 * integrate the AppsFlyer SDK<br>
 * 文档请参看: <a href=
 * "https://support.appsflyer.com/entries/44048785-Android-SDK-Download-Integration-Guide-Chinese"
 * >AppsFlyer</a><br>
 * AppsFlyer后台:<br>
 * https://hq1.appsflyer.com/auth/login<br>
 * 账户：gocommerce@gomo.com 密码：datasungymobile<br>
 *
 * @author mayunlong
 */
public class AppsFlyerProxy {
    private static AppsFlyerProxy sInstance;
    private Context mContext;

    @Override
    protected void finalize() throws Throwable {
        super.finalize();
    }

    private BuySdkInitParams mInitParams;
    public static final String FLYER_KEY = "o6XxR94NFNcyL6NTzsUrRG";
    private Runnable mUploard45Task;

    private SharedPreferences mSp;
    private Runnable mUpload15sTask;
    private List<String> mAdwordsGdnCampaignids;

    private AppsFlyerProxy() {
    }

    public BuySdkInitParams getInitParams() {
        return mInitParams;
    }

    public static AppsFlyerProxy getInstance() {
        if (sInstance == null) {
            synchronized (AppsFlyerProxy.class) {
                sInstance = new AppsFlyerProxy();
            }
        }
        return sInstance;
    }


    public void init(final Application application, final BuySdkInitParams params) {
        mContext = application.getApplicationContext();
        mInitParams = params;
        mAdwordsGdnCampaignids = params.mAdwordsGdnCampaignids;

        mSp = BuyChannelDataMgr.getInstance(mContext).getSharedPreferences(mContext);
        CustomThreadExecutorProxy.getInstance().runOnAsyncThread(new Runnable() {
            @Override
            public void run() {


                //保存查询用户标签接口的相关数据
                if (params != null) {
                    UserTagParam userTagParam = new UserTagParam(StatisticsManager.getGOID(mContext), null, params.mChannel + "", params.mProductKey, params.mAccessKey);
                    mSp.edit().putString(BuySdkConstants.USERTAG_PARAMS, userTagParam.toJsonStr()).commit();
                    mSp.edit().putString(BuySdkConstants.CID_45, params.mUsertypeProtocalCId).commit();
                    mSp.edit().putBoolean(BuySdkConstants.IS_GOKEYBORAD, params.mIsGoKeyboard).commit();
                    mSp.edit().putInt(BuySdkConstants.FUN_ID_45, params.mP45FunId).commit();
                }

                //开启服务端定时器，5天内每8小时检查去服务端check
                long lastCheckTime = BuyChannelDataMgr.getInstance(mContext).getLastCheckTime();

                long triggerTime = System.currentTimeMillis() - lastCheckTime;
                if (triggerTime > BuySdkConstants.INTERVAL_SERVER_CHECKTIME) {
                    triggerTime = BuySdkConstants.INTERVAL_SERVER_CHECKTIME;
                } else {
                    triggerTime = BuySdkConstants.INTERVAL_SERVER_CHECKTIME - triggerTime;
                }

                //检查定时器，并开启
                ServerCheckHelper.getInstance(mContext).cancelCheckServer();
                ServerCheckHelper.getInstance(mContext).startCheckServer(triggerTime);

                LogUtils.i(BuySdkConstants.LOG_TAG, "check定时器");

                upload15Second();
            }
        });


        //初始化APPFLYER方法
        initAf(application);

    }

    /**
     * 初始化APPFLYER方法
     * created by mayunlong.
     * created date 16-11-8.
     */
    private void initAf(Application application) {
        //收集andorid id,4.x的af初始化方法
        if (mInitParams.isSetImei) {
            AppsFlyerLib.getInstance().setImeiData(BuyChannelUtils.getIMEI(application));
        }
        try {
            AppsFlyerLib.getInstance().setAndroidIdData(Machine.getAndroidId(application.getApplicationContext()));
        } catch (Exception e) {

        }
        AppsFlyerLib.getInstance().startTracking(application, FLYER_KEY);
        AppsFlyerLib.getInstance().registerConversionListener(mContext, mAppsFlyerConversionListener);


        //3.X以下的af初始化方法
        //AppsFlyerLib.setAppsFlyerKey(FLYER_KEY);
        //AppsFlyerLib.registerConversionListener(mContext, mAppsFlyerConversionListener);
        //AppsFlyerLib.sendTracking(application.getApplicationContext());
    }


    private AppsFlyerConversionListener mAppsFlyerConversionListener = new AppsFlyerConversionListener() {

        @Override
        public void onInstallConversionDataLoaded(Map<String, String> conversionData) {
            if (conversionData == null || conversionData.isEmpty()) {
                return;
            }


            if (BuyChannelUtils.isOldApkBuy(mContext)) {
                LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::onInstallConversionDataLoaded] 缓存中已是APK买量，" +
                        "不再处理af的回调");
                return;
            }

            if (BuyChannelUtils.isOldUser(mContext)) {
                LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::onInstallConversionDataLoaded] 已经是老用户，" +
                        "不再处理af的回调");
                return;
            }

            boolean isFb = false;
            boolean isAdwords = false;
//            boolean isTwitter = false;
            boolean isOrganic = false;
            boolean isGaUserBuy = false;
            String buyChannel;

            String campaign = "";
            String adSet = "";
            String adGroup = "";
            String mediaSource = "";

            String agency = "";
            String afStatus = "";

            String campaignid = "";
            String campaign_id = ""; // CHECKSTYLE IGNORE
            // 这个是最终的
            String campaignId = "";

            String conversionDataJsonStr = "";

            final Set<String> attrNames = conversionData.keySet();

            for (String attrName : attrNames) {
                if (TextUtils.isEmpty(attrName)) {
                    continue;
                }
                String attrValue = conversionData.get(attrName);
                if (TextUtils.isEmpty(attrValue)) {
                    attrValue = "";
                }

                if (attrName.equals("is_fb")) {
                    isFb = Boolean.TRUE.toString().equalsIgnoreCase(attrValue);
                } else if (attrName.equals("campaign")) {
                    campaign = attrValue;
                } else if (attrName.equals("adset")) {
                    adSet = attrValue;
                } else if (attrName.equals("adgroup")) {
                    adGroup = attrValue;
                } else if (attrName.equals("media_source")) {
                    mediaSource = attrValue;
                } else if (attrName.equals("agency")) {
                    agency = attrValue;
                    if ("null".equalsIgnoreCase(agency)) {
                        agency = "";
                    }
                } else if (attrName.equals("af_status")) {
                    afStatus = attrValue;
                } else if (attrName.equals("campaignid")) {
                    campaignid = attrValue;
                } else if (attrName.equals("campaign_id")) {
                    campaign_id = attrValue;
                }
            }

            String afCampaignId = "null";
            if (!TextUtils.isEmpty(campaignid)) {
                afCampaignId = campaignid;
//                LogUtils.d(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::onInstallConversionDataLoaded]campaignid 不为空：" + campaignid);
            } else if (TextUtils.isEmpty(campaign_id)) {
                afCampaignId = campaign_id;
//                LogUtils.d(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::onInstallConversionDataLoaded]campaign_id 不为空：" + campaign_id);
            }


            // 由于有好多个具有"campaign id"意义的值, 按优先级campaignid > campaign > campaign_id取
            if (!TextUtils.isEmpty(campaignid)) {
                campaignId = campaignid;
            } else if (!TextUtils.isEmpty(campaign)) {
                campaignId = campaign;
            } else if (!TextUtils.isEmpty(campaign_id)) {
                campaignId = campaign_id;
            }

            //测试用，设置AdwordsGdnId
           /* String adwordsGdnId = mSp.getString("AdwordsGdnId", null);
            if (!TextUtils.isEmpty(adwordsGdnId)) {
                List<String> adwordsGdnIds = new ArrayList<>();
                adwordsGdnIds.add(adwordsGdnId);
                mAdwordsGdnCampaignids = adwordsGdnIds;
            }*/

            try {
                JSONObject json = new JSONObject(conversionData);
                conversionDataJsonStr = json.toString();
            } catch (Throwable e) {
                LogUtils.w(BuySdkConstants.LOG_TAG, "warning-->", e);
            }
            LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::onInstallConversionDataLoaded] AppsFlyer原始数据，" +
                    "conversionDataJsonStr: " + conversionDataJsonStr);

            //15内appflyer回调，取消15秒任务
            cancelupload15S(null, conversionDataJsonStr);

            buyChannel = mediaSource;
            if (isFb) {
                buyChannel = "fb";
            }

            if (!isFb && "Facebook Ads".equalsIgnoreCase(mediaSource)) {
                isFb = true;
                buyChannel = "fb";
            }
            // adwords,此处最后一个判断规则可能存在风险，排错时需重点观察
            if (!isFb && ("adwords".equalsIgnoreCase(mediaSource) || "googleadwords_int".equalsIgnoreCase(mediaSource)
                    || (!isAgencyEmpty(agency) && TextUtils.isEmpty(mediaSource)))) {
                isAdwords = true;
                buyChannel = "adwords";
            }

            // twitter
//            if (!isFb && "twitter".equalsIgnoreCase(mediaSource)) {
//                isTwitter = true;
//                buyChannel = "twitter";
//            }
//

            /*// Organic
            if (!isFb && ("organic".equalsIgnoreCase(afStatus))) {
                isOrganic = true;

            }*/
            //Orgnic

            if (TextUtils.isEmpty(adGroup)) {
                adGroup = conversionData.get("adgroup_name");
            }
            if (TextUtils.isEmpty(adSet)) {
                adSet = conversionData.get("adset_name");
            }


            // 提取全部数据用来上传服务器，将referer和appsflyer数据进行上传,通过45协议进行上传
            final String referrer = ReferrerManager.getInstance(mContext).getReferrer();

            UserTypeInfo.FirstUserType userType = null;
            UserTypeInfo.SecondUserType secondUserType = null;
            String associatedObj = null;
            if (isFb) {
                associatedObj = UserStatistics.get45AssociatedObjFB(campaign, adSet, adGroup);
                //fb自投
                userType = UserTypeInfo.FirstUserType.userbuy;
                secondUserType = UserTypeInfo.SecondUserType.FB_AUTO;
                if ("Facebook Ads".equalsIgnoreCase(mediaSource) && campaign.toLowerCase().startsWith("gomo")) {
                    //fb自投
//                    LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::onInstallConversionDataLoaded]Appfly识别为Fb自投买量,上传45协议:"
//                            + associatedObj + ";campaign数据:" + campaign + ";Appfly原始数据:"
//                            + conversionDataJsonStr + ";GA原始数据:" + referrer + "，二级类型为：" + secondUserType);


                } else if ("Facebook Ads".equalsIgnoreCase(mediaSource) && !campaign.toLowerCase().startsWith("gomo")) {

                    userType = UserTypeInfo.FirstUserType.userbuy;
                    secondUserType = UserTypeInfo.SecondUserType.FB_NOTAUTO;
                    //fb非自投
//                    LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::onInstallConversionDataLoaded],Appfly识别为Fb非自投买量,上传45协议:"
//                            + associatedObj + ";campaign数据:" + campaign + ";Appfly原始数据:"
//                            + conversionDataJsonStr + ";GA原始数据:" + referrer + "，二级类型为：" + secondUserType);
                }

            } else if (isAdwords) {

                //Adwords自投
                if ("googleadwords_int".equalsIgnoreCase(mediaSource)) {

                    userType = UserTypeInfo.FirstUserType.userbuy;
                    secondUserType = UserTypeInfo.SecondUserType.ADWORDS_AUTO;
//                    LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::onInstallConversionDataLoaded],Appfly识别为Adwords自投买量,上传45协议:"
//                            + associatedObj + ";mediaSource数据:" + mediaSource + ";Appfly原始数据:"
//                            + conversionDataJsonStr + ";GA原始数据:" + referrer + "，二级类型为：" + secondUserType);

                } else if (!isAgencyEmpty(agency) && TextUtils.isEmpty(mediaSource)) {

                    userType = UserTypeInfo.FirstUserType.userbuy;
                    secondUserType = UserTypeInfo.SecondUserType.ADWORDS_NOTAUTO;
//                    LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::onInstallConversionDataLoaded]" +
//                            ",Appfly识别为Adwords非自投买量,上传45协议:" + associatedObj + ";mediaSource数据:"
//                            + mediaSource + ";Appfly原始数据:" + conversionDataJsonStr + ";GA原始数据:" + referrer
//                            + "，二级类型为：" + secondUserType);
                }

                if (mAdwordsGdnCampaignids != null && mAdwordsGdnCampaignids.contains(campaignId)) {
                    buyChannel = "adwords_gdn";
                    associatedObj = UserStatistics.get45AssociatedObjAdwords(campaign, true);
                    userType = UserTypeInfo.FirstUserType.userbuy;
                    secondUserType = UserTypeInfo.SecondUserType.ADWORDS_AUTO;
                } else {
                    buyChannel = "adwords";
                    associatedObj = UserStatistics.get45AssociatedObjAdwords(campaign, false);
                }


            }
            /*else if (isTwitter) {
   *//*             associatedObj = UserStatistics.get45AssociatedObjTwitter(campaign);
                userType = UserTypeInfo.FirstUserType.userbuy;
                secondUserType = null;
                LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::onInstallConversionDataLoaded],Appfly识别为Twitter买量,上传45协议:" + associatedObj + ";Appfly原始数据:" + conversionDataJsonStr + ";GA原始数据:" + referrer);*//*
            }*/
            // appsflyer不能识别的, 但仍需要将referrer和appsflyer原始数据上传到45协议
            else if (!TextUtils.isEmpty(mediaSource) && isAgencyEmpty(agency) && !isOrganic) {
                isGaUserBuy = true;
                //mediaSource不为空,默认为买量,上传mediaSource
                associatedObj = UserStatistics.get45AssociatedObjOther(mediaSource, campaign, adSet, adGroup);
                userType = UserTypeInfo.FirstUserType.userbuy;
                secondUserType = UserTypeInfo.SecondUserType.GA_USERBUY;

                LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::onInstallConversionDataLoaded]mediaSource不为空,判断为GA买量" +
                        ";Appfly原始数据:" + conversionDataJsonStr + ";GA原始数据:" + referrer);
            } else {//&& BuyChannelApi.getBuyChannelBean(mContext) == null

                userType = organic;
                secondUserType = UserTypeInfo.SecondUserType.GP_ORGNIC;
//                associatedObj = UserStatistics.get45AssociatedObjOther(mediaSource, campaign, adSet, adGroup);
                associatedObj = UserStatistics.get45AssociatedEmptyOther(mediaSource);
                uploadFirst45(referrer, conversionDataJsonStr, BuySdkConstants.UNKNOWN_BUYCHANNEL, associatedObj, agency, userType, secondUserType, campaign, afCampaignId);
                return;
            }


            final String finalConversionDataJsonStr = conversionDataJsonStr;
            final String finalAssociatedObj = associatedObj;
            final String finalAgency = agency;
            final UserTypeInfo.SecondUserType finalSecondUserType = secondUserType;
            final String finalBuyChannel = buyChannel;
            final String finalConversionDataJsonStr1 = conversionDataJsonStr;
            final String finalAssociatedObj1 = associatedObj;

            BuyChannelSetting.getInstance(mContext).setBuyChannel(buyChannel, BuyChannelSetting.ChannelFrom.from_appsflyer, userType,
                    secondUserType, campaign, afCampaignId, associatedObj, agency, conversionDataJsonStr, referrer, null, new SetBuyChannelListener() {
                        @Override
                        public void setBuyChannelSuccess() {
                            mSp.edit().putBoolean(BuySdkConstants.APPFLY_UPLOARD, false).commit();

                            //操作码af_receive,位置１，覆盖原有的buychannel
                            StatisticsDebug.Statistic103Params statistic103Params = new StatisticsDebug.Statistic103Params();
                            statistic103Params.code(BuySdkConstants.DEBUG_CODE4).referrer(referrer).buychannel(finalBuyChannel).position(1).appflyer(finalConversionDataJsonStr1);
                            StatisticsDebug.upload(mContext, statistic103Params);

                            mSp.edit().putString(BuySdkConstants.ASSOCIATE_103, finalAssociatedObj1).commit();

                        }

                        @Override
                        public void setBuyChannelFailure(String position) {
                            //判断是否第一次上传appflyer的45协议
                            if (mSp.getBoolean(BuySdkConstants.APPFLY_UPLOARD, true)) {
                                //说明是第一次af，如果第一次af中setBuyChannel失败，则上传一次45
                                uploard45(finalAssociatedObj, finalConversionDataJsonStr, referrer, finalSecondUserType.getValue() + "", "-1", finalAgency, mInitParams);
                                BuyChannelDataMgr.getInstance(mContext).setConversionStr(finalConversionDataJsonStr);
                                mSp.edit().putBoolean(BuySdkConstants.APPFLY_UPLOARD, false).commit();
                                LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelSetting::uploard45] 上传debug 45协议，保存第一次af的原始数据");

                            }

                            String oldAssociateObj = mSp.getString(BuySdkConstants.ASSOCIATE_103, null);
                            if (oldAssociateObj != null && oldAssociateObj.equals(finalAssociatedObj)) {

                            } else {

                                //操作码af_receive,位置0，没有覆盖原有的buychannel
                                StatisticsDebug.Statistic103Params statistic103Params = new StatisticsDebug.Statistic103Params();
                                statistic103Params.code(BuySdkConstants.DEBUG_CODE4).referrer(referrer).buychannel(finalBuyChannel).position(0).appflyer(finalConversionDataJsonStr1);
                                StatisticsDebug.upload(mContext, statistic103Params);

                                mSp.edit().putString(BuySdkConstants.ASSOCIATE_103, finalAssociatedObj1).commit();
                            }

                        }
                    });
        }

        @Override
        public void onInstallConversionFailure(String errorMessage) {
            LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::onInstallConversionFailure] errorMessage: " + errorMessage);
        }

        @Override
        public void onAppOpenAttribution(
                Map<String, String> attributionData) {
            if (LogUtils.isShowLog()) {
                String jsonStr = "";
                try {
                    JSONObject json = new JSONObject(attributionData);
                    jsonStr = json.toString();
                } catch (Throwable e) {
                    LogUtils.w(BuySdkConstants.LOG_TAG, "warning-->", e);
                }
                LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::onAppOpenAttribution] attributionData: " + jsonStr);
            }
        }

        @Override
        public void onAttributionFailure(String errorMessage) {
            LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::onAttributionFailure] errorMessage: " + errorMessage);
        }
    };


    private void uploadFirst45(final String referrer, final String conversionDataJsonStr, final String buyChannel, final String associatedObj,
                               final String agency, final UserTypeInfo.FirstUserType userType,
                               final UserTypeInfo.SecondUserType secondUserType, final String campaign, final String campaignId) {
//        LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::onInstallReferrer] referrer: " + referrer);
        if (BuyChannelUtils.isOldApkBuy(mContext) || BuyChannelUtils.isOldUser(mContext) || !mSp.getBoolean(BuySdkConstants.APPFLY_UPLOARD, true)) {
            return;
        }
        if (mUploard45Task != null) {
            CustomThreadExecutorProxy.getInstance().cancel(mUploard45Task);
            mUploard45Task = null;
        }
        mUploard45Task = new Runnable() {
            @Override
            public void run() {
                BuyChannelSetting.getInstance(mContext).setBuyChannel(buyChannel, BuyChannelSetting.ChannelFrom.from_appsflyer, userType,
                        secondUserType, campaign, campaignId, associatedObj, agency, conversionDataJsonStr, referrer, null, new SetBuyChannelListener() {
                            @Override
                            public void setBuyChannelSuccess() {
                                mSp.edit().putBoolean(BuySdkConstants.APPFLY_UPLOARD, false).commit();


                                //操作码af_receive,位置１，覆盖原有的buychannel
                                StatisticsDebug.Statistic103Params statistic103Params = new StatisticsDebug.Statistic103Params();
                                statistic103Params.code(BuySdkConstants.DEBUG_CODE4).referrer(referrer).buychannel(buyChannel).position(1).appflyer(conversionDataJsonStr);
                                StatisticsDebug.upload(mContext, statistic103Params);

                                mSp.edit().putString(BuySdkConstants.ASSOCIATE_103, associatedObj).commit();


                            }

                            @Override
                            public void setBuyChannelFailure(String position) {
                                //判断是否第一次上传appflyer的45协议
                                if (mSp.getBoolean(BuySdkConstants.APPFLY_UPLOARD, true)) {
                                    //说明是第一次af，如果第一次af中setBuyChannel失败，则上传一次45

                                    uploard45(associatedObj, conversionDataJsonStr, referrer, secondUserType.getValue() + "", "-1", agency, mInitParams);
                                    BuyChannelDataMgr.getInstance(mContext).setConversionStr(conversionDataJsonStr);

                                    mSp.edit().putBoolean(BuySdkConstants.APPFLY_UPLOARD, false).commit();
                                    LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelSetting::uploard45] 上传debug 45协议，保存第一次af的原始数据");
                                }


                                String oldAssociateObj = mSp.getString(BuySdkConstants.ASSOCIATE_103, null);
                                if (oldAssociateObj != null && oldAssociateObj.equals(associatedObj)) {

                                } else {

                                    //操作码af_receive,位置0，没有覆盖原有的buychannel
                                    StatisticsDebug.Statistic103Params statistic103Params = new StatisticsDebug.Statistic103Params();
                                    statistic103Params.code(BuySdkConstants.DEBUG_CODE4).referrer(referrer).buychannel(buyChannel).position(0).appflyer(conversionDataJsonStr);
                                    StatisticsDebug.upload(mContext, statistic103Params);

                                    mSp.edit().putString(BuySdkConstants.ASSOCIATE_103, associatedObj).commit();
                                }

                            }
                        });
//                LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::uploadFirst45],Appfly识别为自然,且GA没有广播，则认定GP自然，" +
//                        "上传45协议，Appfly原始数据:" + conversionDataJsonStr + ";GA原始数据:" + referrer);
                mUploard45Task = null;
            }

        };
//        LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::uploadFirst45] 15s后，没有GA广播，af又识别为自然，则以af自然为准");
        // 等待15s, ga没回调, 就上传AppFlyer的自然数据到45协议了
        CustomThreadExecutorProxy.getInstance().runOnAsyncThread(mUploard45Task, BuySdkConstants.TIME_APPFLY_UPLOARD);


    }

    public void cancelUploadFirst45(String referrer) {
        //15内有GA回调，取消15秒任务
        if (mUploard45Task != null) {
            CustomThreadExecutorProxy.getInstance().cancel(mUploard45Task);
            mUploard45Task = null;


            LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::cancelUploadFirst45]");
        }
    }

    private void uploard45(String associatedObj, String conversionDataJsonStr, String referrer, String secondUserType, String tag, String afAgency, BuySdkInitParams mInitParams) {

        //BuySdkInitParams mInitParams = AppsFlyerProxy.getInstance().getInitParams();
        UserStatistics.upload45(mContext, mInitParams.mIsGoKeyboard, mInitParams.mP45FunId,
                associatedObj, conversionDataJsonStr, referrer, secondUserType, tag, afAgency, false);
        //mSp.edit().putString(BuySdkConstants.CONVERSIONDATA, conversionDataJsonStr).commit();
    }

    private boolean isAgencyEmpty(String agency) {
        if (TextUtils.isEmpty(agency)) {
            return true;
        }
        if (agency.equalsIgnoreCase("null")) {
            return true;
        }
        return false;
    }


    /**
     * 15s内未返回appflyer或GA数据，则调用此方法，上传一条自然的45协议
     * created by mayunlong.
     * created date 16-10-21.
     */
    private void upload15Second() {


        if (mUpload15sTask != null) {
            CustomThreadExecutorProxy.getInstance().cancel(mUpload15sTask);
            mUpload15sTask = null;
        }

        mUpload15sTask = new Runnable() {
            @Override
            public void run() {
                if (BuyChannelUtils.isOldUser(mContext) || BuyChannelUtils.isOldApkBuy(mContext)) {
                    return;
                }
               /* uploard45(null, null, null, null, -1, null);*/
                UserTypeInfo.FirstUserType userType = null;
                UserTypeInfo.SecondUserType sUserType = null;
                //既没有appflyer,又没有GA，需要根据gp版本区分自然
                if (GpVersionHelper.getInstance(mContext).campareWhitGpVersion6_8_24()) {
                    //没有Ga广播和af信息且gp版本≥6.8.24,判定为非GP自然
                    userType = organic;
                    sUserType = UserTypeInfo.SecondUserType.NOT_GP_ORGNIC;
                    LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::upload15Second] 15内，" +
                            "未返回appflyer和GA数据, 没有GA广播和af信息且gp版本≥6.8.24,判定为非GP自然");
                } else {
                    //没有Ga广播和af信息且gp版本<6.8.24或为gp版本空，判定为GP自然
                    userType = organic;
                    sUserType = UserTypeInfo.SecondUserType.GP_ORGNIC;
                    LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::upload15Second] 15内，" +
                            "未返回appflyer和GA数据, 没有Ga广播和af信息且gp版本<6.8.24或为gp版本空，判定为GP自然," +
                            "判定为GP自然");
                }

                String associatedObj = UserStatistics.get45AssociatedEmptyOther(BuySdkConstants.UNKNOWN_BUYCHANNEL);

                BuyChannelSetting.getInstance(mContext).setBuyChannel(BuySdkConstants.UNKNOWN_BUYCHANNEL, BuyChannelSetting.ChannelFrom.un_known,
                        userType, sUserType, null, null, associatedObj, null, null, null, null, null);
                mUpload15sTask = null;
            }
        };
        LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::upload15Second] 15内，" +
                "未返回appflyer和GA数据, 则判断为自然用户");
        CustomThreadExecutorProxy.getInstance().runOnAsyncThread(mUpload15sTask, BuySdkConstants.TIME_WAITFOR_APPSFLYER_RESULE);
    }

    public void cancelupload15S(String referrer, String conversionDataJsonStr) {
        //15内appflyer回调，取消15秒任务
        if (mUpload15sTask != null) {
            CustomThreadExecutorProxy.getInstance().cancel(mUpload15sTask);
            mUpload15sTask = null;

            LogUtils.i(BuySdkConstants.LOG_TAG, "[AppsFlyerProxy::cancelupload15S] ");
        }
    }


}
