package com.jiubang.commerce.buychannel;

import android.app.Application;
import android.content.Context;
import android.content.SharedPreferences;

import com.jb.ga0.commerce.util.AppUtils;
import com.jb.ga0.commerce.util.DevHelper;
import com.jb.ga0.commerce.util.LogUtils;
import com.jb.ga0.commerce.util.thread.CustomThreadExecutorProxy;
import com.jiubang.commerce.buychannel.buyChannel.Statistics.UserStatistics;
import com.jiubang.commerce.buychannel.buyChannel.bean.BuyChannelBean;
import com.jiubang.commerce.buychannel.buyChannel.bean.UserTypeInfo;
import com.jiubang.commerce.buychannel.buyChannel.manager.InitManager;
import com.jiubang.commerce.buychannel.buyChannel.utils.AppInfoUtils;
import com.jiubang.commerce.buychannel.buyChannel.utils.TextUtils;

/**
 * 买量SDK入口类
 * Created by chenchongji on 2016/6/15.
 *
 * @备注 支持多进程
 */
public class BuyChannelApi {
    /**
     * 买量SDK版本名
     */
    public static final String sVersionName = BuySdkConstants.SDK_VERSION_NAME;
    /**
     * 买量SDK版本号
     */
    public static final int sVersionCode = BuySdkConstants.SDK_VERSION_CODE;

    private static Context sContext;
    private static InitManager sManager;

    /**
     * 开启LOG，放在init方法之前
     */
    public static void setDebugMode() {
        LogUtils.setShowLog(true);
    }

    /**
     * 初始化，在Application::onCreate里调用
     *
     * @param application
     * @param params      买量SDK所需要的相关参数, 通过BuySdkInitParams构造函数创建
     */
    public static void init(final Application application, final BuySdkInitParams params) {
        LogUtils.d(BuySdkConstants.LOG_TAG, "[BuyChannelApi::init]mChannel:" + params.mChannel + ",mP45FunId:" + params.mP45FunId
                + ",mUsertypeProtocalCId:" + params.mUsertypeProtocalCId + ",mIsOldUserWithoutSdk:" + params.mIsOldUserWithoutSdk
                + ",mOldBuyChannel:" + params.mOldBuyChannel + ",mIsGoKeyboard:" + params.mIsGoKeyboard);
        //主要是设置开发助手的配置文件路径
        DevHelper.setRegister(application.getPackageName());
        sContext = application.getApplicationContext();
        CustomThreadExecutorProxy.getInstance().runOnAsyncThread(new Runnable() {
            @Override
            public void run() {
                sManager = InitManager.getInstance(sContext);
                //设置统计SDK的监听
                sManager.setStatisticStateListener();
                //检查本地是否还有缓存的45协议数据
                sManager.check45(2);

                //开子线程获取google广告ID
                AppInfoUtils.getAdvertisingId(sContext);

                SharedPreferences mSp = BuyChannelDataMgr.getInstance(sContext).getSharedPreferences(sContext);

                mSp.edit().putBoolean(BuySdkConstants.IS_GOKEYBORAD, params.mIsGoKeyboard).commit();
                mSp.edit().putInt(BuySdkConstants.FUN_ID_45, params.mP45FunId).commit();

                int channel = params.mChannel;
                //判断apk买量的条件，渠道号10000-19999为apk买量
                if (channel > 9999 && channel < 20000) {
                    String associatedObj = null;
                    String apkBuyChannel = "buychannel_apk_" + channel;
                    if (params.mIsApkUpLoad45) {
                        //apk买量，mIsApkUpLoad45参数为true的情况，则要上传45协议,所以手动拼接
                        associatedObj = UserStatistics.get45AssociatedObjOther(apkBuyChannel, null, channel + "", null);
                    }
                    //判断为apk买量
                    BuyChannelSetting.getInstance(sContext).setBuyChannel(apkBuyChannel,
                            BuyChannelSetting.ChannelFrom.from_client, UserTypeInfo.FirstUserType.apkbuy,
                            UserTypeInfo.SecondUserType.APK_USERBUY, null, null, associatedObj, null, null, null, null, null);
                }

                //判断是不是老用户
                if (params.mIsOldUserWithoutSdk && !sManager.isUpdateBuyChannelSdk()) {
                    sManager.updateOldUser(sContext, params.mOldBuyChannel, params.mIsOldUserWithoutSdk, params.mUsertypeProtocalCId);

                }

                sManager.saveSdkVersion();
            }


        });
        String pkgName = params.mProcessName != null ? params.mProcessName : application.getPackageName();
        String currentProcess = AppUtils.getCurrProcessName(application);
        if (LogUtils.isShowLog()) {
            LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelApi::init] pkgName:" + pkgName + ", currentProcess:" + currentProcess);
        }
        if (params.mIsOldUserWithoutSdk && TextUtils.isEmpty(params.mOldBuyChannel)) {
            LogUtils.w(BuySdkConstants.LOG_TAG, "[BuyChannelApi::init] 老用户的买量渠道不应该空！");
        }
        if (!params.mIsOldUserWithoutSdk && !TextUtils.isEmpty(params.mOldBuyChannel)) {
            LogUtils.w(BuySdkConstants.LOG_TAG, "[BuyChannelApi::init] 新用户不应该有买量渠道！");
        }
        if (pkgName != null && pkgName.equals(currentProcess)) {
            LogUtils.d(BuySdkConstants.LOG_TAG, "[BuyChannelApi::init]: currentProcess:" + currentProcess.toString());
            AppsFlyerProxy.getInstance().init(application, params);

        }

    }


    /**
     * 注册买量渠道名称监听
     *
     * @param context
     * @param listener
     */
    public static void registerBuyChannelListener(Context context, IBuyChannelUpdateListener listener) {
        if (LogUtils.isShowLog()) {
            LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelApi::registerBuyChannelListener] listener:" + listener.getClass().getName());
        }
        BuyChannelDataMgr.getInstance(context).registerBuyChannelListener(listener);
    }

    /**
     * 注销买量渠道名称监听
     *
     * @param context
     * @param listener
     */
    public static void unregisterBuyChannelListener(Context context, IBuyChannelUpdateListener listener) {
        if (LogUtils.isShowLog()) {
            LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelApi::unregisterBuyChannelListener] listener:" + listener.getClass().getName());
        }
        BuyChannelDataMgr.getInstance(context).unregisterBuyChannelListener(listener);
    }

    /**
     * 获取买量渠道的对象
     *
     * @param context
     * @param
     * @return
     */
    public static BuyChannelBean getBuyChannelBean(Context context) {

        BuyChannelBean buyChannelBean = BuyChannelDataMgr.getInstance(context).getBuyChannelBean();

        if (LogUtils.isShowLog()) {
            if (buyChannelBean != null) {
                LogUtils.i(BuySdkConstants.LOG_TAG, "获取buyChannel,[BuyChannelApi::getBuyChannelBean] BuyChannelBean:" + buyChannelBean.toString());
            }

        }
        if (buyChannelBean != null) {
            return buyChannelBean;
        }
        return new BuyChannelBean();

    }

    /**
     * 当给其他产品带量时，请务必调用该接口，买量SDK会对带量链接进行拼接
     *
     * @param context
     * @param url     带量链接url
     * @return String 经过买量SDK拼接后的带量链接
     */
    public static String transformUrl(Context context, String url) {

        String userType = null;
        BuyChannelBean buyChannelBean = BuyChannelDataMgr.getInstance(context).getBuyChannelBean();
        if (buyChannelBean != null) {
            userType = buyChannelBean.getFirstUserType();
        }

        StringBuffer buffer = new StringBuffer();
        buffer.append(url);

        buffer.append(BuySdkConstants.SEPARATOR);
        buffer.append("from_3g_channel%3d");
        buffer.append(userType);

        LogUtils.d(BuySdkConstants.LOG_TAG, "[BuyChannelApi::transformUrl]带量拼接链接：" + buffer.toString());
        return buffer.toString();


    }


    /**
     * 如果错过了初始化时设置老用户，可以调用此方法进行补传
     *
     * @param oldBuyChannel       未接买量SDK之前的老用户的买量渠道名称，新用户直接传null或不传
     * @param isOldUserWithoutSdk 是否未接买量SDK之前的老用户
     */
    public static void setOldUser(String oldBuyChannel, boolean isOldUserWithoutSdk) {
        LogUtils.d(BuySdkConstants.LOG_TAG, "[BuyChannelApi::setOldUser]调用设置老用户接口：oldBuyChannel：" + oldBuyChannel + ",isOldUserWithoutSdk:" + isOldUserWithoutSdk);
        InitManager manager = InitManager.getInstance(sContext);
        if (isOldUserWithoutSdk && !manager.isUpdateBuyChannelSdk()) {
            manager.updateOldUser(sContext, oldBuyChannel, isOldUserWithoutSdk, AppsFlyerProxy.getInstance().getInitParams().mUsertypeProtocalCId);
        }
    }

    /**
     * 获取GA广播referrer的原始数据
     */
    public static String getReferrer(Context context) {
        SharedPreferences sp = BuyChannelDataMgr.getInstance(context).getSharedPreferences(context);
        String referrer = sp.getString(BuySdkConstants.REFERRER, null);
        LogUtils.d(BuySdkConstants.LOG_TAG, "[BuyChannelApi::getReferrer]：Referrer：" + referrer);
        return referrer;
    }
}
