package com.jiubang.commerce.buychannel;

import android.content.Context;
import android.content.SharedPreferences;

import com.jb.ga0.commerce.util.DevHelper;
import com.jb.ga0.commerce.util.LogUtils;
import com.jiubang.commerce.buychannel.buyChannel.Interface.SetBuyChannelListener;
import com.jiubang.commerce.buychannel.buyChannel.bean.BuyChannelBean;
import com.jiubang.commerce.buychannel.buyChannel.bean.UserTypeInfo;
import com.jiubang.commerce.buychannel.buyChannel.utils.BuyChannelUtils;
import com.jiubang.commerce.buychannel.buyChannel.utils.TextUtils;

import java.util.ArrayList;

/**
 * Created by chenchongji on 2016/6/16.
 */
public class BuyChannelDataMgr {
    //配置文件
    private static final String FILE_NAME = "commerce_buychannel";
    //买量渠道名称存储
    private static final String KEY_BUYCHANNEL = "buychannel";

    //最近一次check服务器的时间
    private static final String KEY_LAST_CHECKTIME = "last_checktime";
    //第一次check服务器的时间
    private static final String KEY_FIRST_CHECKTIME = "first_checktime";
    //老用户第一次启动定时查找用户标签表的时间
    private static final String USERTAG_FIRST_CHECKTIME = "usertag_first_checktime";

    private static BuyChannelDataMgr sInstance;
    private SharedPreferences mSp;
    private ArrayList<IBuyChannelUpdateListener> mListeners = new ArrayList<IBuyChannelUpdateListener>(2);
    private byte[] mLocker = new byte[0];


    private BuyChannelDataMgr(Context context) {

        mSp = MPSharedPreferences.getSharedPreferences(context, FILE_NAME, Context.MODE_PRIVATE);
        mSp.registerOnSharedPreferenceChangeListener(
                new SharedPreferences.OnSharedPreferenceChangeListener() {
                    @Override
                    public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String key) {
                        if (KEY_BUYCHANNEL.equals(key)) {
                            notifyBuyChannelUpdate();
                        }
                    }
                }
        );

    }

    public static BuyChannelDataMgr getInstance(Context context) {
        if (sInstance == null) {
            synchronized (BuyChannelDataMgr.class) {
                if (sInstance == null) {
                    sInstance = new BuyChannelDataMgr(context.getApplicationContext());
                }
            }
        }
        return sInstance;
    }

    public void registerBuyChannelListener(IBuyChannelUpdateListener listener) {
        if (listener == null) {
            return;
        }
        synchronized (mLocker) {
            if (!mListeners.contains(listener)) {
                mListeners.add(listener);
            }
        }
    }

    public void unregisterBuyChannelListener(IBuyChannelUpdateListener listener) {
        if (listener == null) {
            return;
        }
        synchronized (mLocker) {
            mListeners.remove(listener);
        }
    }

    public String getBuyChannel() {
        return mSp.getString(KEY_BUYCHANNEL, null);
    }

    public BuyChannelBean getBuyChannelBean() {
        BuyChannelBean testBuyChannelBean = getTestBuyChannelBean();
        if (testBuyChannelBean != null) {
            return testBuyChannelBean;
        }
        return BuyChannelUtils.jsonStr2BuyChannelBean(mSp.getString(KEY_BUYCHANNEL, null));
    }

    private void setBuyChannel(BuyChannelBean newBuyChannelBean) {
        if (newBuyChannelBean == null) {
            return;
        }
        mSp.edit().putString(KEY_BUYCHANNEL, newBuyChannelBean.toJsonStr()).commit();

    }


    /**
     * 获取开发助手中模拟的买量渠道
     *
     * @return
     */
    private BuyChannelBean getTestBuyChannelBean() {
        BuyChannelBean buyChannelBean = null;
        try {
            DevHelper devHelper = new DevHelper();
            String testbuyChannel = devHelper.getValueByKey("testBuyChannel");
            //DevHelper配置文件的格式：testBuyChannel=channelFrom,buyChannel,firstUserType,1
            if (!TextUtils.isEmpty(testbuyChannel)) {
                String[] testString = testbuyChannel.split(",");
                if (testString.length == 4) {
                    buyChannelBean = new BuyChannelBean();
                    buyChannelBean.setChannelFrom(testString[0]);
                    buyChannelBean.setBuyChannel(testString[1]);
                    buyChannelBean.setFirstUserType(testString[2]);
                    buyChannelBean.setSecondUserType(Integer.parseInt(testString[3]));
                    return buyChannelBean;
                }

            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return buyChannelBean;
    }

    //保存appflyer原始数据到本地
    public void setConversionStr(String conversionDataJsonStr) {
        if (!TextUtils.isEmpty(conversionDataJsonStr)) {
            mSp.edit().putString(BuySdkConstants.CONVERSIONDATA, conversionDataJsonStr).commit();
        }

    }

    /**
     * 把buyChannel写到SP中
     * created by mayunlong.
     * created date 16-10-19.
     */
    public void setBuyChannelBean(String buyChannel, BuyChannelSetting.ChannelFrom channel, UserTypeInfo.FirstUserType userType,
                                  UserTypeInfo.SecondUserType juserType, String conversionDataJsonStr,
                                  SetBuyChannelListener listener, String campaign, String campaignId) {
        if (userType == null || channel == null || juserType == null) {
            return;
        }
        if (listener != null) {
            listener.setBuyChannelSuccess();
        }
        BuyChannelBean newBuyChannelBean = new BuyChannelBean();
        newBuyChannelBean.setBuyChannel(buyChannel);
        newBuyChannelBean.setFirstUserType(userType.toString());
        newBuyChannelBean.setChannelFrom(channel.toString());
        newBuyChannelBean.setSecondUserType(juserType.getValue());
        newBuyChannelBean.setSuccessCheck(true);
        newBuyChannelBean.setCampaign(campaign);
        newBuyChannelBean.setCampaignId(campaignId);
        setBuyChannel(newBuyChannelBean);
        setConversionStr(conversionDataJsonStr);
        LogUtils.i(BuySdkConstants.LOG_TAG, "setBuyChannel完毕,[BuyChannelDataMgr::setBuyChannelBean] :buyChannel="
                + buyChannel + ",一级用户类型=" + userType.toString() + ",二级用户类型=" + juserType.getValue() + ",识别来源=" + channel.toString()
                + "买量SDK是否已经成功确认用户身份 " + true);

    }


   /* public long get19LogTime() {
        return mSp.getLong(KEY_19_LOGTIME, 0);
    }

    public void set19LogTime(long logTime) {
        mSp.edit().putLong(KEY_19_LOGTIME, logTime).commit();
    }*/


    public long getLastCheckTime() {
        return mSp.getLong(KEY_LAST_CHECKTIME, 0);
    }

    public void setLastCheckTime(long lastCheckTime) {
        mSp.edit().putLong(KEY_LAST_CHECKTIME, lastCheckTime).commit();
    }

    public long getFirstCheckTime() {
        return mSp.getLong(KEY_FIRST_CHECKTIME, 0);
    }

    public long getFirstCheckUserTagTime() {
        return mSp.getLong(USERTAG_FIRST_CHECKTIME, 0);
    }

    public void saveFirstCheckUserTagTime(long firstCheckTime) {

        mSp.edit().putLong(USERTAG_FIRST_CHECKTIME, firstCheckTime).commit();
    }


    public void setFirstCheckTime(long firstCheckTime) {
        mSp.edit().putLong(KEY_FIRST_CHECKTIME, firstCheckTime).commit();
    }


    private void notifyBuyChannelUpdate() {
        ArrayList<IBuyChannelUpdateListener> listeners = null;
        synchronized (mLocker) {
            listeners = (ArrayList<IBuyChannelUpdateListener>) mListeners.clone();
        }
        BuyChannelBean buyChannelBean = getBuyChannelBean();
        String buyChannel = buyChannelBean.getBuyChannel();
        for (IBuyChannelUpdateListener listener : listeners) {
            if (listener != null) {
                listener.onBuyChannelUpdate(buyChannel);
            }
        }
    }

    public SharedPreferences getSharedPreferences(Context context) {
        if (mSp != null) {
            return mSp;
        }
        mSp = MPSharedPreferences.getSharedPreferences(context, FILE_NAME, Context.MODE_PRIVATE);
        return mSp;
    }
}
