package com.jiubang.commerce.buychannel;

import android.content.Context;
import android.content.SharedPreferences;
import android.text.TextUtils;

import com.jb.ga0.commerce.util.CustomAlarmManager;
import com.jb.ga0.commerce.util.LogUtils;
import com.jb.ga0.commerce.util.thread.CustomThreadExecutorProxy;
import com.jiubang.commerce.buychannel.buyChannel.Interface.SetBuyChannelListener;
import com.jiubang.commerce.buychannel.buyChannel.Statistics.StatisticsDebug;
import com.jiubang.commerce.buychannel.buyChannel.Statistics.UserStatistics;
import com.jiubang.commerce.buychannel.buyChannel.bean.UserTypeInfo;
import com.jiubang.commerce.buychannel.buyChannel.manager.ServerCheckHelper;
import com.jiubang.commerce.buychannel.buyChannel.utils.BuyChannelUtils;

import static com.jiubang.commerce.buychannel.buyChannel.bean.UserTypeInfo.SecondUserType.WITHCOUNT_NOT_ORGNIC;


/**
 * Created by mayunlong on 16-9-28.
 */

public class BuyChannelSetting {

    private static BuyChannelSetting sInstance;
    private Context mContext;
    private Runnable mRefreshUserTagTask;
    private SharedPreferences mSp;

    public static BuyChannelSetting getInstance(Context context) {
        if (sInstance == null) {
            synchronized (com.jiubang.commerce.buychannel.BuyChannelSetting.class) {
                if (sInstance == null) {
                    sInstance = new BuyChannelSetting(context);
                }
            }
        }
        return sInstance;
    }

    private BuyChannelSetting(Context context) {
        mContext = context != null ? context.getApplicationContext() : null;
        mSp = BuyChannelDataMgr.getInstance(mContext).getSharedPreferences(mContext);
    }

    public void setBuyChannel(String buyChannel, ChannelFrom channel, UserTypeInfo.FirstUserType userType,
                              final UserTypeInfo.SecondUserType juserType, String campaign, String campaignId, final String associatedObj, final String agency,
                              final String conversionDataJsonStr, final String referrer, final String tag, SetBuyChannelListener listener) {

        BuyChannelDataMgr buyChannelDataMgr = BuyChannelDataMgr.getInstance(mContext);

        if (userType == null || juserType == null) {

            if (channel.equals(ChannelFrom.from_ga.toString())) {
                //TODO 操作码2,位置5,userType或juserType为空，没有setBuyChannel
                StatisticsDebug.Statistic103Params statistic103ParamsP5 = new StatisticsDebug.Statistic103Params();
                statistic103ParamsP5.code(BuySdkConstants.DEBUG_CODE2).referrer(referrer).position(5).buychannel(buyChannel);
                StatisticsDebug.upload(mContext, statistic103ParamsP5);
            }
            return;
        }

        //旧数据不为空,判断新数据
        switch (userType) {
            case apkbuy://新数据为apk买量
                //覆盖,apk买量是不上传45协议
                buyChannelDataMgr.setBuyChannelBean(buyChannel, channel, userType, juserType, conversionDataJsonStr, listener, campaign, campaignId);
                LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelSetting::setBuyChannel] setBuyChannel　success,userType:" + userType.toString());
//                LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelSetting::setBuyChannel] 新数据为apk买量，对旧数据进行覆盖");
                if (!TextUtils.isEmpty(associatedObj)) {
                    uploard45(associatedObj, agency, conversionDataJsonStr, referrer, juserType.getValue(), tag, false);

                } else {
                    //TODO 操作码2,位置6,apk买量，没有上传45协议
                    StatisticsDebug.Statistic103Params statistic103ParamsP6 = new StatisticsDebug.Statistic103Params();
                    statistic103ParamsP6.code(BuySdkConstants.DEBUG_CODE2).referrer(referrer).position(6).buychannel(buyChannel);
                    StatisticsDebug.upload(mContext, statistic103ParamsP6);
                }
                break;
            case userbuy://新数据为一般买量

                if (BuyChannelUtils.isOldApkBuy(mContext) || BuyChannelUtils.isOldFbAdTw(mContext)) {
                    //旧数据为apk买量，直接return,覆盖不成功
                    if (listener != null) {
                        listener.setBuyChannelFailure("7");
                    }

                    return;
                }
                //到这里说明旧数据已经不是apk买量
                if (isFbAdTw(juserType) || BuyChannelUtils.isOldWithCount(mContext) || BuyChannelUtils.isOldOrgnic(mContext) || BuyChannelUtils.isOldEmpty(mContext)) {
                    //如果新数据为fb,adword,twitter且旧数据不为apk的情况，直接覆盖
                    buyChannelDataMgr.setBuyChannelBean(buyChannel, channel, userType, juserType, conversionDataJsonStr, listener, campaign, campaignId);
                    //45协议上传前，必须先上传19协议
                    uploard45(associatedObj, agency, conversionDataJsonStr, referrer, juserType.getValue(), tag, true);
                    LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelSetting::setBuyChannel] setBuyChannel　success,userType:" + userType.toString());
//                    LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelSetting::setBuyChannel] 新数据为一般买量，" +
//                            "旧数据的用户类型不为一般买量和apk买量，或新数据为fb、adword、twitter，覆盖:" + userType.toString());
                } else {
                    //覆盖不成功
                    if (listener != null) {
                        listener.setBuyChannelFailure("7");
                    }
                }
                break;

            case withCount:
                //旧数据是自然进行覆盖
                if (BuyChannelUtils.isOldOrgnic(mContext) || BuyChannelUtils.isOldEmpty(mContext)) {
                    buyChannelDataMgr.setBuyChannelBean(buyChannel, channel, userType, juserType, conversionDataJsonStr, listener, campaign, campaignId);
                    uploard45(associatedObj, agency, conversionDataJsonStr, referrer, juserType.getValue(), tag, false);
                    LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelSetting::setBuyChannel] setBuyChannel　success,userType:" + userType.toString());
//                    LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelSetting::setBuyChannel] 新数据为带量，" +
//                            "旧数据的用户类型为自然，覆盖:" + userType.toString());

                    if (juserType.toString().equals(WITHCOUNT_NOT_ORGNIC.toString())) {
                        //非自然带量进行找回
                        checkOrgnic(userType, campaign, campaignId);
//                        LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelSetting::setBuyChannel] 新数据为非自然带量," +
//                                "进行check_server");
                    }

                } else {
                    //此处说明覆盖不成功
                    if (listener != null) {
                        listener.setBuyChannelFailure("8");
                    }

                }
                break;
            case organic:
                //新数据为自然，只有旧数据为空的情况，才进行覆盖
                if (BuyChannelUtils.isOldEmpty(mContext)) {

                    buyChannelDataMgr.setBuyChannelBean(buyChannel, channel, userType, juserType, conversionDataJsonStr, listener, campaign, campaignId);
                    uploard45(associatedObj, agency, conversionDataJsonStr, referrer, juserType.getValue(), tag, false);
                    checkOrgnic(userType, campaign, campaignId);
                    LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelSetting::setBuyChannel] setBuyChannel　success,userType:" + userType.toString());
//                    LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelSetting::setBuyChannel] 新数据为自然," +
//                            "需要进行check_server");
                } else {
                    //此处说明覆盖不成功
                    if (listener != null) {
                        listener.setBuyChannelFailure("9");
                    }


                }
                break;

            default:
                //此处说明覆盖不成功
                if (listener != null) {
                    listener.setBuyChannelFailure(null);
                }
                break;
        }

    }

    /**
     * 判断是否是fb,ad,twiter之一,且能覆盖
     * created by mayunlong.
     * created date 16-10-31.
     */
    private boolean isFbAdTw(UserTypeInfo.SecondUserType juserType) {

        /*BuyChannelBean oldBuyChannelBean = BuyChannelDataMgr.getInstance(mContext).getBuyChannelBean();
        if (oldBuyChannelBean.getFirstUserType().equals(apkbuy.toString())) {
            return false;
        }*/
        //已经保存了fb,adwords
        if (BuyChannelUtils.isOldFbAdTw(mContext)) {
            return false;
        }

        if (juserType.equals(UserTypeInfo.SecondUserType.FB_AUTO)) {
            return true;
        }
        if (juserType.equals(UserTypeInfo.SecondUserType.FB_NOTAUTO)) {
            return true;
        }
        if (juserType.equals(UserTypeInfo.SecondUserType.ADWORDS_AUTO)) {
            return true;
        }
        if (juserType.equals(UserTypeInfo.SecondUserType.ADWORDS_NOTAUTO)) {
            return true;
        }
        return false;
    }


    private void uploard45(String associatedObj, String agency, String conversionDataJsonStr, String referrer, int secondUserType, String tag, boolean isAfter19) {

        BuySdkInitParams mInitParams = AppsFlyerProxy.getInstance().getInitParams();

        boolean isGoKeyboard = mSp.getBoolean(BuySdkConstants.IS_GOKEYBORAD, false);
        int fun45id = mSp.getInt(BuySdkConstants.FUN_ID_45, 0);

        if (mInitParams != null) {
            fun45id = fun45id != 0 ? fun45id : mInitParams.mP45FunId;
        }

        UserStatistics.upload45(mContext, isGoKeyboard, fun45id,
                associatedObj, conversionDataJsonStr, referrer, String.valueOf(secondUserType), tag, agency, isAfter19);

    }


    private void checkOrgnic(UserTypeInfo.FirstUserType userType, String campaign, String campaignId) {
        //将第一次和最近一次记录的server-check时间进行清空
        BuyChannelDataMgr.getInstance(mContext).setFirstCheckTime(0);
        BuyChannelDataMgr.getInstance(mContext).setLastCheckTime(0);

//        LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelSetting::setBuyChannel]将第一次和最近一次记录的server-check时间进行清空," +
//                "FirstCheckTime=" + BuyChannelDataMgr.getInstance(mContext).getFirstCheckTime()
//                + ",LastCheckTime=" + BuyChannelDataMgr.getInstance(mContext).getLastCheckTime());

        //取消定时器
        CustomAlarmManager.getInstance(mContext).getAlarm("buychannelsdk").cancelAarm(BuySdkConstants.ALARM_ID);

//        LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelSetting::setBuyChannel]取消定时器");

        LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelSetting::setBuyChannel] 新数据为自然或非自然带量，去服务器进行检查:" + userType.toString());

        SharedPreferences mSp = BuyChannelDataMgr.getInstance(mContext).getSharedPreferences(mContext);
        mSp.edit().putString(BuySdkConstants.NEW_USER_BEFORE, userType.toString()).commit();
        mSp.edit().putString(BuySdkConstants.CAMPAIGN, campaign).commit();
        mSp.edit().putString(BuySdkConstants.CAMPAIGN_ID, campaignId).commit();
        //重新开启定时器，通过用户标签进行识别
        startUserTagTask();
    }


    /**
     * 枚举买量渠道的来源
     * created by mayunlong.
     * created date 16-9-30.
     */
    public static enum ChannelFrom {
        from_ga, from_appsflyer, from_usertag, from_client, un_known, from_oldUser, from_oldUser_usertag
    }


    private void startUserTagTask() {
        if (mRefreshUserTagTask != null) {
            CustomThreadExecutorProxy.getInstance().cancel(mRefreshUserTagTask);
            mRefreshUserTagTask = null;
        }

        mRefreshUserTagTask = new Runnable() {
            @Override
            public void run() {
                //开启server-check
                ServerCheckHelper.getInstance(mContext).startCheckServer(0);
                mRefreshUserTagTask = null;
            }
        };

        //延迟15s执行
        CustomThreadExecutorProxy.getInstance().runOnMainThread(mRefreshUserTagTask, BuySdkConstants.TIME_WAITFOR_REQUEST_USERTAG);
        if (LogUtils.isShowLog()) {
            LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelSetting::startUserTagTask] 延迟15s启动server-chcek");
        }
    }


}
