package com.jiubang.commerce.buychannel;

import com.jiubang.commerce.utils.StringUtils;

import java.util.List;

/**
 * 买量SDK初始化参数
 * Created by chenchongji on 2016/6/20.
 */
public class BuySdkInitParams {
    /**
     * 渠道号,用于判断是否APK买量
     */
    public final int mChannel;

    /**
     * 45协议里的功能点ID，每个产品不一样
     */
    public final int mP45FunId;

    /**
     * "用户来源判断接口协议"里的产品cid
     * 定义详见 http://wiki.3g.net.cn/pages/viewpage.action?pageId=14255114
     */
    public final String mUsertypeProtocalCId;

    /**
     * 19协议上传处理器,用于上传19协议
     */
    public final IProtocal19Handler mProtocal19Handler;

    /**
     * 是否未接买量SDK之前的老用户
     */
    public final boolean mIsOldUserWithoutSdk;

    /**
     * 未接买量SDK之前的老用户的买量渠道名称，新用户直接传null或不传
     */
    public final String mOldBuyChannel;

    /**
     * 应用主Activity的进程名称, 默认null为主进程。输入法、双开等主Ａctivity不在主进程的应用必填此参数
     */
    public final String mProcessName;

    /**
     * 是否GO输入法系列（该系列产品用的统计SDK是定制版，需特殊处理）
     */
    public final boolean mIsGoKeyboard;

    /**
     * adwords-gdn子渠道campaign id数组,由推广组提供,每个产品不一样
     */
    public final List<String> mAdwordsGdnCampaignids;
    /**
     * 长度24的字符串，在广告portal产品管理中生成
     */
    public final String mProductKey;
    /**
     * 长度32的字符串，在广告portal产品管理中生成
     */
    public final String mAccessKey;
    /**
     * 是否允许获取imei
     */
    public final boolean isSetImei;
    /**
     * apk买量是否上传45协议
     */
    public final boolean mIsApkUpLoad45;

    private BuySdkInitParams(Builder builder) {
        mChannel = builder.mChannel;
        mP45FunId = builder.mP45FunId;
        mUsertypeProtocalCId = builder.mUsertypeProtocalCId;
        mProtocal19Handler = builder.mProtocal19Handler;
        mIsGoKeyboard = builder.mIsGoKeyboard;
        mProductKey = builder.mProductKey;
        mAccessKey = builder.mAccessKey;

        mIsOldUserWithoutSdk = builder.mIsOldUserWithoutSdk;
        mOldBuyChannel = builder.mOldBuyChannel;
        mProcessName = builder.mProcessName;
        mAdwordsGdnCampaignids = builder.mAdwordsGdnCampaignids;
        isSetImei = builder.mIsSetImei;
        mIsApkUpLoad45 = builder.mIsApkUpLoad45;
    }

    public static class Builder {
        //必填参数
        public int mChannel;
        public int mP45FunId;
        public String mUsertypeProtocalCId;
        public IProtocal19Handler mProtocal19Handler;
        public boolean mIsGoKeyboard;
        public String mProductKey;
        public String mAccessKey;

        //选填参数
        public boolean mIsOldUserWithoutSdk;
        public String mOldBuyChannel;
        public String mProcessName;
        public List<String> mAdwordsGdnCampaignids;
        public boolean mIsSetImei = true;
        public boolean mIsApkUpLoad45 = false;

        /**
         * @param channel             渠道号,用于判断是否APK买量
         * @param p45FunId            45协议里的功能点ID，每个产品不一样,查询网址　http://wiki.3g.net.cn/pages/viewpage.action?pageId=18784950
         * @param usertypeProtocalCId "用户来源判断接口协议"里的产品cid
         *                            定义详见 http://wiki.3g.net.cn/pages/viewpage.action?pageId=14255114
         * @param p19Handler          19协议处理器接口,用于(在45协议上传前)上传19协议
         * @param isGoKeyboard        是否GO输入法系列（该系列产品用的统计SDK是定制版，需特殊处理）
         * @param productKey          长度24的字符串，在广告portal产品管理中生成,查询链接：http://gouser.3g.net.cn/userManage/login_iframe.jsp?systemid=68&hd=0(如果没有权限，请找黄炯查询)
         * @param accessKey           长度32的字符串，在广告portal产品管理中生成，查询链接：http://gouser.3g.net.cn/userManage/login_iframe.jsp?systemid=68&hd=0(如果没有权限，请找黄炯查询)
         */
        public Builder(String channel, int p45FunId, String usertypeProtocalCId, IProtocal19Handler p19Handler, boolean isGoKeyboard, String productKey, String accessKey) {
            int c = StringUtils.toInteger(channel, Integer.valueOf(0)).intValue();
            channel = c <= 0 ? "200" : String.valueOf(c);
            mChannel = Integer.parseInt(channel);
            mP45FunId = p45FunId;
            mUsertypeProtocalCId = usertypeProtocalCId;
            mProtocal19Handler = p19Handler;
            mIsGoKeyboard = isGoKeyboard;
            mProductKey = productKey;
            mAccessKey = accessKey;
        }

        /**
         * 是否允许获取imei
         */
        public Builder isSetImei(boolean v) {
            mIsSetImei = v;
            return this;
        }

        public Builder isApkUpLoad45(boolean v) {
            mIsApkUpLoad45 = v;
            return this;
        }


        /**
         * 是否未接买量SDK之前的老用户
         */
        public Builder isOldUserWithoutSdk(boolean v) {
            mIsOldUserWithoutSdk = v;
            return this;
        }

        /**
         * 未接买量SDK之前的老用户的买量渠道名称，新用户直接传null或不传
         */
        public Builder oldBuyChannel(String v) {
            mOldBuyChannel = v;
            return this;
        }

        /**
         * 应用主Activity的进程名称, 默认null为主进程。输入法、双开等主Ａctivity不在主进程的应用必填此参数
         */
        public Builder processName(String v) {
            mProcessName = v;
            return this;
        }

        /**
         * 是否GO输入法系列（该系列产品用的统计SDK是定制版，需特殊处理）
         */
       /* public Builder isGoKeyboard(boolean v) {
            mIsGoKeyboard = v;
            return this;
        }*/

        /**
         * adwords-gdn子渠道campaign id集合,由推广组提供,每个产品不一样
         */
        public Builder adwordsGdnCampaignids(List<String> v) {
            mAdwordsGdnCampaignids = v;
            return this;
        }

        public BuySdkInitParams build() {
            return new BuySdkInitParams(this);
        }
    }

    /**
     * 19协议处理器接口
     */
    public static interface IProtocal19Handler {
        void uploadProtocal19();
    }
}
