package com.jiubang.commerce.buychannel;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;

import com.jb.ga0.commerce.util.LogUtils;
import com.jiubang.commerce.buychannel.buyChannel.Statistics.StatisticsDebug;
import com.jiubang.commerce.buychannel.buyChannel.manager.ReferrerManager;
import com.jiubang.commerce.buychannel.buyChannel.utils.BuyChannelUtils;

import java.net.URLDecoder;


/**
 * Created by chenchongji on 2016/6/15.
 */
public class GaTrackerReceiver extends BroadcastReceiver {
    /**
     * 带量到Google Play安装应用后, Google Play客户端发出广播<br>
     */
    private static final String ACTION_INSTALL_REFERRER = "com.android.vending.INSTALL_REFERRER";
    private static final String EXTRA_REFERRER = "referrer";
    private static String lastReferrer;

    @Override
    public void onReceive(final Context context, final Intent intent) {
        if (null == intent) {
            return;
        }


        LogUtils.i(BuySdkConstants.LOG_TAG, "[GaTrackerReceiver::onReceive] intent = " + intent.toString());

//        LogUtils.i(BuySdkConstants.LOG_TAG, "[GaTrackerReceiver::onReceive] 广播所有内容 = " + intent.getExtras());
        final String referrer = intent.getStringExtra(EXTRA_REFERRER);
        LogUtils.i(BuySdkConstants.LOG_TAG, "[GaTrackerReceiver::onReceive] referrer = " + referrer);

        if (referrer != null && referrer.equals(lastReferrer)) {
//            LogUtils.i(BuySdkConstants.LOG_TAG, "[GaTrackerReceiver::onReceive] 两次 referrer相同不做处理");
            return;
        }

        lastReferrer = referrer;

        ReferrerManager refererManager = ReferrerManager.getInstance(context);

        if (refererManager.checkFirstReceiver()) {
            //TODO 操作码3,GA广播一来就上传referrer原始数据
            StatisticsDebug.Statistic103Params statistic103Params = new StatisticsDebug.Statistic103Params();
            statistic103Params.code(BuySdkConstants.DEBUG_CODE3).referrer(referrer);
            StatisticsDebug.upload(context, statistic103Params);
        }

        if (!refererManager.checkFirstReceiver()) {
            //TODO 操作码2，位置1,不是第一次GA广播，不上传45
            StatisticsDebug.Statistic103Params statistic103ParamsP1 = new StatisticsDebug.Statistic103Params();
            statistic103ParamsP1.code(BuySdkConstants.DEBUG_CODE2).referrer(referrer).position(1);
            StatisticsDebug.upload(context, statistic103ParamsP1);
            LogUtils.i(BuySdkConstants.LOG_TAG, "[GaTrackerReceiver::onReceive] 不是第一次GA广播");
            return;
        }

        if (BuyChannelUtils.isOldUser(context)) {
            //TODO 操作码2，位置2,已确定是老用户，不上传45
            StatisticsDebug.Statistic103Params statistic103ParamsP2 = new StatisticsDebug.Statistic103Params();
            statistic103ParamsP2.code(BuySdkConstants.DEBUG_CODE2).referrer(referrer).position(2);
            StatisticsDebug.upload(context, statistic103ParamsP2);
            LogUtils.i(BuySdkConstants.LOG_TAG, "[GaTrackerReceiver::onReceive] 已明确老用户");
            return;
        }

        if (BuyChannelUtils.isOldApkBuy(context)) {
            //TODO 操作码2，位置3,已确定是APK买量，不上传45
            StatisticsDebug.Statistic103Params statistic103ParamsP3 = new StatisticsDebug.Statistic103Params();
            statistic103ParamsP3.code(BuySdkConstants.DEBUG_CODE2).referrer(referrer).position(3);
            StatisticsDebug.upload(context, statistic103ParamsP3);
            LogUtils.i(BuySdkConstants.LOG_TAG, "[GaTrackerReceiver::onReceive] 已明确APK买量");
            return;
        }


        final String refer;
        try {
            //15s内GA有广播接收，取消15s上传45协议的任务
            AppsFlyerProxy.getInstance().cancelupload15S(referrer, null);
            AppsFlyerProxy.getInstance().cancelUploadFirst45(referrer);
            refer = URLDecoder.decode(referrer, "utf-8");
            //referrer中解析出utm_source
            String utmSource = refererManager.getUtmSource(refer);
            LogUtils.i(BuySdkConstants.LOG_TAG, "[GaTrackerReceiver::onReceive] 接收第一次GA广播，referrer = " + referrer + ",utmSource=" + utmSource);
            //GA规则解析utmSource
            refererManager.analyseUtmSource(utmSource, refer);
            //记录已经成功第一次接收广播
            refererManager.saveFirstReceiver();
            //保存第一次广播的referrer
            refererManager.saveReferrer(referrer);

        } catch (Exception e) {
            e.printStackTrace();
            LogUtils.e(BuySdkConstants.LOG_TAG, e.toString());
            //referrer中解析出utm_source
            String utmSource = refererManager.getUtmSource(referrer);
            LogUtils.i(BuySdkConstants.LOG_TAG, "[GaTrackerReceiver::onReceive] 接收第一次GA广播，referrer = " + referrer + ",utmSource=" + utmSource);
            //GA规则解析utmSource
            refererManager.analyseUtmSource(utmSource, referrer);
            //记录已经成功第一次接收广播
            refererManager.saveFirstReceiver();
            //保存第一次广播的referrer
            refererManager.saveReferrer(referrer);

        }


    }


}
