package com.jiubang.commerce.buychannel.buyChannel.Statistics;

import android.content.Context;
import android.content.SharedPreferences;

import com.gau.go.gostaticsdk.StatisticsManager;
import com.jb.ga0.commerce.util.LogUtils;
import com.jb.ga0.commerce.util.http.GoHttpHeadUtil;
import com.jb.ga0.commerce.util.statistics.BaseStatistic;
import com.jb.ga0.commerce.util.thread.CustomThreadExecutorProxy;
import com.jiubang.commerce.buychannel.AppsFlyerProxy;
import com.jiubang.commerce.buychannel.BuyChannelDataMgr;
import com.jiubang.commerce.buychannel.BuySdkConstants;
import com.jiubang.commerce.buychannel.BuySdkInitParams;
import com.jiubang.commerce.buychannel.buyChannel.database.BuychannelDbHelpler;
import com.jiubang.commerce.buychannel.buyChannel.database.StaticsTable;
import com.jiubang.commerce.buychannel.buyChannel.manager.InitManager;
import com.jiubang.commerce.buychannel.buyChannel.utils.BuyChannelUtils;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

/**
 * 45统计协议
 * Created by chenchongji on 2016/6/14.
 */
public class Statistics45 extends BaseStatistic {

    /**
     * 统计日志序列号
     */
    private static final int LOG_ID = 45;
    private Runnable mUpload45Task;

    /**
     * 上传使用45协议的统计数据。
     * "日志序列||Android ID||日志打印时间||功能点ID||统计对象
     * ||操作代码||操作结果||国家||渠道||版本号
     * ||版本名||入口||tab分类||位置||imei
     * ||goid||关联对象||备注||GADID"
     *
     * @param context
     * @param isGoKeyboard 是否输入法产品
     * @param params       统计数据
     * @param isAfter19
     * @备注 103统计日志比101多一个字段"广告ID"
     */

    public static void upload(final Context context, boolean isGoKeyboard, Statistic45Params params, boolean isAfter19) {
        StringBuffer buffer = new StringBuffer();
        //判断是否为输入法
        if (isGoKeyboard) {
            //当前时间(输入法需要特殊处理)
            buffer.append(System.currentTimeMillis());
            buffer.append(SEPARATOR);
        }

        // 日志序列
        buffer.append(LOG_ID);
        buffer.append(SEPARATOR);
        // Android ID
        appendStatisticField(buffer, GoHttpHeadUtil.getAndroidId(context));
        buffer.append(SEPARATOR);
        // 日志打印时间
        appendStatisticField(buffer, getCurrentBeiJingTime());
        buffer.append(SEPARATOR);
        // 功能点ID
        buffer.append(params.mFunId);
        buffer.append(SEPARATOR);
        // 统计对象(mapId)
        appendStatisticField(buffer, params.mSender);
        buffer.append(SEPARATOR);

        // 操作代码
        appendStatisticField(buffer, params.mOperationCode);
        buffer.append(SEPARATOR);
        // 操作结果-----0:未成功,1:成功(默认成功)
        appendStatisticField(buffer, params.mOperationResult);
        buffer.append(SEPARATOR);
        // 国家
        appendStatisticField(buffer, GoHttpHeadUtil.getCountry(context));
        buffer.append(SEPARATOR);
        // 渠道
        appendStatisticField(buffer, params.mChannel);
        buffer.append(SEPARATOR);
        // 版本号
        buffer.append(GoHttpHeadUtil.getVersionCode(context));
        buffer.append(SEPARATOR);

        // 版本名
        appendStatisticField(buffer, GoHttpHeadUtil.getVersionName(context));
        buffer.append(SEPARATOR);
        // 入口
        appendStatisticField(buffer, params.mEntrance);
        buffer.append(SEPARATOR);
        // AF明细
        appendStatisticField(buffer, params.mAfDetail);
        buffer.append(SEPARATOR);
        // Referrer
        appendStatisticField(buffer, params.mReferrer);
        buffer.append(SEPARATOR);
        // imei
        appendStatisticField(buffer, StatisticsManager.sIMEI);
        buffer.append(SEPARATOR);
        // goid
        appendStatisticField(buffer, StatisticsManager.getGOID(context));
        buffer.append(SEPARATOR);
        // 关联对象
        appendStatisticField(buffer, params.mAssociatedObj);
        buffer.append(SEPARATOR);
        //AF Agency
        appendStatisticField(buffer, params.mAfAgency);
        buffer.append(SEPARATOR);
        // 广告ID GADID
        appendStatisticField(buffer, params.mAdvertId);
        buffer.append(SEPARATOR);
        //gp版本名
        appendStatisticField(buffer, params.mGpVersionName);
        buffer.append(SEPARATOR);
        //用户类型
        appendStatisticField(buffer, params.mUserType);
        buffer.append(SEPARATOR);
        //原用户类型标志
        appendStatisticField(buffer, params.mTag);
        buffer.append(SEPARATOR);
        //推广SDK版本
        appendStatisticField(buffer, params.mSdkVersionCode);
        buffer.append(SEPARATOR);
        //是否可以获取SIM卡国家
        appendStatisticField(buffer, BuyChannelUtils.getCountrySuccess(context));

        if (isAfter19) {
            //需要在19协议上传成功后才上传45协议
            SharedPreferences sp = BuyChannelDataMgr.getInstance(context).getSharedPreferences(context);
            boolean hasUpload = sp.getBoolean(BuySdkConstants.KEY_HAS_UPLOAD_19, false);
            if (hasUpload) {
                //说明之前已经成功上传19协议
//                StatisticsManager.getInstance(context).upLoadStaticData(buffer.toString());
                StatisticsManager.getInstance(context).upLoadStaticData(45, params.mFunId, buffer.toString());
                LogUtils.i(BuySdkConstants.LOG_TAG, "[Statistics45::upload] " +
                        "之前已经有19协议上传成功，直接上传45协议");

                LogUtils.v(BuySdkConstants.LOG_TAG, "/功能点ID : " + params.mFunId
                        + "   /统计对象 : " + params.mSender
                        + "   /操作代码 : " + params.mOperationCode
                        + "   /操作结果 : " + params.mOperationResult
                        + "   /入口 : " + params.mEntrance
                        + "   /AF明细 : " + params.mAfDetail
                        + "   /Referrer : " + params.mReferrer
                        + "   /关联对象 : " + params.mAssociatedObj
                        + "   /广告ID : " + params.mAdvertId
                        + "   /AF Agency : " + params.mAfAgency
                        + "   gp版本名 : " + params.mGpVersionName
                        + "   用户类型 : " + params.mUserType
                        + "   原用户类型标志 : " + params.mTag
                        + "   推广SDK版本 : " + params.mSdkVersionCode
                        + "   是否可以获取SIM卡国家 : " + BuyChannelUtils.getCountrySuccess(context)
                );

            } else {
                LogUtils.i(BuySdkConstants.LOG_TAG, "[Statistics45::upload] 之前未上传19协议，新数据为买量，" +
                        "要先传19协议再上传45协议");
                //先上传19协议，同时把45协议的数据保存到本地
                BuySdkInitParams buySdkInitParams = AppsFlyerProxy.getInstance().getInitParams();
                if (buySdkInitParams.mProtocal19Handler != null) {
                    buySdkInitParams.mProtocal19Handler.uploadProtocal19();
                }
//                LogUtils.i(BuySdkConstants.LOG_TAG, "[Statistics45::upload] 先把45协议缓存到本地，" +
//                        "等待19协议上传成功再上传45协议");
                //保存功能id
                sp.edit().putInt(BuySdkConstants.FUN_ID_45, params.mFunId).commit();

                final StringBuffer bufferStr = buffer;
                CustomThreadExecutorProxy.getInstance().runOnAsyncThread(new Runnable() {
                    @Override
                    public void run() {
                        BuychannelDbHelpler dbHelpler = BuychannelDbHelpler.getInstance(context);
                        StaticsTable.insert(dbHelpler, bufferStr.toString());
                    }
                });

//                String old45Str = sp.getString(BuySdkConstants.KEY_STATISTICS_45_DATA, null);
//                if (!TextUtils.isEmpty(old45Str)) {
//                    StringBuffer bufferStr = new StringBuffer();
//                    bufferStr.append(old45Str);
//                    bufferStr.append("$");
//                    bufferStr.append(buffer.toString());
//                    sp.edit().putString(BuySdkConstants.KEY_STATISTICS_45_DATA, bufferStr.toString()).commit();
//                } else {
//                    sp.edit().putString(BuySdkConstants.KEY_STATISTICS_45_DATA, buffer.toString()).commit();
//                }

                upload45NotWait(context, params);


            }


        } else {
            LogUtils.i(BuySdkConstants.LOG_TAG, "[Statistics45::upload] 新数据不是买量，" +
                    "直接上传45协议");
            // 直接上传统计数据
//            StatisticsManager.getInstance(context).upLoadStaticData(buffer.toString());
            StatisticsManager.getInstance(context).upLoadStaticData(45, params.mFunId, buffer.toString());
            LogUtils.v(BuySdkConstants.LOG_TAG, "/功能点ID : " + params.mFunId
                    + "   /统计对象 : " + params.mSender
                    + "   /操作代码 : " + params.mOperationCode
                    + "   /操作结果 : " + params.mOperationResult
                    + "   /入口 : " + params.mEntrance
                    + "   /AF明细 : " + params.mAfDetail
                    + "   /Referrer : " + params.mReferrer
                    + "   /关联对象 : " + params.mAssociatedObj
                    + "   /广告ID : " + params.mAdvertId
                    + "   /AF Agency : " + params.mAfAgency
                    + "   gp版本名 : " + params.mGpVersionName
                    + "   用户类型 : " + params.mUserType
                    + "   原用户类型标志 : " + params.mTag
                    + "   推广SDK版本 : " + params.mSdkVersionCode
                    + "   是否可以获取SIM卡国家 : " + BuyChannelUtils.getCountrySuccess(context)
            );

        }


    }

    private static void upload45NotWait(final Context context, final Statistic45Params params) {

        Runnable mUpload45Task = new Runnable() {

            @Override
            public void run() {
                SharedPreferences sp = BuyChannelDataMgr.getInstance(context).getSharedPreferences(context);
                boolean hasUpload = sp.getBoolean(BuySdkConstants.KEY_HAS_UPLOAD_19, false);
                if (!hasUpload) {
                    InitManager.getInstance(context).check45(1);
                    //TODO 操作码1,位置1,15S以后19协议仍然没有回调，上传45协议
                    StatisticsDebug.Statistic103Params statistic103ParamsP1 = new StatisticsDebug.Statistic103Params();
                    statistic103ParamsP1.code(BuySdkConstants.DEBUG_CODE1).referrer(params.mReferrer).position(1).buychannel(params.mSender);
                    StatisticsDebug.upload(context, statistic103ParamsP1);

                }

            }
        };

        CustomThreadExecutorProxy.getInstance().runOnMainThread(mUpload45Task, BuySdkConstants.TIME_WAITFOR_APPSFLYER_RESULE);
    }


    public static String getCurrentBeiJingTime() {
        String stamp = null;

        try {
            Date now = new Date(System.currentTimeMillis());
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS", Locale.US);
            dateFormat.setTimeZone(TimeZone.getTimeZone("GMT+8"));
            stamp = dateFormat.format(now);
        } catch (Exception var5) {
        }

        return stamp;
    }

    /**
     * 45统计参数
     *
     * @author chenchongji
     *         2016年3月14日
     */
    public static class Statistic45Params {
        /**
         * 功能点ID
         */
        protected int mFunId;

        public String getSender() {
            return mSender;
        }

        /**
         * 统计对象
         */
        protected String mSender;

        /**
         * 操作代码
         */
        protected String mOperationCode;
        /**
         * 操作结果
         */
        protected String mOperationResult = "1";
        /**
         * 渠道
         */
        protected String mChannel;

        /**
         * 入口
         */
        protected String mEntrance;
        /**
         * AF明细
         */
        protected String mAfDetail;
        /**
         * Referrer
         */
        protected String mReferrer;

        /**
         * 关联对象
         */
        protected String mAssociatedObj;
        /**
         * 备注
         */
        protected String mRemark;
        /**
         * 广告ID
         */
        protected String mAdvertId;
        /**
         * gp版本名
         */
        protected String mGpVersionName;

        /**
         * 用户类型
         */
        protected String mUserType;

        /**
         * 原用户类型标志
         */
        protected String mTag;
        /**
         * 推广SDK版本
         */
        protected String mSdkVersionCode;
        /**
         * AF Agency
         */
        protected String mAfAgency;


        public String getUserType() {
            return mUserType;
        }

        /**
         * 功能点ID
         *
         * @param funId {@link Statistic45Params#mFunId}
         * @return
         */
        public Statistic45Params funId(int funId) {
            mFunId = funId;
            return this;
        }

        /**
         * 统计对象
         *
         * @param sender {@link Statistic45Params#mSender}
         * @return
         */
        public Statistic45Params sender(String sender) {
            mSender = sender;
            return this;
        }

        /**
         * 操作代码
         *
         * @param operationCode {@link Statistic45Params#mOperationCode}
         * @return
         */
        public Statistic45Params operationCode(String operationCode) {
            mOperationCode = operationCode;
            return this;
        }

        /**
         * 操作结果
         *
         * @param operationResult {@link Statistic45Params#mOperationResult}
         * @return
         */
        public Statistic45Params operationResult(String operationResult) {
            mOperationResult = operationResult;
            return this;
        }

        /**
         * 渠道
         *
         * @param channel {@link Statistic45Params#mChannel}
         * @return
         */
        public Statistic45Params channel(String channel) {
            mChannel = channel;
            return this;
        }

        /**
         * 入口
         *
         * @param entrance {@link Statistic45Params#mEntrance}
         * @return
         */
        public Statistic45Params entrance(String entrance) {
            mEntrance = entrance;
            return this;
        }

        /**
         * AF明细
         *
         * @param afDetail {@link Statistic45Params#mAfDetail}
         * @return
         */
        public Statistic45Params afDetail(String afDetail) {
            mAfDetail = afDetail;
            return this;
        }

        /**
         * 位置
         *
         * @param referrer {@link Statistic45Params#mReferrer}
         * @return
         */
        public Statistic45Params referrer(String referrer) {
            mReferrer = referrer;
            return this;
        }

        /**
         * 关联对象
         *
         * @param associatedObj {@link Statistic45Params#mAssociatedObj}
         * @return
         */
        public Statistic45Params associatedObj(String associatedObj) {
            mAssociatedObj = associatedObj;
            return this;
        }

        /**
         * 备注
         *
         * @param remark {@link Statistic45Params#mRemark}
         * @return
         */
        public Statistic45Params remark(String remark) {
            mRemark = remark;
            return this;
        }

        /**
         * 广告ID
         *
         * @param advertId {@link Statistic45Params#mAdvertId}
         * @return
         */
        public Statistic45Params advertId(String advertId) {
            mAdvertId = advertId;
            return this;
        }

        /**
         * 用户类型
         *
         * @param userType {@link Statistic45Params#mUserType}
         * @return
         */
        public Statistic45Params userType(String userType) {
            mUserType = userType;
            return this;
        }

        /**
         * gp版本名
         *
         * @param gpVersionName {@link Statistic45Params#mGpVersionName}
         * @return
         */
        public Statistic45Params gpVersionName(String gpVersionName) {
            mGpVersionName = gpVersionName;
            return this;
        }

        public Statistic45Params tag(String tag) {
            mTag = tag;
            return this;
        }

        public Statistic45Params sdkVersionCode(String sdkVersionCode) {
            mSdkVersionCode = sdkVersionCode;
            return this;
        }

        public Statistic45Params afAgency(String afAgency) {
            mAfAgency = afAgency;
            return this;
        }




    }
}
