package com.jiubang.commerce.buychannel.buyChannel.manager;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.text.TextUtils;

/**
 * Created by mayunlong on 16-10-12.
 */

public class GpVersionHelper {

    private static GpVersionHelper sInstance;
    private Context mContext;
    private PackageManager mPackageManager;

    public static GpVersionHelper getInstance(Context context) {
        if (sInstance == null) {
            synchronized (GpVersionHelper.class) {
                if (sInstance == null) {
                    sInstance = new GpVersionHelper(context);
                }
            }
        }
        return sInstance;
    }

    private GpVersionHelper(Context context) {
        mContext = context != null ? context.getApplicationContext() : null;
        mPackageManager = mContext.getPackageManager();
    }

    public String getGpVersionName() {
        String versionName = "";
        try {
            PackageInfo info = mPackageManager.getPackageInfo("com.android.vending", PackageManager.GET_UNINSTALLED_PACKAGES);
            versionName = info.versionName;
        } catch (Exception e) {
            e.printStackTrace();
        }
//        LogUtils.i(BuySdkConstants.LOG_TAG, "[GpVersionHelper::getGpVersionName]:gp版本名=" + versionName);
        return versionName;
    }

    /***
     * GP版本与6.8.24比较
     *
     * @return GP版本≥6.8.24,则返回true，GP版本<6.8.24，GP版本为空，则返回false
     */
    public boolean campareWhitGpVersion6_8_24() {
        String versionName = getGpVersionName();
        // 获取不到，认为GP版本未空，直接返回true
        if (TextUtils.isEmpty(versionName)) {
            return false;
        }
        //不包含“.”，也没版本判断，但GP是有的，可以认为是低版本或者未知版本，返回false
        if (!versionName.contains(".")) {
            return false;
        }
        //把.换成#，在截取，直接以.截取有问题
        String[] versionSp = versionName.split("\\.");
        int[] v6_8_24 = {6, 8, 24};
        try {
            //只比较前面3位，要大于6.8.24
            //第一位
            if (versionSp.length < 1) {
                return false;
            }
            int a = Integer.parseInt(versionSp[0]);
            if (a > v6_8_24[0]) {
                return true;
            } else if (a < v6_8_24[0]) {
                return false;
            }
            //说明第一位已经相等，第二位
            if (versionSp.length < 2) {
                return false;
            }
            int b = Integer.parseInt(versionSp[1]);
            if (b > v6_8_24[1]) {
                return true;
            } else if (b < v6_8_24[1]) {
                return false;
            }
            //说明第一位\第而二位已经相等，第三位
            if (versionSp.length < 3) {
                return false;
            }
            int c = Integer.parseInt(versionSp[2]);
            if (c >= v6_8_24[2]) {
                //前面三位都相等了，判断为相等
                return true;
            } else {
                return false;
            }

        } catch (Exception e) {
            //一旦解析错误，非数字类型，认为不一定≥6.8.24，false
            e.printStackTrace();
            return false;
        }
    }

}
