package com.jiubang.commerce.buychannel.buyChannel.manager;

import android.content.ContentValues;
import android.content.Context;
import android.content.SharedPreferences;

import com.gau.go.gostaticsdk.StatisticStateListener;
import com.gau.go.gostaticsdk.StatisticsManager;
import com.jb.ga0.commerce.util.CustomAlarm;
import com.jb.ga0.commerce.util.CustomAlarmManager;
import com.jb.ga0.commerce.util.LogUtils;
import com.jb.ga0.commerce.util.thread.CustomThreadExecutorProxy;
import com.jiubang.commerce.buychannel.BuyChannelDataMgr;
import com.jiubang.commerce.buychannel.BuyChannelSetting;
import com.jiubang.commerce.buychannel.BuySdkConstants;
import com.jiubang.commerce.buychannel.buyChannel.Statistics.StatisticsDebug;
import com.jiubang.commerce.buychannel.buyChannel.bean.UserTypeInfo;
import com.jiubang.commerce.buychannel.buyChannel.database.BuychannelDbHelpler;
import com.jiubang.commerce.buychannel.buyChannel.database.StaticsTable;
import com.jiubang.commerce.buychannel.buyChannel.utils.BuyChannelUtils;
import com.jiubang.commerce.buychannel.buyChannel.utils.TextUtils;

import java.util.List;

/**
 * Created by mayunlong on 16-11-17.
 */

public class InitManager {
    private static InitManager sInstance;

    private Context mContext;
    private SharedPreferences mSp;

    public static InitManager getInstance(Context context) {
        if (sInstance == null) {
            synchronized (InitManager.class) {
                if (sInstance == null) {
                    sInstance = new InitManager(context);
                }
            }
        }
        return sInstance;
    }

    private InitManager(Context context) {
        mContext = context != null ? context.getApplicationContext() : null;
        mSp = BuyChannelDataMgr.getInstance(mContext).getSharedPreferences(mContext);

    }

    public void setStatisticStateListener() {
        //设置统计SDK的监听
        try {
            StatisticsManager.getInstance(mContext).setStatisticStateListener(sStatisticsListener);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    private StatisticStateListener sStatisticsListener = new StatisticStateListener() {


        @Override
        public void onCtrlInfoInsertToDB(int i, ContentValues contentValues) {
            //Log.d(BuySdkConstants.LOG_TAG, "upload listener onCtrlInfoInsertToDB " + " i = " + i);
        }

        @Override
        public void onStatisticDataInsertToDB(String s, int i, int i1, String s1) {
            /*if (!TextUtils.isEmpty(s1)) {
                String data[] = s1.split("||");
                if (data != null && data.length > 0) {
                    String protocolId = data[0];
                    if (protocolId.equals("45")) {
                        SharedPreferences mSp = BuyChannelDataMgr.getInstance(sContext).getSharedPreferences(sContext);
                        mSp.edit().putBoolean(BuySdkConstants.KEY_HAS_UPLOAD_45, true).commit();
                        //将45协议数据保存起来
                        Log.d(BuySdkConstants.LOG_TAG, "protocol 45 data had insert to db");
                    }
                }
            }*/
            //Log.d(BuySdkConstants.LOG_TAG, "upload listener onStatisticDataInsertToDB s = " + s + " i = " + i + " i1 = " + i1 + " s1 = " + s1);
        }

        @Override
        public void onUploadStatisticDataStart(String s, int i, int i1, String s1) {
//            LogUtils.d(BuySdkConstants.LOG_TAG, "upload listener onUploadStatisticDataStart s = " + s + " i = " + i + " i1 = " + i1 + " s1 = " + s1);
        }


        @Override
        public void onUploadStatisticDataSuccess(String s, int i, int i1, String s1) {

            LogUtils.d(BuySdkConstants.LOG_TAG, "[BuyChannelApi::onUploadStatisticDataSuccess]upload listener onUploadStatisticDataSuccess s = " + s + " i = " + i + " i1 = " + i1 + " s1 = " + s1);
            if (i1 == BuySdkConstants.PROTOCOL_ID_19) {
                SharedPreferences sp = BuyChannelDataMgr.getInstance(mContext).getSharedPreferences(mContext);
                boolean hasUpload = sp.getBoolean(BuySdkConstants.KEY_HAS_UPLOAD_19, false);
                if (!hasUpload) {
                    LogUtils.d(BuySdkConstants.LOG_TAG, "[BuyChannelApi::onUploadStatisticDataSuccess]first time upload 19 success");
                    try {
                        sp.edit().putBoolean(BuySdkConstants.KEY_HAS_UPLOAD_19, true).commit();
                        /*Set<String> set = new HashSet<String>();
                        set = mSp.getStringSet(BuySdkConstants.KEY_STATISTICS_45_DATA, new HashSet<String>());
                        if (set != null && set.size() > 0) {
                            Iterator<String> it = set.iterator();
                            while (it.hasNext()) {
                                String statistics45Data = it.next();
                                StatisticsManager.getInstance(sContext).upLoadStaticData(statistics45Data);
                                Log.d(BuySdkConstants.LOG_TAG, "upload 45 after 19 upload success,45协议内容：" + statistics45Data);
                            }

                        }*/
                        uploadSp45(sp, 0);

                    } catch (Exception e) {
                        LogUtils.d(BuySdkConstants.LOG_TAG, " error：" + e);
                    }

                }
            }
        }

        @Override
        public void onUploadStatisticDataFailed(String s, int i, int i1, String s1) {
            LogUtils.d(BuySdkConstants.LOG_TAG, "[BuyChannelApi::onUploadStatisticDataFailed]upload listener 统计协议上传失败 s = " + s + " i = " + i + " i1 = " + i1 + " s1 = " + s1);
        }
    };

    /**
     * 将缓存中的45协议数据进行上传
     * created by mayunlong.
     * created date 16-11-16.
     */
    private void uploadSp45(SharedPreferences sp, final int position) {
        final int funId = sp.getInt(BuySdkConstants.FUN_ID_45, 0);

        CustomThreadExecutorProxy.getInstance().runOnAsyncThread(new Runnable() {
            @Override
            public void run() {
                BuychannelDbHelpler dbHelpler = BuychannelDbHelpler.getInstance(mContext);
                List statics45List = StaticsTable.queryAll(dbHelpler);

                if (statics45List.size() != 0) {
                    if (position == 2) {
                        //TODO 操作码1,位置2,应用启动后，发现上次未传的45统计（通常是定时到前程序被终止），直接上传
                        StatisticsDebug.Statistic103Params statistic103ParamsP2 = new StatisticsDebug.Statistic103Params();
                        statistic103ParamsP2.code(BuySdkConstants.DEBUG_CODE1).position(2);
                        StatisticsDebug.upload(mContext, statistic103ParamsP2);
                    }
                }

                for (int i = 0; i < statics45List.size(); i++) {

                    StatisticsManager.getInstance(mContext).upLoadStaticData(45, funId, statics45List.get(i).toString());
                    LogUtils.d(BuySdkConstants.LOG_TAG, "[BuyChannelApi::uploadSp45]upload 45 after 19 upload success");
                }

                StaticsTable.deleteAll(dbHelpler);
            }
        });

//        String old45Str = sp.getString(BuySdkConstants.KEY_STATISTICS_45_DATA, null);
//        if (!TextUtils.isEmpty(old45Str)) {
//
//            if (position == 2) {
//                //TODO 操作码1,位置2,应用启动后，发现上次未传的45统计（通常是定时到前程序被终止），直接上传
//                StatisticsDebug.Statistic103Params statistic103ParamsP2 = new StatisticsDebug.Statistic103Params();
//                statistic103ParamsP2.code(BuySdkConstants.DEBUG_CODE1).position(2);
//                StatisticsDebug.upload(mContext, statistic103ParamsP2);
//            }
//
//            String[] old45Strs = old45Str.split(BuySdkConstants.SEPERATOR_45_SP);
//            for (int y = 0; y < old45Strs.length; y++) {
//
////                StatisticsManager.getInstance(mContext).upLoadStaticData(old45Strs[y]);
//                StatisticsManager.getInstance(mContext).upLoadStaticData(45, funId, old45Strs[y]);
//                LogUtils.d(BuySdkConstants.LOG_TAG, "[BuyChannelApi::uploadSp45]upload 45 after 19 upload success");
//                /*try {
//                    Thread.sleep(1000);
//                } catch (InterruptedException e) {
//                    e.printStackTrace();
//                }*/
//
//            }
//            //清空数据
//            sp.edit().putString(BuySdkConstants.KEY_STATISTICS_45_DATA, null).commit();
//        }

    }

    /**
     * 检查本地缓存45协议数据的方法
     * created by mayunlong.
     * created date 16-11-16.
     */
    public void check45(int position) {


//        boolean hasUpload = mSp.getBoolean(BuySdkConstants.KEY_HAS_UPLOAD_19, false);
//        if (hasUpload) {
//            LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelApi::check45] " + "已经有19协议成功上传，检查本地缓存是否还有45协议数据");
//            uploadSp45(mSp);
//
//        }
        mSp = BuyChannelDataMgr.getInstance(mContext).getSharedPreferences(mContext);
        uploadSp45(mSp, position);
    }


    /**
     * 是否买量SDK自己升级
     * created by mayunlong.
     * created date 16-11-17.
     */
    public boolean isUpdateBuyChannelSdk() {

        int oldVersionCode = mSp.getInt(BuySdkConstants.BUY_SDK_VERSIONCODE, 0);
        if (oldVersionCode > 0) {

            LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelApi::isUpdateBuyChannelSdk] " + "之前已经接过买量SDK");
            return true;
        }
        return false;
    }

    /**
     * 保存买量SDK的版本数据
     * created by mayunlong.
     * created date 16-11-17.
     */
    public void saveSdkVersion() {
        CustomAlarmManager.getInstance(mContext).getAlarm("saveVersionCode").alarmRepeat(BuySdkConstants.ALARM_ID_VERSIONCODE, BuySdkConstants.INTERVAL_SAVE_VERSION,
                BuySdkConstants.INTERVAL_SERVER_CHECKTIME, true,
                new CustomAlarm.OnAlarmListener() {

                    @Override
                    public void onAlarm(int i) {
                        boolean isSaveVersionCode = mSp.getBoolean(BuySdkConstants.SAVE_VERSIONCODE, false);
                        if (isSaveVersionCode) {
                            CustomAlarmManager.getInstance(mContext).getAlarm("saveVersionCode").cancelAarm(BuySdkConstants.ALARM_ID_VERSIONCODE);
                            return;
                        }


                        int oldVersionCode = mSp.getInt(BuySdkConstants.BUY_SDK_VERSIONCODE, 0);
                        if (BuySdkConstants.SDK_VERSION_CODE > oldVersionCode) {
                            mSp.edit().putInt(BuySdkConstants.BUY_SDK_VERSIONCODE, BuySdkConstants.SDK_VERSION_CODE).commit();
                            mSp.edit().putBoolean(BuySdkConstants.SAVE_VERSIONCODE, true).commit();
                        }
                    }
                });


    }

    /**
     * 老用户升级时必须调用该方法，以便进行用户类型的兼容
     * created by mayunlong.
     * created date 16-10-24.
     */
    public void updateOldUser(final Context context, String buyChannel, boolean isOldUser, String cId) {


        mSp.edit().putString(BuySdkConstants.PRODUCT_ID, cId).commit();
        String oldUserMsg = mSp.getString(BuySdkConstants.OLD_USER_MSG, null);
        if (!TextUtils.isEmpty(oldUserMsg)) {
            //已经存储买量SDK版本信息，说明之前已经是老用户升级，不再进行老用户身份查询
            LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelApi::updateOldUser] 缓存中已有旧用户的信息," +
                    "说明之前已经老用户升级过，不再查询老用户标签");
            return;
        }


        if (TextUtils.isEmpty(buyChannel) && isOldUser) {
            //传isOldUser为true,但buyChannel为空的情况
            buyChannel = "null";

            /*//这种情况，我们默认将老用户设置为自然
            savaOrganicOldUser(context, buyChannel);*/
        }

        //存储老用户信息到sp
        saveOldUserMsg(buyChannel, isOldUser);
        if (BuyChannelUtils.isOldApkBuy(context)) {
            LogUtils.i(BuySdkConstants.LOG_TAG, "[InitManager::updateOldUser] 已缓存APK买量，不去查找老用户身份");
            return;
        }
        //注册网络变化广播接收者，一旦有网络，就去核实老用户身份
        OldUserUpHelper.getInstance(context).updateOldUser(BuySdkConstants.CHECK_USERTAG_OLDUSER);

    }

    /**
     * 存储老用户身份为自然老用户
     * created by mayunlong.
     * created date 16-11-18.
     */
    private void savaOrganicOldUser(Context context, String buyChannel) {
        LogUtils.i(BuySdkConstants.LOG_TAG, "[InitManager::savaOrganicOldUser] 老用户buyChannel：" + buyChannel + "默认设置自然老用户");
        UserTypeInfo.FirstUserType firstUserType = UserTypeInfo.FirstUserType.organic;
        UserTypeInfo.SecondUserType secondUserType = UserTypeInfo.SecondUserType.GP_ORGNIC;
        BuyChannelDataMgr.getInstance(context).setBuyChannelBean(buyChannel, BuyChannelSetting.ChannelFrom.from_oldUser, firstUserType, secondUserType, null, null, null, null);
    }


    /**
     * 将老用户信息存储到sp的方法
     * created by mayunlong.
     * created date 16-11-18.
     */
    private void saveOldUserMsg(String buyChannel, boolean isOldUser) {
        StringBuffer buffer = new StringBuffer();
        buffer.append(buyChannel);
        buffer.append(BuySdkConstants.SEPARATOR);
        buffer.append(isOldUser);
        LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelApi::updateOldUser] 是否老用户" +
                isOldUser + ",老用户buyChannel:" + buyChannel);
        //只要调用了一次该接口，就会存储老用户信息，避免多次查询老用户
        mSp.edit().putString(BuySdkConstants.OLD_USER_MSG, buffer.toString()).commit();
    }


}
