package com.jiubang.commerce.buychannel.buyChannel.manager;

import android.content.Context;
import android.content.SharedPreferences;

import com.jb.ga0.commerce.util.LogUtils;
import com.jb.ga0.commerce.util.thread.CustomThreadExecutorProxy;
import com.jiubang.commerce.buychannel.BuyChannelDataMgr;
import com.jiubang.commerce.buychannel.BuyChannelSetting;
import com.jiubang.commerce.buychannel.BuySdkConstants;
import com.jiubang.commerce.buychannel.buyChannel.Interface.SetBuyChannelListener;
import com.jiubang.commerce.buychannel.buyChannel.Statistics.StatisticsDebug;
import com.jiubang.commerce.buychannel.buyChannel.Statistics.UserStatistics;
import com.jiubang.commerce.buychannel.buyChannel.bean.UserTypeInfo;
import com.jiubang.commerce.buychannel.buyChannel.utils.BuyChannelUtils;
import com.jiubang.commerce.buychannel.buyChannel.utils.TextUtils;

import static com.jiubang.commerce.buychannel.BuySdkConstants.FUN_ID_45;
import static com.jiubang.commerce.buychannel.BuySdkConstants.IS_GOKEYBORAD;

/**
 * Created by Administrator on 2016/9/24.
 */

public class ReferrerManager {


    private static ReferrerManager sInstance;
    private Context mContext;
    private SharedPreferences mSp;

    public static ReferrerManager getInstance(Context context) {
        if (sInstance == null) {
            synchronized (ReferrerManager.class) {
                if (sInstance == null) {
                    sInstance = new ReferrerManager(context);
                }
            }
        }
        return sInstance;
    }

    private ReferrerManager(Context context) {
        mContext = context != null ? context.getApplicationContext() : null;
        mSp = BuyChannelDataMgr.getInstance(mContext).getSharedPreferences(mContext);

    }

    public boolean checkFirstReceiver() {
        return mSp.getBoolean(BuySdkConstants.IS_FIRST_RECEIVER, true);
    }

    public void saveFirstReceiver() {
        mSp.edit().putBoolean(BuySdkConstants.IS_FIRST_RECEIVER, false).commit();

    }

    public void saveReferrer(String referrer) {
        if (!TextUtils.isEmpty(referrer)) {
            referrer = referrer.replaceAll("\\n", "");
            referrer = referrer.replaceAll("\\u007C", "#");
        }
        mSp.edit().putString(BuySdkConstants.REFERRER, referrer).commit();

    }

    public String getReferrer() {
        String oldReferrer = mSp.getString(BuySdkConstants.REFERRER, null);
        return oldReferrer;
    }


    public String getCampaign(String referrer) {
        if (!TextUtils.isEmpty(referrer)) {
            String[] splits = referrer.split("&");
            if (splits != null && splits.length >= 0) {
                for (String string : splits) {
                    if (string != null && string.toLowerCase().contains("campaign")) {
                        String[] utmSources = string.split("=");
                        if (utmSources != null && utmSources.length > 1) {
                            return utmSources[1];
                        }
                    }
                }
            }
        }
        return null;
    }

    public String getCampaignId(String referrer) {
        if (!TextUtils.isEmpty(referrer)) {
            String[] splits = referrer.split("&");
            if (splits != null && splits.length >= 0) {
                for (String string : splits) {
                    if (string != null && string.toLowerCase().contains("campaignid")) {
                        String[] utmSources = string.split("=");
                        if (utmSources != null && utmSources.length > 1) {
                            return utmSources[1];
                        }
                    }
                }
            }
        }
        return null;
    }


    public String getUtmSource(String referrer) {
        if (!TextUtils.isEmpty(referrer)) {
            String[] splits = referrer.split("&");
            if (splits != null && splits.length >= 0) {
                for (String string : splits) {
                    if (string != null && string.contains("utm_source")) {
                        String[] utmSources = string.split("=");
                        if (utmSources != null && utmSources.length > 1) {
                            return utmSources[1];
                        }
                    }
                }
            }
        }
        return null;
    }

    public String getFromUserType(String referrer) {
        if (!TextUtils.isEmpty(referrer)) {
            String[] splits = referrer.split("&");
            if (splits != null && splits.length >= 0) {
                for (String string : splits) {
                    if (string != null && string.contains("from_3g_channel")) {
                        String[] fromUserType = string.split("=");
                        if (fromUserType != null && fromUserType.length > 1) {
                            return fromUserType[1];
                        }
                    }
                }
            }
        }
        return null;
    }

    public void analyseUtmSource(String utmSource, String referrer) {
        UserTypeInfo.FirstUserType userType = null;
        UserTypeInfo.SecondUserType sUserType = null;

        boolean isGpOrgnic = false;
        boolean isNotGpOrgnic = false;

        GpVersionHelper gpVersionHelper = GpVersionHelper.getInstance(mContext);
//        LogUtils.i(BuySdkConstants.LOG_TAG, "[ReferrerManager::analyseUtmSource]解析GA中的UtmSource为[" + utmSource + "]");

        if (referrer == null) {
            //操作码2,位置4,referrer为空，没有进行GA的setBuyChannel
            StatisticsDebug.Statistic103Params statistic103ParamsP4 = new StatisticsDebug.Statistic103Params();
            statistic103ParamsP4.code(BuySdkConstants.DEBUG_CODE2).referrer(referrer).position(4);
            StatisticsDebug.upload(mContext, statistic103ParamsP4);
            LogUtils.i(BuySdkConstants.LOG_TAG, "[ReferrerManager::analyseUtmSource] refferer为空");
            return;
        }

        if (TextUtils.isEmpty(utmSource) && !gpVersionHelper.campareWhitGpVersion6_8_24()) {
            //gp自然
            isGpOrgnic = true;
            LogUtils.i(BuySdkConstants.LOG_TAG, "[ReferrerManager::analyseUtmSource]解析出UtmSource为空，且gp版本小于6.8.24或没有gp版本,判定为gp自然，isGpOrgnic=" + isGpOrgnic);

        }

        if (TextUtils.isEmpty(utmSource) && gpVersionHelper.campareWhitGpVersion6_8_24()) {
            //非gp自然
            isNotGpOrgnic = true;
            LogUtils.i(BuySdkConstants.LOG_TAG, "[ReferrerManager::analyseUtmSource]解析出UtmSource为空，且gp版本大于6.8.24,判定为非gp自然，isNotGpOrgnic=" + isNotGpOrgnic);

        }


        if (!TextUtils.isEmpty(referrer)) {
            referrer = referrer.replaceAll("\\n", "");
            referrer = referrer.replaceAll("\\u007C", "#");
        }


        if (!TextUtils.isEmpty(utmSource) && utmSource.contains("not set")) {
            //定义为非自然带量
            userType = UserTypeInfo.FirstUserType.withCount;
            sUserType = UserTypeInfo.SecondUserType.WITHCOUNT_NOT_ORGNIC;
            LogUtils.i(BuySdkConstants.LOG_TAG, "[ReferrerManager::analyseUtmSource]utmSource为not_set,认定为非自然带量，userType=" + userType + "，二级类型为：" + sUserType);

        }

        if (!TextUtils.isEmpty(utmSource) && utmSource.toLowerCase().contains("google-play")) {
            //GP自然用户
            userType = UserTypeInfo.FirstUserType.organic;
            sUserType = UserTypeInfo.SecondUserType.GP_ORGNIC;
            LogUtils.i(BuySdkConstants.LOG_TAG, "[ReferrerManager::analyseUtmSource]解析GA中的UtmSource，" +
                    "包含google-play关键字，判定为gp自然，isGpOrgnic=" + isGpOrgnic);

        } else if (!TextUtils.isEmpty(utmSource) && referrer.contains("gokey_channel") && referrer.contains("gokey_click_id") ||
                !TextUtils.isEmpty(utmSource) && referrer.contains("zerokey_channel") && referrer.contains("zerokey_click_id")) {

            userType = UserTypeInfo.FirstUserType.userbuy;
            sUserType = UserTypeInfo.SecondUserType.GA_USERBUY;
            LogUtils.i(BuySdkConstants.LOG_TAG, "[ReferrerManager::analyseUtmSource]解析GA中的UtmSource，解析出用户类型为买量用户，userType=" + userType + "，二级类型为：" + sUserType);

        } else if (referrer.contains(BuySdkConstants.FROM_3G_CHANNEL)) {
            userType = UserTypeInfo.FirstUserType.withCount;
            sUserType = UserTypeInfo.SecondUserType.WITHCOUNT_NOT_ORGNIC;
            //取出关键字信息
            String fromUserType = getFromUserType(referrer);
            if (!TextUtils.isEmpty(fromUserType)) {
                //在UtmSource不为空且不包含google-play的情况下，不是买量，那就是带量


//                LogUtils.i(BuySdkConstants.LOG_TAG, "[ReferrerManager::analyseUtmSource]解析GA中的UtmSource，解析出用户类型为带量用户，userType=" + userType + "，二级类型为：" + sUserType);

                //区分是哪几种带量
                if (fromUserType.equals(UserTypeInfo.FirstUserType.apkbuy.toString())) {
                    //apk买量的带量
                    sUserType = UserTypeInfo.SecondUserType.WITHCOUNT_NOT_ORGNIC;
                    LogUtils.i(BuySdkConstants.LOG_TAG, "[ReferrerManager::analyseUtmSource]带量规则解析GA中的UtmSource，解析出用户类型为非自然的带量，from_3g_channel=" + fromUserType + "，二级类型为：" + sUserType);
                }

                if (fromUserType.equals(UserTypeInfo.FirstUserType.withCount.toString())) {
                    //带量的带量
                    sUserType = UserTypeInfo.SecondUserType.WITHCOUNT_NOT_ORGNIC;
                    LogUtils.i(BuySdkConstants.LOG_TAG, "[ReferrerManager::analyseUtmSource]带量规则解析GA中的UtmSource，解析出用户类型为非自然的带量，from_3g_channel=" + fromUserType + "，二级类型为：" + sUserType);
                }


                if (fromUserType.equals(UserTypeInfo.FirstUserType.userbuy.toString())) {
                    //一般买量的带量
                    sUserType = UserTypeInfo.SecondUserType.WITHCOUNT_NOT_ORGNIC;
                    LogUtils.i(BuySdkConstants.LOG_TAG, "[ReferrerManager::analyseUtmSource]带量规则解析GA中的UtmSource，解析出用户类型为非自然的带量，from_3g_channel=" + fromUserType + "，二级类型为：" + sUserType);

                }

                if (fromUserType.equals(UserTypeInfo.FirstUserType.organic.toString())) {
                    //自然的带量
                    sUserType = UserTypeInfo.SecondUserType.WITHCOUNT_ORGNIC;
                    LogUtils.i(BuySdkConstants.LOG_TAG, "[ReferrerManager::analyseUtmSource]带量规则解析GA中的UtmSource，解析出用户类型为自然的带量，from_3g_channel=" + fromUserType + "，二级类型为：" + sUserType);

                }
            }


        } else if (isGpOrgnic) {
            //GP自然用户
            userType = UserTypeInfo.FirstUserType.organic;
            sUserType = UserTypeInfo.SecondUserType.GP_ORGNIC;

        } else if (isNotGpOrgnic) {
            //非GP自然用户
            userType = UserTypeInfo.FirstUserType.organic;
            sUserType = UserTypeInfo.SecondUserType.NOT_GP_ORGNIC;

        } else {
            //没有from_3g_channel标识，会认定为非自然带量
            userType = UserTypeInfo.FirstUserType.withCount;
            sUserType = UserTypeInfo.SecondUserType.WITHCOUNT_NOT_ORGNIC;
            LogUtils.i(BuySdkConstants.LOG_TAG, "[ReferrerManager::analyseUtmSource]没有from_3g_channel标识,又不是买量，判断为认定为非自然带量，userType=" + userType + "，二级类型为：" + sUserType);
        }


        final String finalReferrer = referrer;
        final UserTypeInfo.SecondUserType finalSUserType = sUserType;
        BuyChannelSetting.getInstance(mContext).setBuyChannel(utmSource, BuyChannelSetting.ChannelFrom.from_ga,
                userType, sUserType, null, null, referrer, null, null, referrer, null, new SetBuyChannelListener() {

                    @Override
                    public void setBuyChannelSuccess() {

                    }

                    @Override
                    public void setBuyChannelFailure(final String position) {
                        CustomThreadExecutorProxy.getInstance().runOnAsyncThread(new Runnable() {
                            @Override
                            public void run() {
                                if (ReferrerManager.getInstance(mContext).checkFirstReceiver() && !BuyChannelUtils.isOldApkBuy(mContext)) {
                                    //说明是第一次GA，如果第一次GA中setBuyChannel失败，则上传一次45
//                            BuySdkInitParams mInitParams = AppsFlyerProxy.getInstance().getInitParams();
                                    if (mSp == null) {
                                        mSp = BuyChannelDataMgr.getInstance(mContext).getSharedPreferences(mContext);
                                    }
                                    boolean isGoKeyBoard = mSp.getBoolean(IS_GOKEYBORAD, false);
                                    int fun45Id = mSp.getInt(FUN_ID_45, 0);
                                    //存放原始数据的45协议，
                                    UserStatistics.upload45(mContext, isGoKeyBoard, fun45Id,
                                            finalReferrer, null, finalReferrer, String.valueOf(finalSUserType.getValue()), "-1", null, false);
//                            LogUtils.i(BuySdkConstants.LOG_TAG, "[ReferrerManager::analyseUtmSource] 第一次GA中setBuyChannel失败，上传一次45");
                                } else if (BuySdkConstants.Position_103.POSITION_7.getValue().equals(position)) {
                                    //操作码2,位置7,GA买量，旧的已经存在APK或fb,adwords，没有上传45协议
                                    StatisticsDebug.Statistic103Params statistic103ParamsP7 = new StatisticsDebug.Statistic103Params();
                                    statistic103ParamsP7.code(BuySdkConstants.DEBUG_CODE2).referrer(finalReferrer).position(7);
                                    StatisticsDebug.upload(mContext, statistic103ParamsP7);

                                } else if (BuySdkConstants.Position_103.POSITION_8.getValue().equals(position)) {
                                    //操作码2,位置8,GA带量，没有覆盖 成功，没有上传45协议
                                    StatisticsDebug.Statistic103Params statistic103ParamsP8 = new StatisticsDebug.Statistic103Params();
                                    statistic103ParamsP8.code(BuySdkConstants.DEBUG_CODE2).referrer(finalReferrer).position(8);
                                    StatisticsDebug.upload(mContext, statistic103ParamsP8);
                                } else if (BuySdkConstants.Position_103.POSITION_9.getValue().equals(position)) {
                                    //操作码2,位置9,GA自然，旧的优先级高，没有覆盖，没有上传45协议
                                    StatisticsDebug.Statistic103Params statistic103ParamsP9 = new StatisticsDebug.Statistic103Params();
                                    statistic103ParamsP9.code(BuySdkConstants.DEBUG_CODE2).referrer(finalReferrer).position(9);
                                    StatisticsDebug.upload(mContext, statistic103ParamsP9);

                                }

                            }
                        });

                    }
                });


    }
}
