package com.jiubang.commerce.buychannel.buyChannel.manager;

import android.content.Context;

import com.jb.ga0.commerce.util.CustomAlarm;
import com.jb.ga0.commerce.util.CustomAlarmManager;
import com.jb.ga0.commerce.util.LogUtils;
import com.jiubang.commerce.buychannel.BuyChannelApi;
import com.jiubang.commerce.buychannel.BuyChannelDataMgr;
import com.jiubang.commerce.buychannel.BuyChannelSetting;
import com.jiubang.commerce.buychannel.BuySdkConstants;
import com.jiubang.commerce.buychannel.buyChannel.bean.BuyChannelBean;
import com.jiubang.commerce.buychannel.buyChannel.utils.BuyChannelUtils;

/**
 * Created by mayunlong on 16-9-29.
 */

public class ServerCheckHelper {

    private static ServerCheckHelper sInstance;
    private Context mContext;

    public static ServerCheckHelper getInstance(Context context) {
        if (sInstance == null) {
            synchronized (ServerCheckHelper.class) {
                if (sInstance == null) {
                    sInstance = new ServerCheckHelper(context);
                }
            }
        }
        return sInstance;
    }

    private ServerCheckHelper(Context context) {
        mContext = context != null ? context.getApplicationContext() : null;
    }

    public void startCheckUserTag(final long triggerTime) {

        BuyChannelDataMgr.getInstance(mContext).saveFirstCheckUserTagTime(System.currentTimeMillis());
        LogUtils.i(BuySdkConstants.LOG_TAG, "[ServerCheckHelper::startCheckUserTag]2天内，每间隔８小时，再去查询一次买量标签表" );
        CustomAlarmManager.getInstance(mContext).getAlarm(BuySdkConstants.USERTAG_ALARM_NAME).alarmRepeat(BuySdkConstants.USERTAG_ALARM_ID, triggerTime,
                BuySdkConstants.INTERVAL_SERVER_CHECKTIME, true, new CustomAlarm.OnAlarmListener() {
                    @Override
                    public void onAlarm(int i) {

                        long firstCheckTime = BuyChannelDataMgr.getInstance(mContext).getFirstCheckUserTagTime();
                        if (firstCheckTime != 0) {

                            //检查时间是否超过2天
                            long intervalTime = System.currentTimeMillis() - firstCheckTime;
                            LogUtils.i(BuySdkConstants.LOG_TAG, "[ServerCheckHelper::startCheckServer] intervalTime:" + intervalTime);

                            //超过2天不再执行，取消定时器
                            if (intervalTime - BuySdkConstants.USERTAG_LIMIT_DAY > 0) {
                                LogUtils.i(BuySdkConstants.LOG_TAG, "[ServerCheckHelper::startCheckUserTag] 已经超过2天时间，不再检测用户标签接口]");
                                CustomAlarmManager.getInstance(mContext).getAlarm(BuySdkConstants.USERTAG_ALARM_NAME).cancelAarm(BuySdkConstants.USERTAG_ALARM_ID);
                                return;
                            }
                        }

                        OldUserUpHelper.getInstance(mContext).checkUserTagAfterUserTable(BuySdkConstants.CHECK_USERTAG_USERTABLE_OLDUSER);


                    }
                });
    }

    public void startCheckServer(final long triggerTime) {


        CustomAlarmManager.getInstance(mContext).getAlarm("buychannelsdk").alarmRepeat(BuySdkConstants.ALARM_ID, triggerTime,
                BuySdkConstants.INTERVAL_SERVER_CHECKTIME, true,
                new CustomAlarm.OnAlarmListener() {
                    @Override
                    public void onAlarm(int i) {

                        //check前检查旧的是否为买量用户或自然带量，如果是，就不需要check了
                        BuyChannelBean oldBuyChannelBean = BuyChannelApi.getBuyChannelBean(mContext);
                        if (oldBuyChannelBean != null) {
                            if (BuyChannelUtils.isOldUserBuy(mContext)) {
                                LogUtils.i(BuySdkConstants.LOG_TAG, "[ServerCheckHelper::startCheckServer] 旧数据已经是买量，无需再server-check");
                                cancelCheckServer();
                                return;
                            }
                            if (BuyChannelUtils.isOldApkBuy(mContext)) {
                                LogUtils.i(BuySdkConstants.LOG_TAG, "[ServerCheckHelper::startCheckServer] 旧数据已经是apk买量，无需再server-check");
                                cancelCheckServer();
                                return;
                            }

                            if (oldBuyChannelBean.getSecondUserType() == 0) {
                                LogUtils.i(BuySdkConstants.LOG_TAG, "[ServerCheckHelper::startCheckServer] 旧数据已经是自然带量，无需再server-check");
                                cancelCheckServer();
                                return;
                            }

                            if (oldBuyChannelBean.getChannelFrom().equals(BuyChannelSetting.ChannelFrom.from_oldUser.toString())) {
                                LogUtils.i(BuySdkConstants.LOG_TAG, "[ServerCheckHelper::startCheckServer] 旧数据已经老用户并确定身份，无需再server-check");
                                cancelCheckServer();
                                return;
                            }


                        }

                        long firstCheckTime = BuyChannelDataMgr.getInstance(mContext).getFirstCheckTime();
                        if (firstCheckTime != 0) {

                            //检查时间是否超过5天
                            long intervalTime = System.currentTimeMillis() - firstCheckTime;
                            LogUtils.i(BuySdkConstants.LOG_TAG, "[ServerCheckHelper::startCheckServer] intervalTime:" + intervalTime);

                            //超过5天不再执行，取消定时器
                            if (intervalTime - BuySdkConstants.LIMIT_DAY > 0) {
                                LogUtils.i(BuySdkConstants.LOG_TAG, "[ServerCheckHelper::startCheckServer] 已经超过5天时间，不再检测]");
                                CustomAlarmManager.getInstance(mContext).getAlarm("buychannelsdk").cancelAarm(BuySdkConstants.ALARM_ID);
                                return;
                            }
                        }

                        OldUserUpHelper.getInstance(mContext).refreshNewUserTag(BuySdkConstants.CHECK_USERTAG_NEWUSER);


                    }
                });

    }

    public void cancelCheckUserTag() {
        CustomAlarmManager.getInstance(mContext).getAlarm(BuySdkConstants.USERTAG_ALARM_NAME).cancelAarm(BuySdkConstants.USERTAG_ALARM_ID);
    }

    public void cancelCheckServer() {
        CustomAlarmManager.getInstance(mContext).getAlarm(BuySdkConstants.ALARM_NAME).cancelAarm(BuySdkConstants.ALARM_ID);
    }
}
