package com.jiubang.commerce.buychannel.buyChannel.manager;

import android.content.Context;
import android.content.SharedPreferences;

import com.jb.ga0.commerce.util.LogUtils;
import com.jiubang.commerce.ad.AdSdkApi;
import com.jiubang.commerce.ad.bean.AdUserTagInfoBean;
import com.jiubang.commerce.ad.manager.AdSdkManager;
import com.jiubang.commerce.ad.manager.AdSdkSetting;
import com.jiubang.commerce.ad.params.UserTagParams;
import com.jiubang.commerce.buychannel.BuyChannelDataMgr;
import com.jiubang.commerce.buychannel.BuySdkConstants;
import com.jiubang.commerce.buychannel.buyChannel.Interface.OldUserTagListenner;
import com.jiubang.commerce.buychannel.buyChannel.bean.UserTagParam;
import com.jiubang.commerce.buychannel.buyChannel.bean.UserTypeInfo;

/**
 * Created by mayunlong on 16-10-21.
 */

public class UserTagHelper {
    private static UserTagHelper sInstance;
    private Context mContext;
    private AdUserTagInfoBean mAdUserTagInfoBean;

    public static UserTagHelper getInstance(Context context) {
        if (sInstance == null) {
            synchronized (UserTagHelper.class) {
                if (sInstance == null) {
                    sInstance = new UserTagHelper(context);
                }
            }
        }
        return sInstance;
    }

    private UserTagHelper(Context context) {
        mContext = context != null ? context.getApplicationContext() : null;
    }


    /**
     * 向服务器请求标签接口，返回二级用户类型，非买量和请求失败，均返回null
     * created by mayunlong.
     * created date 16-10-24.
     */
    public UserTypeInfo.SecondUserType requestUserTag(final OldUserTagListenner listenner, boolean isNew) {
        SharedPreferences sp = BuyChannelDataMgr.getInstance(mContext).getSharedPreferences(mContext);
        //买量SDK里面保存的请求标签接口的相关参数
        UserTagParam params = UserTagParam.jsonStr2Bean(sp.getString(BuySdkConstants.USERTAG_PARAMS, null));
        if (params == null) {
            return null;
        }
        //构造传给广告SDK的参数
        UserTagParams tagParams = new UserTagParams(params.mGoId, params.mGoogleId, params.mChannel, params.mProductKey, params.mAccessKey);
        AdSdkApi.requestUserTags(mContext, new AdSdkManager.IAdvertUserTagResultListener() {
            @Override
            public void onAdRequestSuccess(AdUserTagInfoBean adUserTagInfoBean) {

                LogUtils.i(BuySdkConstants.LOG_TAG, "[UserTagHelper::requestUserTag]广告SDK获取用户标签成功");
                LogUtils.i("AutoTestAFib", "广告SDK获取用户标签成功");
                if (listenner != null) {
                    listenner.requestUserTagSuccess();
                }
            }


            @Override
            public void onAdRequestFail(int i) {

                LogUtils.i(BuySdkConstants.LOG_TAG, "[UserTagHelper::requestUserTag]广告SDK获取用户标签失败,错误代码：" + i);
                LogUtils.i("AutoTestAFib", "广告SDK获取用户标签失败,错误代码：" + i);

            }
        }, tagParams, isNew);

        if (isNew) {
            mAdUserTagInfoBean = AdSdkSetting.getInstance(mContext)
                    .getUserTagInfoBean();
        } else {
            mAdUserTagInfoBean = AdSdkSetting.getInstance(mContext).getOldUserTagInfoBean();
        }


        if (mAdUserTagInfoBean != null) {
            LogUtils.i(BuySdkConstants.LOG_TAG, "[UserTagHelper::requestUserTag]标签列表内容为," + mAdUserTagInfoBean.getUserTags().toString());
            LogUtils.i("AutoTestAFib", "标签列表内容为," + mAdUserTagInfoBean.getUserTags().toString());
        }

        //根据找回的买量标签进行各种判断
        if (mAdUserTagInfoBean.isTag("C2_APK")) {
            //APK买量
            if (mAdUserTagInfoBean != null) {
                LogUtils.i(BuySdkConstants.LOG_TAG, "[UserTagHelper::requestUserTag]通过标签识别为APK买量，标签列表内容为,"
                        + mAdUserTagInfoBean.getUserTags().toString());
            }
            return UserTypeInfo.SecondUserType.APK_USERBUY;
        } else if (mAdUserTagInfoBean.isTag("C2_GMFB")) {
            //FB自投
            if (mAdUserTagInfoBean != null) {
                LogUtils.i(BuySdkConstants.LOG_TAG, "[UserTagHelper::requestUserTag]通过标签识别为FB自投，标签列表内容为,"
                        + mAdUserTagInfoBean.getUserTags().toString());
            }
            return UserTypeInfo.SecondUserType.FB_AUTO;

        } else if (mAdUserTagInfoBean.isTag("C2_FB")) {
            //FB非自投
            if (mAdUserTagInfoBean != null) {
                LogUtils.i(BuySdkConstants.LOG_TAG, "[UserTagHelper::requestUserTag]通过标签识别为FB非自投，标签列表内容为,"
                        + mAdUserTagInfoBean.getUserTags().toString());
            }
            return UserTypeInfo.SecondUserType.FB_NOTAUTO;
        } else if (mAdUserTagInfoBean.isTag("C2_GMADW")) {
            //Adwords自投
            if (mAdUserTagInfoBean != null) {
                LogUtils.i(BuySdkConstants.LOG_TAG, "[UserTagHelper::requestUserTag]通过标签识别为Adwords自投，标签列表内容为,"
                        + mAdUserTagInfoBean.getUserTags().toString());
            }
            return UserTypeInfo.SecondUserType.ADWORDS_AUTO;
        } else if (mAdUserTagInfoBean.isTag("C2_ADW")) {
            //Adwords非自投
            if (mAdUserTagInfoBean != null) {
                LogUtils.i(BuySdkConstants.LOG_TAG, "[UserTagHelper::requestUserTag]通过标签识别为Adwords非自投，标签列表内容为,"
                        + mAdUserTagInfoBean.getUserTags().toString());
            }
            return UserTypeInfo.SecondUserType.ADWORDS_NOTAUTO;
        } else if (mAdUserTagInfoBean.isTag("C2_GA")) {
            //识别为GA买量
            if (mAdUserTagInfoBean != null) {
                LogUtils.i(BuySdkConstants.LOG_TAG, "[UserTagHelper::requestUserTag]通过标签识别为GA买量，标签列表内容为,"
                        + mAdUserTagInfoBean.getUserTags().toString());
            }
            return UserTypeInfo.SecondUserType.GA_USERBUY;
        } else if (mAdUserTagInfoBean.isTag("C2_3G")) {
            //自然带量
            if (mAdUserTagInfoBean != null) {
                LogUtils.i(BuySdkConstants.LOG_TAG, "[UserTagHelper::requestUserTag]通过标签识别为自然带量，标签列表内容为,"
                        + mAdUserTagInfoBean.getUserTags().toString());
            }
            return UserTypeInfo.SecondUserType.WITHCOUNT_ORGNIC;
        } else if (mAdUserTagInfoBean.isTag("C2")) {
            //识别不出具体二级类型
            if (mAdUserTagInfoBean != null) {
                LogUtils.i(BuySdkConstants.LOG_TAG, "[UserTagHelper::requestUserTag]通过标签识别为买量用户,识别不出具体二级类型，标签列表内容为,"
                        + mAdUserTagInfoBean.getUserTags().toString());
            }
            return UserTypeInfo.SecondUserType.UNKNOWN_USERBUY;
        } else {
            LogUtils.i(BuySdkConstants.LOG_TAG, "[UserTagHelper::requestUserTag]通过标签识为非买量用户");
        }
        return null;
    }
}
