package com.jiubang.commerce.buychannel.buyChannel.utils;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.content.pm.Signature;
import android.text.TextUtils;
import android.util.Base64;
import android.util.Log;

import com.jb.ga0.commerce.util.LogUtils;
import com.jiubang.commerce.buychannel.BuyChannelDataMgr;
import com.jiubang.commerce.buychannel.BuySdkConstants;

import java.security.MessageDigest;

/**
 * @author linzewu
 * @date 16-8-26
 */
public class AppInfoUtils {

    private static String sGoogleAdvertingId;    // google ad id
    public static final String GOOGLE_ADVERTING_DEFAULT_ID = "UNABLE-TO-RETRIEVE";

    /**
     * 获取程序的版本versionCode
     *
     * @param context Context
     * @return 程序自身的版本versionCode
     */
    public static int getVersionCode(Context context) {
        int versionCode = 0;
        String packageName = context.getPackageName();
        PackageInfo packageInfo;
        try {
            packageInfo = context.getPackageManager().getPackageInfo(packageName, 0);
            versionCode = packageInfo.versionCode;
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }

        return versionCode;
    }

    /**
     * 获取程序的版本
     *
     * @param context Context
     * @return 程序自身的版本
     */
    public static String getVersionName(Context context) {
        String versionName = null;
        String packageName = context.getPackageName();
        PackageInfo packageInfo;
        try {
            packageInfo = context.getPackageManager().getPackageInfo(packageName, 0);
            versionName = packageInfo.versionName;
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }

        return versionName;
    }

    /**
     * 获取程序的名称
     *
     * @param context
     * @return
     */
    public static String getAppName(Context context) {
        String packageName = context.getPackageName();
        PackageInfo packageInfo;
        try {
            packageInfo = context.getPackageManager().getPackageInfo(
                    packageName, 0);
            return packageInfo.applicationInfo.loadLabel(
                    context.getPackageManager()).toString();
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }

        return "";
    }

    /**
     * <br>功能简述:获取Advertising Id
     * <br>功能详细描述:
     * <br>注意:the device specific Advertising ID provided by the Google Play Services, <em>null</em> if an error occurred.
     *
     * @param context
     * @returns
     */
    public static String getAdvertisingId(final Context context) {
        if (!TextUtils.isEmpty(sGoogleAdvertingId)) {
            return sGoogleAdvertingId;
        }
        sGoogleAdvertingId = BuyChannelDataMgr.getInstance(context).getSharedPreferences(context).getString(BuySdkConstants.GOOGLE_AD_ID, null);

        if (!TextUtils.isEmpty(sGoogleAdvertingId)) {
            return sGoogleAdvertingId;
        }

//        if (!isMainThread()) {
//            sGoogleAdvertingId = getGoogleAdvertisingId(context);
//            if (!TextUtils.isEmpty(sGoogleAdvertingId)) {
//                new SharedPreferencesUtils(context, SharedPreferencesUtils.GOOGLE_AD_ID).putString( SharedPreferencesUtils.GOOGLE_AD_ID, sGoogleAdvertingId);
//                return sGoogleAdvertingId;
//            }
//        } else {
        new Thread(new Runnable() {

            @Override
            public void run() {
                sGoogleAdvertingId = getGoogleAdvertisingId(context);
                if (!TextUtils.isEmpty(sGoogleAdvertingId)) {

                    SharedPreferences sp = BuyChannelDataMgr.getInstance(context).getSharedPreferences(context);
                    sp.edit().putString(BuySdkConstants.GOOGLE_AD_ID, sGoogleAdvertingId).commit();

                }
            }
        }, "getAdvertisingId").start();
//        }

        return GOOGLE_ADVERTING_DEFAULT_ID;
    }

    private static String getGoogleAdvertisingId(Context context) {
        AdvertisingIdClient.AdInfo info = null;
        try {
            info = AdvertisingIdClient.getAdvertisingIdInfo(context);
        } catch (IllegalStateException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } catch (Throwable e) {
            e.printStackTrace();
        }

        if (info != null) {
            return info.getId();
        } else {
            return null;
        }
    }

    /**
     * 获取正在运行桌面包名（注：存在多个桌面时且未指定默认桌面时，该方法返回Null,使用时需处理这个情况）
     */
    public static String getLauncherPackageName(Context context) {
        try {
            final Intent intent = new Intent(Intent.ACTION_MAIN);
            intent.addCategory(Intent.CATEGORY_HOME);
            final ResolveInfo res = context.getPackageManager().resolveActivity(intent, 0);
            if (res.activityInfo == null) {
                // should not happen. A home is always installed, isn't it?
                return null;
            }
            if (res.activityInfo.packageName.equals("android")) {
                // 有多个桌面程序存在，且未指定默认项时；
                return null;
            } else {
                return res.activityInfo.packageName;
            }
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 应用是否存在
     *
     * @param context
     * @param apkName
     * @return
     */
    public static boolean isAppExit(Context context, String apkName) {
        PackageInfo pkgInfo = null;
        try {
            PackageManager pm = context.getPackageManager();
            pkgInfo = pm.getPackageInfo(apkName, 0);
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
        return pkgInfo != null ? true : false;
    }

    /**
     * 打开应用
     *
     * @param context
     * @param apkName
     */
    public static void openAppWithPkgName(Context context, String apkName) {
        try {
            PackageManager pm = context.getPackageManager();
            Intent intent = new Intent();
            intent = pm.getLaunchIntentForPackage(apkName);
            context.startActivity(intent);
        } catch (Exception e) {
            LogUtils.e(null, e.toString());
        }
    }

    /**
     * 输出密钥散列，需要在打出正式包的时候观察log，一般情况下不调用
     * printKeyHash
     */
    public static void printKeyHash(Context context) {
        // Add code to print out the key hash
        try {
            PackageInfo info = context.getPackageManager().getPackageInfo("com.jiubang.commerce" +
                            ".gomultiple",
                    PackageManager.GET_SIGNATURES);
            for (Signature signature : info.signatures) {
                MessageDigest md = MessageDigest.getInstance("SHA");
                md.update(signature.toByteArray());
                Log.d("KeyHash:", Base64.encodeToString(md.digest(), Base64.DEFAULT));
            }
        } catch (Exception e) {
        }
    }

    /**
     * 获取应用程序的包名
     */
    public static String getPackageName(Context context) {
        return context.getPackageName();

    }

}
