package com.jiubang.commerce.buychannel.buyChannel.utils;

import android.content.Context;
import android.content.SharedPreferences;
import android.telephony.TelephonyManager;
import android.text.TextUtils;

import com.jb.ga0.commerce.util.LogUtils;
import com.jiubang.commerce.buychannel.BuyChannelDataMgr;
import com.jiubang.commerce.buychannel.BuySdkConstants;
import com.jiubang.commerce.buychannel.buyChannel.bean.BuyChannelBean;
import com.jiubang.commerce.buychannel.buyChannel.bean.UserTypeInfo;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.Locale;

/**
 * Created by Administrator on 2016/9/25.
 */

public class BuyChannelUtils {


    /**
     * 获取SIM卡国家是否成功，返回1为成功，0为不成功
     * created by mayunlong.
     * created date 16-10-24.
     */
    public static String getCountrySuccess(Context context) {
        String ret = null;

        try {
            TelephonyManager e = (TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE);
            if (e != null) {
                ret = e.getSimCountryIso().toUpperCase();
                return 0 + "";
            }
        } catch (Throwable var3) {

        }

        if (TextUtils.isEmpty(ret)) {
            ret = Locale.getDefault().getCountry().toUpperCase();
            return -1 + "";
        }

        return -1 + "";
    }

    /**
     * 获取缓存中的45协议utm_source数据
     * created by mayunlong.
     * created date 16-11-3.
     */
    public static String getOldSender(Context context) {
        SharedPreferences sp = BuyChannelDataMgr.getInstance(context).getSharedPreferences(context);
        return sp.getString(BuySdkConstants.OLD_SENDER, null);
    }
    /**
     * 设置缓存中的45协议utm_source数据
     * created by mayunlong.
     * created date 16-11-3.
     */
    public static void setOldSender(Context context, String sender) {
        SharedPreferences sp = BuyChannelDataMgr.getInstance(context).getSharedPreferences(context);
        sp.edit().putString(BuySdkConstants.OLD_SENDER, sender).commit();
    }

    public static String getOldUserType(Context context) {
        SharedPreferences sp = BuyChannelDataMgr.getInstance(context).getSharedPreferences(context);
        return sp.getString(BuySdkConstants.OLD_USERTUPE, null);
    }

    public static void setOldUserType(Context context, String userType) {
        SharedPreferences sp = BuyChannelDataMgr.getInstance(context).getSharedPreferences(context);
        sp.edit().putString(BuySdkConstants.OLD_USERTUPE, userType).commit();
    }



    public static BuyChannelBean jsonStr2BuyChannelBean(String jsonString) {
        if (TextUtils.isEmpty(jsonString)) {
            return null;
        }
        try {
            JSONObject object = new JSONObject(jsonString);
            BuyChannelBean buyChannelBean = new BuyChannelBean();
            buyChannelBean.setChannelFrom(object.getString("channelFrom"));
            buyChannelBean.setBuyChannel(object.getString("buyChannel"));
            buyChannelBean.setFirstUserType(object.getString("firstUserType"));
            buyChannelBean.setSecondUserType(Integer.parseInt(object.getString("userType")));
            buyChannelBean.setSuccessCheck(Boolean.parseBoolean(object.optString("isSuccessCheck")));
            buyChannelBean.setCampaign(object.optString("campaign"));
            buyChannelBean.setCampaignId(object.optString("campaignId"));
            return buyChannelBean;
        } catch (JSONException e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 是否老用户
     * created by mayunlong.
     * created date 16-11-3.
     */
    public static boolean isOldUser(Context context) {
        SharedPreferences mSp = BuyChannelDataMgr.getInstance(context).getSharedPreferences(context);

        String oldUserMsg = mSp.getString(BuySdkConstants.OLD_USER_MSG, null);
        boolean isOldUser = false;
        if (!TextUtils.isEmpty(oldUserMsg)) {
            String[] oldMsg = oldUserMsg.split(BuySdkConstants.SEPARATOR);

            if (oldMsg.length < 3) {

                isOldUser = Boolean.parseBoolean(oldMsg[1]);

            }
            LogUtils.i(BuySdkConstants.LOG_TAG, "获取老用户,[BuyChannelUtils::isOldUser]   isOldUser:" + isOldUser);

        }
        return isOldUser;
    }

    /**
     * 缓存数据是否为apk买量
     * created by mayunlong.
     * created date 16-11-3.
     */
    public static boolean isOldApkBuy(Context context) {
        //取出缓存的买量信息，查看是否为Apk买量
        BuyChannelBean oldBuyChannelBean = BuyChannelDataMgr.getInstance(context).getBuyChannelBean();
        if (oldBuyChannelBean != null && oldBuyChannelBean.getFirstUserType().equals(UserTypeInfo.FirstUserType.apkbuy.toString())) {
            LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelUtils::isOldApkBuy] 缓存中，已经保存了apk买量:" + oldBuyChannelBean.toString());
            return true;
        }
        return false;

    }

    /**
     * 缓存数据是否为带量
     * created by mayunlong.
     * created date 16-11-3.
     */
    public static boolean isOldWithCount(Context context) {
        //取出缓存的买量信息，查看是否为带量
        BuyChannelBean oldBuyChannelBean = BuyChannelDataMgr.getInstance(context).getBuyChannelBean();
        if (oldBuyChannelBean != null && oldBuyChannelBean.getFirstUserType().equals(UserTypeInfo.FirstUserType.withCount.toString())) {
            LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelUtils::isOldWithCount] 缓存中，已经保存了带量:" + oldBuyChannelBean.toString());
            return true;
        }
        return false;
    }

    /**
     * 缓存数据是否为自然
     * created by mayunlong.
     * created date 16-11-3.
     */
    public static boolean isOldOrgnic(Context context) {
        //取出缓存的买量信息，查看是否为自然
        BuyChannelBean oldBuyChannelBean = BuyChannelDataMgr.getInstance(context).getBuyChannelBean();
        if (oldBuyChannelBean != null && oldBuyChannelBean.getFirstUserType().equals(UserTypeInfo.FirstUserType.organic.toString())) {
            LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelUtils::isOldOrgnic] 缓存中，已经保存了自然:" + oldBuyChannelBean.toString());
            return true;
        }
        return false;
    }

    /**
     * 缓存中是否fb,adwords,twitter之一
     * created by mayunlong.
     * created date 16-11-3.
     */
    public static boolean isOldFbAdTw(Context context) {
        //取出缓存的买量信息，查看是否为fb,adwords
        BuyChannelBean oldBuyChannelBean = BuyChannelDataMgr.getInstance(context).getBuyChannelBean();
        if (oldBuyChannelBean != null && oldBuyChannelBean.getSecondUserType() == 2) {
            LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelUtils::isOldFbAdTw] 缓存中，已经保存了fb自投:" + oldBuyChannelBean.toString());
            return true;
        }

        if (oldBuyChannelBean != null && oldBuyChannelBean.getSecondUserType() == 3) {
            LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelUtils::isOldFbAdTw] 缓存中，已经保存了fb非自投:" + oldBuyChannelBean.toString());
            return true;
        }

        if (oldBuyChannelBean != null && oldBuyChannelBean.getSecondUserType() == 4) {
            LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelUtils::isOldFbAdTw] 缓存中，已经保存了adwords自投:" + oldBuyChannelBean.toString());
            return true;
        }

        if (oldBuyChannelBean != null && oldBuyChannelBean.getSecondUserType() == 6) {
            LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelUtils::isOldFbAdTw] 缓存中，已经保存了adwords非自投:" + oldBuyChannelBean.toString());
            return true;
        }
        return false;
    }

    /**
     * explain 缓存数据是否为空
     * created by mayunlong.
     * created date 16-11-3.
     */
    public static boolean isOldEmpty(Context context) {
        //取出缓存的买量信息，查看是否为空
        BuyChannelBean oldBuyChannelBean = BuyChannelDataMgr.getInstance(context).getBuyChannelBean();
        if (oldBuyChannelBean == null) {
//            LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelUtils::isOldEmpty] 缓存中数据为空");
            return true;
        }
        return false;
    }

    /**
     * 缓存数据是否为买量用户
     * created by mayunlong.
     * created date 16-11-3.
     */
    public static boolean isOldUserBuy(Context context) {
        //取出缓存的买量信息，查看是否为自然
        BuyChannelBean oldBuyChannelBean = BuyChannelDataMgr.getInstance(context).getBuyChannelBean();
        if (oldBuyChannelBean != null && oldBuyChannelBean.getFirstUserType().equals(UserTypeInfo.FirstUserType.userbuy.toString())) {
            LogUtils.i(BuySdkConstants.LOG_TAG, "[BuyChannelUtils::isOldUserBuy] 缓存中数据为一般买量数据");
            return true;
        }
        return false;
    }


    public static String getIMEI(Context context) {
        TelephonyManager telephonyManager = (TelephonyManager)context.getSystemService(Context.TELEPHONY_SERVICE);
        String imei = telephonyManager.getDeviceId();
        return imei;
    }

}
