package com.jiubang.core.message;

import java.util.ArrayList;
import java.util.List;

import android.os.Handler;
import android.os.Looper;
import android.os.Message;

/**
 * 消息管理器
 * 
 * @author yuankai
 * @version 1.0
 */
public class MessageManager {
	private final static int HANDLE_MSG = 0;
	private final static int HANDLE_MSG_BROADCAST = 1;
	
	private MessageHandlerPool mHandlerPool;
	private IMessageFilter mMessageFilter = null;
	
	/**
	 * 基础消息的ID，框架内部用途，其他所有外部定义的消息必须大于它
	 */
	public final static int BASE_MSG_ID = -1;
	private boolean mStop = false;
	
	public MessageManager()
	{
		mHandlerPool = new MessageHandlerPool();
	}
	
	public void setFilter(IMessageFilter filter)
	{
		mMessageFilter = filter;
	}
	
	static void handleBean(MessageBean bean)
	{
		if (bean == null)
		{
			return;
		}
		
		List list = bean.mObjects.getList();
		int len = list.size();
		if(bean.mFilter != null) {
			// 为了兼容之前的做法，用这个比较恶心的方法
			// 变长参数不应该超过6个，超过则需要建class
			if (0 ==len) {
				bean.mFilter.handleMessage(bean.mWho, bean.mMsgId, bean.mParam);
			} else if (1 == len){
				bean.mFilter.handleMessage(bean.mWho, bean.mMsgId, bean.mParam, bean.getObject(0));
			} else if (2 == len){
				bean.mFilter.handleMessage(bean.mWho, bean.mMsgId, bean.mParam, bean.getObject(0), bean.getObject(1));
			} else if (3 == len){
				bean.mFilter.handleMessage(bean.mWho, bean.mMsgId, bean.mParam, bean.getObject(0), bean.getObject(1), bean.getObject(2));
			} else if (4 == len){
				bean.mFilter.handleMessage(bean.mWho, bean.mMsgId, bean.mParam, bean.getObject(0), bean.getObject(1), bean.getObject(2), bean.getObject(3));
			} else if (5 == len){
				bean.mFilter.handleMessage(bean.mWho, bean.mMsgId, bean.mParam, bean.getObject(0), bean.getObject(1), bean.getObject(2), bean.getObject(3), bean.getObject(4));
			} else if (6 == len){
				bean.mFilter.handleMessage(bean.mWho, bean.mMsgId, bean.mParam, bean.getObject(0), bean.getObject(1), bean.getObject(2), bean.getObject(3), bean.getObject(4), bean.getObject(5));
			}
		}
		
		List<IMessageHandler> handlers = bean.mHandlers;
		if (null != handlers) {
			// 为了兼容之前的做法，用这个比较恶心的方法
			// 变长参数不应该超过6个，超过则需要建class
			final int size = handlers.size();
			for (int i = 0; i < size; i++) {
				IMessageHandler handler = handlers.get(i);
				if (null != handler) {
					if (0 ==len) {
						handler.handleMessage(bean.mWho, bean.mMsgId, bean.mParam);
					} else if (1 == len){
						handler.handleMessage(bean.mWho, bean.mMsgId, bean.mParam, bean.getObject(0));
					} else if (2 == len){
						handler.handleMessage(bean.mWho, bean.mMsgId, bean.mParam, bean.getObject(0), bean.getObject(1));
					} else if (3 == len){
						handler.handleMessage(bean.mWho, bean.mMsgId, bean.mParam, bean.getObject(0), bean.getObject(1), bean.getObject(2));
					} else if (4 == len){
						handler.handleMessage(bean.mWho, bean.mMsgId, bean.mParam, bean.getObject(0), bean.getObject(1), bean.getObject(2), bean.getObject(3));
					} else if (5 == len){
						handler.handleMessage(bean.mWho, bean.mMsgId, bean.mParam, bean.getObject(0), bean.getObject(1), bean.getObject(2), bean.getObject(3), bean.getObject(4));
					} else if (6 == len){
						handler.handleMessage(bean.mWho, bean.mMsgId, bean.mParam, bean.getObject(0), bean.getObject(1), bean.getObject(2), bean.getObject(3), bean.getObject(4), bean.getObject(5));
					}
				}
			}
		}
	}
	
	/**
	 * 消息发送接口
	 * 
	 * @param type 消息类型
	 * @param handlerId 接收的ID
	 * @param msgId 消息ID
	 * @param param 辅助参数
	 * @param object 传递的对象
	 * @param objects 传递的对象队列
	 * @return 是否处理
	 */
	public boolean send(Object who, int handlerId, int msgId, int param, Object ...object)
	{
		if(mStop) {
			return false;
		}
		
		IMessageHandler handler = mHandlerPool.getHandler(msgId, handlerId);
		if(handler != null) {
			return handler.handleMessage(who, msgId, param, object);
		}
		return false;
	}
	
	/**
	 * 发送同步到handler上的消息
	 * @param handlerId
	 * @param msgId
	 * @param param
	 * @param object
	 * @param objects
	 */
	public void sendHandler(Object who, int handlerId, int msgId, int param, Object ...object)
	{
		if (mStop)
		{
			return;
		}
		
		IMessageHandler handler = mHandlerPool.getHandler(msgId, handlerId);
		List<IMessageHandler> handlers = new ArrayList<IMessageHandler>();
		handlers.add(handler);
		MessageBean bean = new MessageBean(who, handlers, msgId, param, object);
		
		mHandler.obtainMessage(HANDLE_MSG, bean).sendToTarget();
	}
	
	/**
	 * 同步广播消息
	 * 
	 * @param type 消息类型
	 * @param msgId 消息ID
	 * @param param 辅助参数
	 * @param object 传递的对象
	 * @param objects 传递的对象队列
	 */
	public boolean sendBroadcast(Object who, int msgId, int param, Object ...object)
	{
		boolean ret = false;
		if(mStop) {
			return ret;
		}
		
		// 过滤器优先处理广播的消息
		if(mMessageFilter != null) {
			mMessageFilter.handleMessage(who, msgId, param, object);
		}
		
		List<IMessageHandler> handlers = mHandlerPool.getAllHandlers(msgId);
		if(handlers != null) {
			final int size = handlers.size();
			for(int i = 0;i < size; i++) {
				ret |= handlers.get(i).handleMessage(who, msgId, param, object);
			}
		}
		
		return ret;
	}
	
	/**
	 * 带先后次序的同步广播消息
	 * 
	 * @param type 消息类型
	 * @param msgId 消息ID
	 * @param param 辅助参数
	 * @param object 传递的对象
	 * @param objects 传递的对象队列
	 * @param orderedHandlers 要求被排在前面的接收者ID数组
	 */
	public boolean sendBroadcastbyOrders(Object who, int msgId, int param, final int[] orderedHandlers, Object ...object)
	{
		boolean ret = false;
		if(mStop) {
			return ret;
		}
		
		// 过滤器优先处理广播的消息
		if(mMessageFilter != null) {
			ret |= mMessageFilter.handleMessage(who, msgId, param, object);
		}
		
		List<IMessageHandler> handlers = mHandlerPool.getAllHandlers(msgId, orderedHandlers);
		if(handlers != null) {
			final int size = handlers.size();
			for(int i = 0;i < size; i++) {
				ret |= handlers.get(i).handleMessage(who, msgId, param, object);
			}
		}
		
		return ret;
	}
	
	/**
	 * 发送同步到handler上的消息
	 * @param handlerId
	 * @param msgId
	 * @param param
	 * @param object
	 * @param objects
	 */
	public void sendBroadcastHandler(Object who, int msgId, int param, Object ...object)
	{
		if(mStop) {
			return;
		}
		
		List<IMessageHandler> handlers = mHandlerPool.getAllHandlers(msgId);
		MessageBean bean = new MessageBean(who, handlers, msgId, param, object);
		bean.mFilter = mMessageFilter;
		mHandler.obtainMessage(HANDLE_MSG_BROADCAST, bean).sendToTarget();
	}
	
	/**
	 * 提交一次在UI线程上执行的请求
	 * @param who
	 * @param runnable 需要在UI线程上执行的操作
 	 * @param atFront 是否优先执行
	 */
	public void postUiRunnable(Object who, Runnable runnable, boolean atFront) {
		if(mStop){
			return;
		}
		
		if(atFront) {
			mHandler.postAtFrontOfQueue(runnable);
		} else {
			mHandler.post(runnable);
		}
	}
	
	/**
	 * 添加消息处理者
	 * 
	 * @param handler 消息处理者
	 * @return 是否添加成功
	 */
	public boolean registMsgHandler(final IMessageHandler handler)
	{
		if(mStop) {
			return false;
		}
		return mHandlerPool.registMsgHandler(handler);
	}
	
	public boolean registMsgHandler(final IMessageHandler handler, final int msgId)
	{
		if(!mStop && msgId > BASE_MSG_ID)
		{
			return mHandlerPool.registMsgHandler(handler, msgId);
		}
		return false;
	}
	
	public boolean unRegistMsgHandler(final IMessageHandler handler, final int msgId)
	{
		if(mStop) {
			return false;
		}
		return mHandlerPool.unRegistMsgHandler(handler, msgId);
	}
	
	/**
	 * 移除消息处理者
	 * 
	 * @param handler 消息处理者
	 * @return 是否移除成功
	 */
	public boolean unRegistMsgHandler(IMessageHandler handler)
	{
		if(mStop) {
			return false;
		}
		
		return mHandlerPool.unRegistMsgHandler(handler);
	}

	public void cleanup()
	{
		mStop = true;
		if(mHandlerPool != null) {
			mHandlerPool.cleanup();
			mHandlerPool = null;
		}
	}
	
	private Handler mHandler = new Handler(Looper.getMainLooper()) {
		public void handleMessage(Message msg) {
			if (mStop) {
				return;
			}
			
			if(MessageManager.this.mStop) {
				return;
			}
			
			switch (msg.what) {
				case HANDLE_MSG:
				case HANDLE_MSG_BROADCAST: {
					MessageManager.handleBean((MessageBean) msg.obj);
					break;
				}
				
				default:
					break;
			}
		};
	};
}

/**
 * 消息数据结构
 * 
 * @author yuankai
 * @version 1.0
 */
class MessageBean
{
	Object mWho; 						// 发送者
	List<IMessageHandler> mHandlers;	// 处理者
	IMessageFilter mFilter;
		
	/**
	 * 消息ID
	 * {@link IDiyMsgIds}
	 */
	int mMsgId;
	
	/**
	 * 传递的辅助int型参数
	 */
	int mParam;
	
	/**
	 * 传递的对象列表
	 */
	Varargs mObjects;
	
	/**
	 * 消息数据结构构造方法
	 * 
	 * @param handlerId 处理者ID
	 * @param msgId 消息ID
	 * @param param 传递的辅助int型参数
	 * @param object 传递的对象
	 * @param objects 传递的对象列表
	 */
	public MessageBean(Object who, List<IMessageHandler> handlers, int msgId, int param, Object ...object) {
		mHandlers = handlers;
		mMsgId = msgId;
		mParam = param;
		mObjects = new Varargs(object);
	}
	
	public Object getObject(int index) {
		if (null == mObjects)
			return null;
		
		return mObjects.getObject(index);
	}
}

/**
 * 消息队列
 * 
 * @author yuankai
 * @version 1.0
 */
class MessageQueue extends Queue {
	final boolean add(MessageBean bean) {
		return super.add(bean);
	}
	
	public final MessageBean pop() {
		return (MessageBean)super.pop();
	}
	
	final boolean set(int position, MessageBean bean) {
		return super.set(position, bean);
	}
}

