package com.go.launcher.util;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import android.annotation.SuppressLint;
import android.annotation.TargetApi;
import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.os.Build;
import android.os.Process;
import android.text.TextUtils;

import com.go.launcher.taskmanager.ProcessFilenameFilter;
import com.go.launcher.taskmanager.ProcessHelperBean;

/**
 * 帮助5.1.1以上系统找到正在运行的应用列表
 * @author zhanghuijun
 *
 */
public class ProcessHelperUtil {

	/**
	 * proc路径
	 */
	public static final String PATH_PROC = "/proc/";
	/**
	 * 进程文件名过滤器
	 */
	private static ProcessFilenameFilter sProcessFilenameFilter = new ProcessFilenameFilter(); 
	/**
	 * 需要过滤的UID进程
	 */
	private static List<Integer> sProcessUidFilter = new ArrayList<Integer>() {
		{
			add(0);		// ROOT_UID
			add(Process.SYSTEM_UID);
			add(Process.PHONE_UID);
			add(2000);	// SHELL_UID 
			add(1007);	// LOG_UID
			add(1010);	// WIFI_UID
			add(1013);	// MEDIA_UID
			add(1019);	// DRM_UID
			add(1016);	// VPN_UID
			add(1027);	// NFC_UID
			add(1002);	// BLUETOOTH_UID
		}
	};
	
	/**
	 * 获取/proc/路径下的进程文件夹
	 * @param paramFilenameFilter
	 * @return
	 */
	public static List<String> getProcFilenameList() {
		File localFile = new File(PATH_PROC);
		List<String> filenameList = null;
		if (!localFile.isDirectory()) {
			filenameList = new ArrayList<String>();
		} else {
			String[] fileString = localFile.list(sProcessFilenameFilter);
			if (fileString != null) {
//				for (int j = 0; j < fileString.length; j++) {
//					filenameList.add(PATH_PROC + fileString[j] + "/");
//				}
				filenameList = Arrays.asList(fileString);
			} else {
				filenameList = new ArrayList<String>();
			}
		}
		return filenameList;
	}
	
	/**
	 * 获取进程Cmdline文件内容
	 * @return
	 */
	public static String getProcCmdline(String filePath) {
		String content = FileUtils.readFileToString(filePath + "cmdline");
		if (!TextUtils.isEmpty(content)) {
			return content.trim().split("\00")[0];
		} 
		return "";
	}
	
	/**
	 * 获取进程oom_adj文件内容
	 * @param paramString
	 * @return
	 */
	public static int getProcOomadj(String filePath) {
		String content = FileUtils.readFileToString(filePath + "oom_adj");
		if (!TextUtils.isEmpty(content)) {
			try {
				int i = Integer.parseInt(content);
				return i;
			} catch (NumberFormatException e) {
				e.printStackTrace();
				return -1;
			}
		} 
		return -1;
	}
	
	/**
	 * 获取进程PPID
	 * @param paramString
	 * @return
	 */
	public static int getProcPPid(String filePath) {
		String content = FileUtils.readFileToString(filePath + "status");
		if (!TextUtils.isEmpty(content)) {
			String[] contentArr = content.split(System.getProperty("line.separator"));
			String tempStr = "";
			String[] tempStrArr = null;
			for (int i = 0; i < contentArr.length; i++) {
				tempStr = contentArr[i];
				if (!tempStr.contains("PPid:")) {
					continue;
				}
				tempStrArr = tempStr.split("\\s+");
				if (tempStrArr.length != 2) {
					continue;
				}
				try {
					return Integer.parseInt(tempStrArr[1]);
				} catch (NumberFormatException e) {
					e.printStackTrace();
					return -1;
				}
			}
		}
		return -1;
	}
	
	/**
	 * 获取进程UID
	 * @param paramString
	 * @return
	 */
	public static int getProcUid(String filePath) {
		String content = FileUtils.readFileToString(filePath + "status");
		if (!TextUtils.isEmpty(content)) {
			String[] contentArr = content.split(System.getProperty("line.separator"));
			String tempStr = "";
			String[] tempStrArr = null;
			for (int i = 0; i < contentArr.length; i++) {
				tempStr = contentArr[i];
				if (!tempStr.contains("Uid:")) {
					continue;
				}
				tempStrArr = tempStr.split("\\s+");
				if (tempStrArr.length < 2) {
					continue;
				}
				try {
					return Integer.parseInt(tempStrArr[1]);
				} catch (NumberFormatException e) {
					e.printStackTrace();
					return -1;
				}
			}
		}
		return -1;
	}
	
	/**
	 * 获取运行状态
	 * @param paramString
	 * @return
	 */
	public static String getProcRunningState(String filePath) {
		String content = FileUtils.readFileToString(filePath + "status");
		if (!TextUtils.isEmpty(content)) {
			String[] contentArr = content.split(System.getProperty("line.separator"));
			String tempStr = "";
			String[] tempStrArr = null;
			for (int i = 0; i < contentArr.length; i++) {
				tempStr = contentArr[i];
				if (!tempStr.contains("State:")) {
					continue;
				}
				tempStrArr = tempStr.split("\\s+");
				if (tempStrArr.length != 3) {
					continue;
				}
				return tempStrArr[2];
			}
		}
		return "";
	}
	
	/**
	 * 获取/proc/路径下的进程信息（包含部分系统进程）
	 * 注意：此方法获取的是/proc/下面的进程信息，其中有可能包含非正在运行中的应用，所以可以根据下面两个点进行过滤：
	 * 1、使用AppUtils.isAppStop()方法过滤已经停止运行的应用；
	 * 2、根据getProcRunningState获取进程运行状态，如果运行状态是D，也就是dead，即可过滤
	 * 本方法没有进行以上两点过滤，有需要可自行过滤
	 * @return
	 */
	public static List<ProcessHelperBean> getProcBeans(Context context) {
		List<String> filenameList = getProcFilenameList();
		List<ProcessHelperBean> procBeans = new ArrayList<ProcessHelperBean>();
		String filepath = "";
		for (int i = 0; i < filenameList.size(); i++) {
			try {
				filepath = PATH_PROC + filenameList.get(i) +  "/";
				int pid = Integer.parseInt(filenameList.get(i));
				ProcessHelperBean bean = new ProcessHelperBean();
				bean.setPid(pid);
				bean.setProcessName(getProcCmdline(filepath));
				bean.setPPid(getProcPPid(filepath));
				bean.setUid(getProcUid(filepath));
				bean.setPkgLists(context.getPackageManager().getPackagesForUid(ProcessHelperUtil.getProcUid(filepath)));
				procBeans.add(bean);
			} catch (NumberFormatException e) {
				e.printStackTrace();
				continue;
			} catch (Exception e) {
				e.printStackTrace();
				continue;
			}
		}
		return procBeans;
	}
	
	/**
	 * 获取正在运行的应用（包含部分系统应用）
	 * 注意：此方法获取的是/proc/下面的进程信息，其中有可能包含非正在运行中的应用，所以可以根据下面两个点进行过滤：
	 * 1、使用AppUtils.isAppStop()方法过滤已经停止运行的应用；
	 * 2、根据getProcRunningState获取进程运行状态，如果运行状态是D，也就是dead，即可过滤
	 * 本方法没有进行以上两点过滤，有需要可自行过滤
	 * 
	 * @return 返回一个以包名为key，该包名应用正在运行的进程id组(pid组)为value的Map
	 */
	@SuppressLint("DefaultLocale")
	public static Map<String, List<Integer>> getRunningAppProcesses(Context context) {
		List<ProcessHelperBean> procBeans = getProcBeans(context);
		Map<String, List<Integer>> runningAppProcessesMap = new HashMap<String, List<Integer>>();
		for (int i = 0; i < procBeans.size(); i++) {
			ProcessHelperBean bean = procBeans.get(i);
			if (bean != null) {
				if (sProcessUidFilter.contains(bean.getUid())) {	// 过滤一些系统UID的应用
					continue;
				}
				String[] pkgList = bean.getPkgLists();
				if (pkgList == null) {
					continue;
				}
				for (int j = 0; j < pkgList.length; j++) {
					List<Integer> mPids = null;
					if (!bean.getProcessName().toLowerCase().contains(pkgList[j].toLowerCase())) {
						// 过滤一些很有可能没用的包名
						continue;
					}
					if (runningAppProcessesMap.containsKey(pkgList[j])) {
						mPids = runningAppProcessesMap.get(pkgList[j]);
					} else {
						mPids = new ArrayList<Integer>();
					}
					mPids.add(bean.getPid());
					runningAppProcessesMap.put(pkgList[j], mPids);
				}
			}
		}
		return runningAppProcessesMap;
	}
	
	/**
     * 判断一个应用是否已经停止运行.<br>
     * 
     * @param context
     * @param packageName
     * @return
     */
    @TargetApi(Build.VERSION_CODES.HONEYCOMB_MR1)
    public static boolean isAppStop(Context context, String packageName) {
        boolean isStop = false;
        try {
            ApplicationInfo app = context.getPackageManager()
                    .getApplicationInfo(packageName, 0);
            isStop = (app.flags & ApplicationInfo.FLAG_STOPPED) != 0;
        } catch (Exception e) {
            isStop = true; // 通常是程序不存在了吧...
            e.printStackTrace();
        }
        return isStop;
    }
}
